/* Compound.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _COMPOUND_H
#define _COMPOUND_H

#include <list>
#include "my_library.h"
//#include "Reaction.h"

using namespace std;
class Reaction;

/**
 * Contains all information on a compound. A compound is identified by an id. KEGG ids are like: C00001 and EcoCyc ids are like... For each compound, you have the reactions in which it is involved. You also have a chemical description of the compound enabling compound comparison.
 */
class Compound
{
 private:
  char *id;
  char *name;
  list<Reaction *> *left_reactions;   
  list<Reaction *> *right_reactions;
  list<Reaction *> *left_right_reactions;
  list<Reaction *> *no_status_reactions;
  list<Reaction *> *reactions;
  int nb_edges;   //number of edges the compound is responsible for
  bool **visited_edges;  //matrix where true means the edge between the two corresponding reactions has been visited
  //we could use  a half matrix

  bool *frags; // array of chemical groups
  int *int_frags;
 public:
  Compound(char *, char*);
  ~Compound();
  void reset(); //different from the constructor, it reinitializes the variables that need to when the program is lauched several times

  /*communication with other classes*/
  list<Reaction *> *get_left_reactions();
  list<Reaction *> *get_right_reactions();
  list<Reaction *> *get_no_status_reactions();
  list<Reaction *> *get_reactions();
  bool * get_frags();
  int *get_int_frags();
  char *get_id();
  char *get_name();
  int get_nb_edges();

  /*data reading*/
  int add_left_reaction(Reaction *);
  int add_right_reaction(Reaction *);
  int add_left_right_reaction(Reaction *);
  int add_no_status_reaction(Reaction *);
  void concatenate_reactions();
  void set_frags(int *);
  void set_edges();

  /*compound comparison*/
  bool is_similar_to(Compound *);
  bool is_similar_to(Compound *,int threshold);

  /*edge functions*/
  bool is_visited(Reaction *, Reaction *);
  void set_visited(Reaction *, Reaction *);
  void set_unvisited(Reaction *, Reaction *);
  int arity();
  int get_nb_reactions();

  /*print functions*/
  void draw(ostream &); //prints with ""
  void print(ostream &);
  void print_id(ostream &);
  void print();
  void print_nb_reactions();
  void print_visited_edges();
};


/**
 *used for defining an order in Compounds useful to sort lists
 *the compounds are ordered by frequency (i.e. number of reactions in which they are involved)
 */
struct Compound_Compar //: greater<Compound *>
{  
  bool operator()(Compound* left,Compound* right) 
  {  
    return (left->get_reactions()->size() > right->get_reactions()->size());
  }  
};


#endif
