/* Data.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _DATA_H
#define _DATA_H

#include <map>
#include <list>
#include "Reaction.h"
#include "Compound.h"
#include "Enzyme.h"
#include "Pathway.h"
#include "Organism.h"


/**
 * References all the data used in the program, this class is a bit like a database. Basically, you have data about reactions, enzymes, compounds. Additionnally, you can have data about pathways and organisms. This is used for later analysis but not in the algorithm.
 */
class Data
{ 
  
  
  /* here we choose to give a public access to the attributes in order not to make it to heavy*/
 public:
  
  /*Data as lists*/
  list<Reaction *> *reactions;  
  list<Compound *> *compounds;
  list<Enzyme *> *enzymes;
  list<Pathway *> *pathways;
  list<Organism *> *organisms;

  /*Data as arrays*/
  Reaction **tab_reactions;
  int nb_reactions;
  Compound **tab_compounds;
  int nb_compounds;
  /* Enzyme **tab_enzymes; */
/*   int nb_enzymes; */

  /*Array of all possible EC numbers (useful to get the correspondance between EC numbers and colours)*/
  int **colours_EC;
  char **colours_str;
  int nb_colours;
  /**
   *tab giving the correspondance for colours cut at given threshold. correspondance_colours[i][j] gives the colour i cut at threshold j-1. If you need to see the EC number then you should also use colours_EC. Ex: colours_EC[correspondance_colours[i][j]]
   */
  int **correspondance_colours;


  /*Constructor + destructor*/
  Data(char *reaction_compound_enzyme_file); //general case where all data is in the same file (Ecocyc data)
  Data(char *reaction_compound_file,char *reaction_enzyme_file); //parameters are input data files
  Data(char *reaction_compound_file,char *reaction_enzyme_file, char *frag_decomposition); //parameters are input data files, this constructor is called when input and output compounds are specified
  ~Data();
  int ** fill_colours(); //fills the EC_numbers arrays which gives the correspondance for int, string and int* (3 ways of representing the colours
  void assign_colours_to_reactions(); //gives colours (integers) to the reactions of the dataset. If a reaction has EC_number 1.1.1.1, it will get 4 colours (the one for 1, the one for 1.1, the one for 1.1.1 and the one for 1.1.1.1)

  /*Index functions : you give the name, it returns the memory adress (we should use a map object for this)*/
  Compound *compound_adress_book(char *cpd_id);
  Reaction *reaction_adress_book(char *rea_id);
  Enzyme *enzyme_adress_book(char *ec);
  Pathway *pathway_adress_book(char *pat_id);
  Organism *organism_adress_book(char *org_id);
  
  /*parsing*/
  int reaction_compound_enzyme_gene_parser(char *);
  int reaction_compound_enzyme_parser(char *);
  int reaction_compound_parser(char*);
  int enzyme_reaction_parser(char*);
  int frag_decomposition_parser(char *);
  int enzyme_organism_parser(char *);
  
  /*preprocessing*/
  int remove_frequent_compounds(int);
  
  /*stat functions : useful to know global properties of your data*/
  void print_identity_card();
  void print_identity_card(std::ostream&);
  void print_enzymes(int, char *);
  void print_enzyme_arity(int, char *);
  void print_compound_arity(char *);
  void print_reaction_colours();
  void print_nb_compounds_per_reaction();
  void print_nb_reactions_per_compound();
  void print_possible_ecs(std::ostream&); //prints all possible ECs you can use to build your motif. The list is sorted.
  
  /*output function : useful to communicate data to other people*/
  void print_pathway_reaction(std::ostream&);
  void print_reaction_pathway(std::ostream&);
  void print_reaction_enzyme(std::ostream&);
  void print_enzyme_reaction(std::ostream&);
  void print_reaction_labels(int, std::ostream&); //useful for testing distribution of motifs for Sophie
  
  /*reset functions : useful when you launch the program several times (then you need to reset some variables like distance and visited_edges) - this is needed in the graph building procedure*/
  void reset_compounds();
  void reset_reactions();
  
  /**
   *initialization function used for systematic analysis of all ECs cut at given threshold. It fills a tab with all possible EC numbers of the dataset cut at given threshold.
   */
  int fill_tab_enzymes(int threshold,char **tab);
  int fill_tab_enzymes(int threshold,int **tab); 
  
  
  
  /**
   *initialisation function used to optimize systematic analysis. The tab will give the number of children a partial EC number has.
   */
  void fill_nb_children(int threshold, int n, int *tab);
};

#endif
