/* Enzyme.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _ENZYME_H
#define _ENZYME_H

#include "my_library.h"
#include <list>

class Reaction;
class Organism;

using namespace std;

/**
 * Contains all information on an enzyme. An enzyme is identified by an EC number. Note that there is an ambiguity here since EC numbers can span several molecules. In the future, there might be another intermediary object called EC number. For each enzyme, you have the reaction it catalyzes and the organisms that are supposed to have this catalytic activity (in the case of KEGG data).
 */
class Enzyme
{
 private:
  char *ec;
  int EC_number[4];
  list <Reaction *> *catalysed_reactions;
  list <Organism *> *organisms;
 public:
  Enzyme(char *ec_str);
  ~Enzyme();

  /*outside*/
  char *get_ec();
  void get_partial_ec(int, char *); 
  int get_ec_size();
  int *get_EC_number();
  list <Reaction *> *get_catalysed_reactions();   //just because the attribute is private, we need this function

  /*data reading*/
  int add_reaction(Reaction *);
  void add_organism(Organism *);

  /*enzyme comparison*/
  /**
   *Comparison of two ec numbers. If the EC numbers are identical down to a given threshold, then this function returns 1. Incomplete EC numbers are filled with jokers (-1). Jokers match no other numbers.
   */
  bool is_similar_to(Enzyme *,int);
  /**
   *Comparison of two ec numbers. If the EC numbers are identical down to a given threshold, then this function returns 1. The threshold is given implicitely by the incompleteness of the query enzyme. Incomplete EC numbers are filled with jokers (-1). Jokers match no other numbers.
   */
  bool is_similar_to(Enzyme *);  //the threshold is implicitely given by the incompleteness of teh ec number in the motif
  int arity();

  /*print functions*/
  bool print(int, FILE *);
  void print();
  void print(std::ostream&);
  void print(int,std::ostream&);
  /* void print_enzyme_reactions(ostream &); //prints one line for each couple enzyme reaction */
};

/**
 *used for defining an order in Enzymes useful to sort lists
 */
struct Enzyme_Compar //: greater<Enzyme *>
{  
  bool operator()(Enzyme* left,Enzyme* right) 
  {  
    int i=0;
    while (left->get_EC_number()[i]==right->get_EC_number()[i])
      i++;
    return (left->get_EC_number()[i] < right->get_EC_number()[i]);
  }  
};

#endif
