/* Io.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _IO_H
#define _IO_H

#include "my_library.h"
#include <list>
using namespace std;
/* #include "Metabolic_Motif.h" */
/* #include "Solution_Group.h" */
class Metabolic_Motif;
class Solution_Group;
class Data;
class Graph;
class Reaction;
class Reaction_Graph;
class Bipartite_Graph;

/**
 * Class describing Input/Output. This class deals with the user, it reads the parameter values and prints the output files.
 */
class Io
{
 private:
  /**
   *threshold used for compound comparison.
   */
  int sigma_C;
  /**
   *number of reactions badly colored allowed in a solution between properly colored reactions.
   */
  int local_gap;
  /**
   *number of reactions badly colored allowed in a solution.
   */
  int global_gap;
  /**
   *percentage of colors from the motif that really have to be in the occurrence.
   */
  int quorum;
  /**
   *mode in which you wish to launch the program depending at which step you want to stop. 0:filter 1:solutions 2:grouped solutions
   */
  int mode;
  /**
   *quantity of information you need to print for your solutions. 
   */
  int verbose;
  /**
   *set to 1 if you want to consider topology to group your motifs
   */
  bool append;
/**
   *set to 1 if you want your results to be appended to the file of results
   */

  int topology;

  /**
   *set to 1 if you want to consider the order of colours to group your motifs
   */
  int order;
  

  /**
   *threshold used for EC number comparison (and thereby reaction comparison). This option is not necessary in a search query (the threshold comparison can be implicitely given by the incompleteness of the EC numbers of the query). On the other hand, it has so be defined for an inference query on metabolic graph.
   */
  int sigma_E;
  /**
   *size of the motifs to enumerate in inference mode.
   */
  int motif_size;
  /**
   *very important. Gives the list of EC numbers you want to look for.
   */
  char *enzymes;
  /**
   *gives the input compound of your motif.
   */
  char *input_compound;
  /**
   *gives the output compound of your motif.
   */
  char *output_compound;

  /**
   *file containing reactions, EC and compounds, necessary for building the network and analysing well in detail
   */
  char *data_file;

  /**
   *file containing a list of motifs to search for. (as a first step, this is used only for refinement and is associated to the option --motifs)
   */
  char *motifs_file;

  /**
   *file containing a list of enzymes. Inference is then performed from this list of enzymes instead of the whole list in Data.
   */
  char *enzymes_file;

  /**
   *path to a place where one can write things. This is especially useful for the web version since the server has a special directory where we have the right to write.
   */
  char *write_path;


  /**
   *number of compounds considered as ubiquitous in the data and that you wish to remove from your dataset
   */
  int nb_frequent_compounds;

  /**
   *file giving the topology of the network as an adjacency matrix
   */
  char *graph_file;
  /**
   *file giving the labels of the nodes of the network (you can have several labels per node)
   */
  char *labels_file;

  /**
   *file in which the results of your search are saved
   */
  char *results_file;
  
  /**
   *boolean telling you if you want to generate data for fabien (visualisation tasks)
   */
  int fabien_jourdan;

  /**
   *boolean telling you if you want to generate data for stephane (statistical analysis)
   */
  int stephane_robin;

  /**
   *boolean telling you if you want to generate data for alberto (combinatorial analysis)
   */
  int alberto_marchetti;

  /**
   *boolean telling you if you want to generate data for etienne (reaction and compound degree in bipartite graph)
   */
  int etienne_birmele;

  /**
   *boolean telling you if you want to generate data in mfinder format
   */
  int mfinder;

  /**
   *boolean telling you if you want to generate data in sif format
   */
  int sif;

  /**
   *boolean telling you if you want to print the identity_card of the data
   */
  int stat;


  /**
   *boolean telling you if you want to print the list of EC numbers (partial and complete) of the data. Note that all partial ECs of complete ECs will be printed even if they are not contained in the data.
   */
  int ec_list;

  /**
   *number of random graphs to generate in order to assess the over-representation of a motif
   */
  int nb_simulations;

  /**
   *Boolean telling you if you want to print only p_values to the
   *file. Useful to perform some tests on which is the good number of
   *simulations to perform. Vicente Acuna advised me on this.
   */
  bool p_only;

  /**
   *Global variable to indicate the permutability of the network 
   */
  int *perm;


  /**
   *Boolean indicating whether you want to study the permutability of your metabolic network 
   */
  bool permutability;


/**
   *Boolean indicating whether you want to compute p-values or not
   */
  bool no_p_value;











 public:
/**
 * Prints help on parameters in standard output. 
 */
  void printHelp();

/**
 * Prints simplified version of help screen.
 */
  void printHelpShort();


/**
 * Prints parameters set by user in standard output. 
 */
  void printSettings();
  /**
   * Read parameters given by user as a command line. 
   */
  void parse(int, char**);

  /**
   * Default constructor
   */
  Io();
  /**
   * Main constructor
   */
  Io(int, char**);
  /**
   * Destructor
   */
  ~Io();

  /**
   * Generate 1000 random graphs and store them in an array so that
   * they are available for further analysis.
   */
  void shuffle_graphs(Reaction_Graph *,Reaction_Graph **, int);
  
  /**
   * Generate 1000 random graphs and store them in an array so that
   * they are available for further analysis.
   */
  void shuffle_graphs(Graph *,Graph **, int);
  


  /**
   *Search motif in 1000 random graphs and compare the number of
   *occurrences to the observed number of occurrences. The p-value is
   *the number of times the real number is smaller than the simulated
   *ones over the number of simulations.
   */
  float calculate_p_value(Reaction_Graph **,int *,int, int);

  /**
   *Same as previous function but for general graphs
   */
  float calculate_p_value(Graph **,int *,int, int);

  

  /**
   *Search motif in 1000 random graphs and compare the number of
   *occurrences to the observed number of occurrences. The job is also
   *done after grouping the solutions by connected components. The
   *p-value is the number of times the real number of connected
   *components is smaller than the simulated ones over the number of
   *simulations.
   */
  float calculate_c_value(Reaction_Graph **,Metabolic_Motif *, int, int, Reaction **);

  /**
   *Same as previous function but for general graphs ant outputs both p and c value
   */
  int calculate_p_c_value(Graph **,int *,int, int, int, float *,float *);


  /**
   *Search motif in 1000 random graphs, group the occurrences by
   *topology, and compare the results to the real graph.
   */
  int calculate_p_values(Reaction_Graph **simulations, Metabolic_Motif * myMotif, int nb_observed_topologies, int *observed_topologies_counts, long int *observed_topologies1, long int *observed_topologies2, float *p_s, Reaction **tab_reactions, Bipartite_Graph *myBipartiteGraph);


  /**
   *Search motif in random graphs and print to file the results:
   *number of occurrences, number of connected components and size of
   *the connected components.
   */
  void fprint_stats_for_count(Reaction_Graph **,Metabolic_Motif *, Reaction **);



  /**
   *Print p-value to output. If the proportion of graphs in which the
   *number of occurrences is greater than the number of solutions is
   *0, then the p-value is said to be lower than the minimum
   *measurable (1/ns). Additionnally, the p-value is rounded 4 numbers
   *after the *comma (this should be enough).
   */
  void print_p_value(float, std::ostream&,int);

  /**
   *Sorts results_file by ascending p-values
   */
  void sort_by_p_value();

  /**
   *Sorts the output file by block according to ascending p-values. A
   *block starts with # and is composed of a motif (set of EC numbers)
   *followed by a line of properties, followed by a list of
   *occurrences. This format is used for Fabien Jourdan's vizualizer.
   */
  void sort_by_block();
  
  /**
   * Print to file information on solution groups. 
   */
  //void fprint_useful_info(Metabolic_Motif *, list <Solution_Group *> *, list <Solution_Group *> *); 
  
  
  /**
   *Print to file the EC numbers and the number of occurrences
   */
  //void fprint_nb_occurrences(Metabolic_Motif *, list <Solution_Group *> *);

  /**
   *Print to file the header of the results file explaining what the
   *file contains. Every line starts with a !
   */
  int fprint_header(std::ostream&);

  /**
   *Print to file the EC numbers, the p-value, and the solutions as
   *Reaction lists. Several levels of details can be printed depending
   *on the mode and verbose option. Mode controls how far you go in
   *the calculation, verbose controls how far you go in the printing.
   */
  void fprint_solutions(char *, Solution_Group *, float, float, std::ostream&);

  /**
   *Print to file topological motifs and their p-value 
  */
  void fprint_solutions(char *, list<Solution_Group *> *, float *, std::ostream&);


  /**
   *Print to file the EC numbers and the number of occurrences
   */
  void fprint_nb_solutions(char *, int, std::ostream&);

  /**
   *Print to file the motif (set of integers) and the number of occurrences
   */
  void fprint_nb_solutions(int *,int, int, std::ostream&);


  /**
   *Generic function to search for a motif specified as a set of
   *enzymes and store solutions in a group of solutions. The p-value
   *indicating if the motif is over-represented is also returned as
   *well as the p-value telling if the number of connected components
   *is surprising. Tambien the p_values indicating if the subgroups of
   *occurrences (corresponding to topological motifs) are
   *over-represnted may also be returned.
   */
  float *motif_search(Data *myData, char *enzymes, Solution_Group *all_solutions, list<Solution_Group *> *topology_solutions, Reaction_Graph **, Reaction_Graph *whole_reaction_graph, float *p, float *c, float *p_s, int *nb_observed_topologies);

  /**
   *Generic function to search for a motif specified as a set of
   *enzymes and store solutions in a group of solutions
   */
  int motif_search(Data *myData, Graph *graph, char *enzymes);


  /**
   *Given a list of motif, search for these motifs and group them by
   *toplogy to see if they still have several connected components.
   */
  void refine_motifs(Data *myData, Reaction_Graph *whole_reaction_graph, Reaction_Graph **simulations);


  /**
   * Performs motif search on the network for all motifs of fixed
   * size. All motifs of fixed size are searched for using a special
   * trick (first search for motifs with smaller threshold)
   */
  void motif_inference(Data *myData, Reaction_Graph *whole_reaction_graph, int threshold, int motif_size, Reaction_Graph **simulations);

  /**
   * Performs motif search on the network for all motifs of fixed
   * size. All motifs of fixed size are searched for using a special
   * trick (first search for motifs with smaller threshold). This
   * function is specific to random graphs.
   */
  void motif_inference(Data *myData, Graph *graph, int threshold, int motif_size);


  /**
   * Performs motif search on the network for all motifs of fixed
   * size. All motifs of fixed size are searched. No trick is used.
   */
  void motif_inference(Graph *graph, int motif_size, Graph **randomized_graphs);

  /**
   * Performs motif search on the network for all motifs of fixed
   * size. All motifs of fixed size are searched for using a special
   * trick (first search for motifs with smaller threshold)
   */
  void motif_inference_from_enzymes(Data *myData, Reaction_Graph *whole_reaction_graph, int motif_size, Reaction_Graph **simulations);

};



#endif
