/* Reaction.h
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _REACTION_H
#define _REACTION_H

#include <list>
#include "Enzyme.h"
#include "Pathway.h"
#include "Compound.h"
#include "my_library.h"
#include <iostream>
#include <cstring>

using namespace std;

class Reaction_Graph;
class Data;

/**
 * Contains all information on a reaction. A reaction is identified by an id. KEGG ids are like: R00001 and EcoCyc ids are like RXN... For each reaction, you have the enzymes that catalyse it, the substrates, the compounds and the pathways in which it is involved.
 */
class Reaction
{
 private:
  char *id;  //unique identificator
  //char *name;
  list<Enzyme *>  *enzymes; //one reaction can be catalysed by several enzymes
  list<Compound *> *left_compounds;
  list<Compound *> *right_compounds;
  list<Compound *> *no_status_compounds;
  list<Compound *> *left_right_compounds;
  list<Compound *> *compounds;
  list<Pathway *> *pathways;
  int distance;  //number of edges in the reaction network separating from a relevant reaction (i. e. a reaction catalysed by an enzyme from the motif) -- this is useless now that we changed the way of building the graph
  //distance = 0 means the reaction is relevant
  int first_colour;  //first enzyme from the motif that matches an enzyme that catalyses the reaction (the first in the enzyme list) useful to draw
  list<int> *colours;

 public:
  Reaction(char *id);
  ~Reaction();
  int operator<(const Reaction *rhs) const;
  void reset();  //different from the constructor, it reinitializes the variables when the program is lauched several times

  list<Compound *> * get_right_compounds();
  list<Compound *> * get_left_compounds();
  list<Compound *> * get_no_status_compounds();
  list<Compound *> * get_compounds();
  list<Enzyme *>  *get_enzymes();
  list<Pathway *>  *get_pathways();

  int add_left_compound(Compound *);
  int add_right_compound(Compound *);
  int add_left_right_compound(Compound *);
  int add_no_status_compound(Compound *);
  int which_side(Compound *); //indicates on which side the compound is, returns 1 if LEFT, 2 if RIGHT and returns 3 if we don't know (yet?)
  void concatenate_compounds();
  int add_enzyme(Enzyme *);
  int add_pathway(Pathway *);

  int remove_compound(Compound *);

  char *get_id();
  int get_distance();
  void set_distance(int);

  int get_first_colour();
  list<int> *get_colours();
  void set_first_colour(int);
  void add_colour(int);
  void add_colours(list<int> *);
  void clear_colours(); //reinitializes the colour list to an empty list in case you want to launch the program several times
  int arity();

  /*does the reaction have a compound that matches those from the motif ?*/
  void contains_similar_compounds(Compound *, Compound *, bool *matched_compounds, int threshold);

  void draw(ostream &);  //prints to file but with quotes so that the program neato does not get confused.
  void print_colours();
  void print(ostream &);
  void print_enzymes(ostream &);
  void print_pathways(ostream &);
  void print_reaction_pathways(ostream &); //prints one line for each couple reaction pathway
  void print_reaction_enzymes(ostream &); //prints one line for each couple reaction pathway
  void print();
  void print(ostream &, int verbose);
  void print(int verbose);
  void print_compounds();
  void print_nb_compounds();


  // extra attribute for optimization
  Reaction_Graph *connected_component; // for optimization : we don't need to run the whole list of reaction graphs (i.e. connex components) if each reaction knows which comonent it belongs to
};


/**
 *used for defining an order between reactions useful to sort lists
 */
struct Reaction_Compar //: greater<Reaction *>
{
  bool operator()(Reaction* left,Reaction* right)
  {
    //cout<<"je suis dans le tri, je compare "<<left->get_id()<<" et "<<right->get_id()<<endl;
    //cout<<"le rsultat est "<<strcmp(left->get_id(),right->get_id())<<endl;
    return !strcmp(left->get_id(),right->get_id());
    /* int i=0; */
/*     while (left->get_EC_number()[i]==right->get_EC_number()[i]) */
/*       i++; */
/*     return (left->get_EC_number()[i] < right->get_EC_number()[i]); */
  }
};

#endif
