/* Solution.h
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _SOLUTION_H
#define _SOLUTION_H

#include "Reaction.h"
#include "Bipartite_Graph.h"
#include <list>

/**
 * Contains all information on a solution of the search problem
 * (occurrence of the motif. A solution is a list of reactions, it
 * could be an array of reactions actually. You can calculate the
 * number of pathways it spans. Intra-pathway solutions
 * (i.e. solutions that span a single pathway) are easier to interpret
 * biologically. Inter-pathway solutions might meaningless, they might
 * also represent alternative routes.
 */
class Solution
{
 private:
  list<Reaction *> *reactions;  //it could be an array instead of a list

  int *rea_tab; //integers stand for reactions. The integers used HAVE
		//to correspond to indices of reactions in the
		//tab_reaction of DATA so that we can link that to the
		//bipartite graph later on.
  int size; //number of reactions in the solution

  Bipartite_Graph *bipartite_graph;
  long int topo1;
  long int topo2;


  list<Pathway *> *max_pathways;
  list<Pathway *> *min_pathways;

  /*some info about inner compounds*/
  list<list <Compound *> *> *inner_compounds;  //added to print the compounds too, easy to interpret when the solution is linear in the bipartite graph
  list<Compound *> ***compound_matrix;//graph giving the topology of the solution + compounds on the edges

  /*some info about outer compounds*/
  Reaction *first_reaction; //not really used for the moment
  Reaction *last_reaction;


 public:
  Solution(list<Reaction *> *);
  Solution(int *,int,Reaction **,int *);
  ~Solution();

  /*outside*/
  list<Reaction *> *get_reactions();
  int get_size();
  long int get_topo1();
  long int get_topo2();
  long int get_topo(bool order);


  /*input/output compounds treatment*/
  bool contains_similar_compounds(Compound *cpd1, Compound *cpd2, int threshold);

  /*group the linear solutions*/
  bool same_order(Solution *);

  /*group the solutions according to their topology in the bipartite graph*/
  bool same_topology(Solution *);

  /*more info on pathways*/
  void calculate_min_pathways();
  void calculate_max_pathways();
  bool is_intra_pathway();
  int nb_max_pathways(); //returns the number of pathways spanned by the solution

  /*more info on compounds*/
  float cpd_arity();
  int nb_inner_compounds();
  void fill_inner_compounds();
  void fill_compound_matrix();
  bool is_linear();

  /*graph structure (topology) of the occurrence*/
  void set_bipartite_graph(Bipartite_Graph *,int *colors_to_filter);
  void compute_canonical_labelling(bool order);

  /*print functions*/
  void print();
  void print(std::ostream&);
  void print(std::ostream&,int verbose);
  void print(int verbose);
  void print_bipartite_graph();
  void print_colour_order();
  void print_more();
  void print_enzymes_pathways();

};



#endif
