/* Solution_Group.h
 *
 * Copyright (C) 2007 Laboratoire de BiomÁtrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _SOLUTION_GROUP_H
#define _SOLUTION_GROUP_H


#include "Solution.h"
#include "Metabolic_Motif.h"

//using namespace std;

/**
 * This class corresponds to a class of solutions. Classes can be defined according to several criteria. For instance, all solutions that share the same input compound can be gathered in the same group. Or all solutions that share a reaction with each other can be gathered in the same group.
 */


class Solution_Group
{
 private:
  list<Solution *> *solutions;
 public:
  Solution_Group();
  Solution_Group(Solution *);
  ~Solution_Group(); //only deletes the group
  void delete_solutions(); //deletes the solutions inside the group
  void clear_solutions();  //light version of the destructor (useless ?)
  list<Solution *> * get_solutions();
  void add(Solution *);
  void add_group(Solution_Group *);
  void copy_group(Solution_Group *);
  int size();

  long int get_topo(int order); //gives the topology of the first solution of the group

  /*useful for stats*/
  /**
   *gives the number of solutions that are totally included in a pathway.
   */
  int nb_intra_pathway();
  /**
   *gives the average number of pathways spanned by an occurrence (solution)
   */
  float mean_nb_pathway_per_occurrence();
  /**
   *gives the variance of the number of pathways spanned by an occurrence
   */
  float var_nb_pathway_per_occurrence();
  float cpd_arity();
  int nb_max_sol(Metabolic_Motif *);

  /*precalculation of properties on solutions*/
  void set_bipartite_graph(Bipartite_Graph *,int *colors_to_filter);
  void compute_canonical_labelling(bool order);


  /*organization of solutions*/
 /*  void withdraw_redundance(); */
  void filter_by_compounds(Compound *input, Compound *output, int threshold);
  void withdraw_non_linear_solutions(); /*keep only linear occurrences (linear in the bipartite graph)*/
  void filter_by_nb_compounds(int motif_size);  /*keeps only solutions with a given number of input compounds*/
  void group_by_reaction(list<Solution_Group *> *groups);
  void group_by_order(list<Solution_Group *> *groups);  //groups together solutions with colors in the same order (this is done only with linear solutions)
  void group_by_topology(list<Solution_Group *> *groups);   //groups together solutions with same topology (done only for discriminating linear/non linear solutions)
  void count_observed_topologies(long int *, long int *, int, int *); //modified version taking into account already established topologies  
  bool contains_similar_solution(Solution *);

  /*print functions*/
  void print();
  void print_bipartite_graphs();
  void print_bipartite_graph(); //only print one representative of the group, the first one
  void print_colour_order(); //only print one representative of the group, the first one
  void print(std::ostream&);
  void print_intra_pathway(std::ostream&);
  void print(int verbose);  //print with several levels of detail
  void print(std::ostream&, int verbose);
};



#endif
