/* Bipartite_Graph.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biométrie et Biologie Évolutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Bipartite_Graph.h"
#include <iostream>
#include "../include/my_library.h"
#include <cstdlib>
#include <cmath>

#include <fstream>
//#include <map>
//using namespace std;

/*for Nauty*/
//#define MAXN 100
//#include "../../nauty/nauty.h"


Bipartite_Graph::Bipartite_Graph(int nb_reactions, int nb_compounds)
{
  /*matrix creation + initialisation*/
  adjacency_matrix = new int *[nb_reactions];
  for (int i=0;i<nb_reactions;i++)
    {
      adjacency_matrix[i] = new int[nb_compounds];
      for (int j=0;j<nb_compounds;j++)
	adjacency_matrix[i][j]=0;
    }

  this->nb_compounds=nb_compounds;
  this->nb_reactions=nb_reactions;
  reaction_colors=NULL;
  compounds=NULL;
  reactions=NULL;
}

/*creates a bipartite graph from a set of reactions and a set of compounds*/
Bipartite_Graph::Bipartite_Graph(Reaction **reactions, int nb_reactions, Compound **compounds, int nb_compounds)
{
  this->nb_compounds = nb_compounds;
  this->nb_reactions = nb_reactions;
  
  /*matrix creation + initialisation*/
  adjacency_matrix = new int *[nb_compounds];
  for (int i=0;i<nb_compounds;i++)
    {
      adjacency_matrix[i] = new int[nb_reactions];
      for (int j=0;j<nb_reactions;j++)
	adjacency_matrix[i][j]=0;
    }

  /*fill matrix*/
  int which_side = 0;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  which_side = reactions[j]->which_side(compounds[i]);
	  adjacency_matrix[i][j]=which_side;
	}
    }

  /*old way of filling the labels: they are not copied*/
  /*node labeling*/
  // this->reactions = reactions;
//   this->compounds = compounds;

  /*new way of filling the labels, the Reaction * are copied.*/
  /*fill labels*/
  this->reactions = new Reaction *[nb_reactions];
  this->compounds = new Compound *[nb_compounds];
  for (int j=0;j<nb_reactions;j++)
    this->reactions[j]=reactions[j];
  for (int i=0;i<nb_compounds;i++)
    this->compounds[i]=compounds[i];

  /*fill reaction_colors*/
  reaction_colors = new list<int> *[nb_reactions];
  for (int j=0;j<nb_reactions;j++)
    {
      reaction_colors[j]=new list<int>;
      copy(reactions[j]->get_colours()->begin(),reactions[j]->get_colours()->end(),back_inserter(*reaction_colors[j]));
    }

}



/**
 * Copy+filter constructor. The filtered graph will contain the
 * reactions asked as well as the inner compounds linking those
 * reactions. Additionnally, the colors of the selected reactions will
 * be filtered (meaning that only the ones corresponding to the
 * threshold are kept.
 *
 * @param bipartite_graph_to_filter bipartite graph corresponding to the whole network
 * @param rea_tab array of integers corresponding to reactions to keep (!!integers correspond to indices of reactions in the whole list of reactions)
 * @param nb_reactions number of reactions to keep
 *
 */
Bipartite_Graph::Bipartite_Graph(Bipartite_Graph *bipartite_graph_to_filter, int *rea_tab, int nb_reactions, int *colors_to_filter)
{
  /*first settings for number of reactions and number of compounds
    (the number of compounds will be adjusted afterwards*/
  this->nb_reactions=nb_reactions;
  this->nb_compounds=bipartite_graph_to_filter->get_nb_compounds();

  /*examine lines to see if we can filter some of them (a compound
    that is involved in less than 2 reactions won't be kept, since we
    want to keep only inner compounds)*/
  int nb_edges_in_line=0;
  list<int> *compounds_to_keep = new list<int>; //intermediary structure to keep track of the compounds that satisfy the condition to be kept.

  for (int i=0;i<nb_compounds;i++)
    {
      nb_edges_in_line=0;
      for (int j=0;j<nb_reactions;j++)
	{
	  if (bipartite_graph_to_filter->get_adjacency_matrix()[i][rea_tab[j]])
	    nb_edges_in_line++;
	}

      /*the compound is involved in more than 2 reactions*/
      if (nb_edges_in_line > 1)
	compounds_to_keep->push_back(i);
    }

  /*adjust the number of compounds in the filtered graph*/
  nb_compounds=compounds_to_keep->size();


  /*create columns corresponding to kept compounds and fill them*/
  adjacency_matrix = new int *[nb_compounds];
  list<int>::iterator it=compounds_to_keep->begin();
  for (int i=0;i<nb_compounds;i++)
    {
      adjacency_matrix[i]=new int[nb_reactions];
      for (int j=0;j<nb_reactions;j++)
	adjacency_matrix[i][j]=bipartite_graph_to_filter->get_adjacency_matrix()[*it][rea_tab[j]];
      it++;
    }

  /*adjust reaction_colors (and filter them)*/
  /*by default, each reaction has 4 colors (corresponding to the 4
  thresholds)*/
  reaction_colors = new list<int> *[nb_reactions];
  for (int j=0;j<nb_reactions;j++)
    {
      reaction_colors[j]=new list<int>;
      for (list<int>::iterator it = bipartite_graph_to_filter->get_reaction_colors()[rea_tab[j]]->begin() ; it != bipartite_graph_to_filter->get_reaction_colors()[rea_tab[j]]->end(); it++)
	if (is_in_tab(*it,colors_to_filter,nb_reactions))
	  reaction_colors[j]->push_back(*it);
    }


  /*adjust labels (reactions, compounds) - memory allocation + fill*/
  reactions = new Reaction *[nb_reactions];
  compounds = new Compound *[nb_compounds];
  for (int j=0;j<nb_reactions;j++)
    reactions[j]=bipartite_graph_to_filter->get_reactions()[rea_tab[j]];

  int i=0;
  for (list<int>::iterator it=compounds_to_keep->begin(); it!=compounds_to_keep->end(); it++)
    {
      compounds[i]=bipartite_graph_to_filter->get_compounds()[*it];
      i++;
    }



  /*adjust labels (no need to store that information, it will not be used)*/
  // reactions=NULL;
//   compounds=NULL;

  delete compounds_to_keep;
}

/*creates a bipartite graph from a set of reactions (the compounds are fetched inside the function)*/
// Bipartite_Graph::Bipartite_Graph(Reaction **reactions, int nb_reactions)
// {
//   /*get reactions*/
//   //this->reactions = new (Reaction *)[nb_reactions];

//   /*get compounds*/
//   for (int j=0;j<nb_reactions;j++)
//     {
//       list<Compound *>::iterator it = reactions[j]->get_compounds()->begin();
//       while (it != reactions[j]->get_compounds()->end())

//     }


//   this->nb_compounds = nb_compounds;
//   this->nb_reactions = nb_reactions;
  
//   /*matrix creation + initialisation*/
//   adjacency_matrix = new (int *)[nb_compounds];
//   for (int i=0;i<nb_compounds;i++)
//     {
//       adjacency_matrix[i] = new int[nb_reactions];
//       for (int j=0;j<nb_reactions;j++)
// 	adjacency_matrix[i][j]=0;
//     }

//   /*node labeling*/
//   this->reactions = reactions;
//   this->compounds = compounds;

//   /*fill matrix*/
//   int which_side = 0;
//   for (int i=0;i<nb_compounds;i++)
//     {
//       for (int j=0;j<nb_reactions;j++)
// 	{
// 	  which_side = reactions[j]->which_side(compounds[i]);
// 	  adjacency_matrix[i][j]=which_side;
// 	}
//     }
// }



Bipartite_Graph::~Bipartite_Graph()
{
  for (int i=0;i<nb_compounds;i++)
    delete[](adjacency_matrix[i]);
  delete[](adjacency_matrix);


  if (reactions != NULL)
    delete[] reactions;

  if (compounds != NULL)
    delete[] compounds;

  if (reaction_colors != NULL)
    {    
      for (int j=0;j<nb_reactions;j++)
	delete reaction_colors[j];
      delete[] reaction_colors;
    }

}

/**
 *Get the number of reactions of the bipartite graph.
 *
 * @return number of reactions
 */
int Bipartite_Graph::get_nb_reactions()
{
  return nb_reactions;
}

/**
 *Get the number of compounds of the bipartite graph.
 *
 * @return number of compounds
 */
int Bipartite_Graph::get_nb_compounds()
{
  return nb_compounds;
}


/**
 *Get the list of reactions of the bipartite graph.
 *
 * @return array of reactions
 */
Reaction **Bipartite_Graph::get_reactions()
{
  return reactions;
}


/**
 *Get the list of compounds of the bipartite graph.
 *
 * @return array of compounds
 */
Compound **Bipartite_Graph::get_compounds()
{
  return compounds;
}

/**
 *Get the adjacency matrix of the bipartite graph
 *
 * @return adjacency matrix describing the topology of the graph
 */
int **Bipartite_Graph::get_adjacency_matrix()
{
  return adjacency_matrix;
}


/**
 *Get the reaction_colors of the bipartite graph
 *
 * @return reaction colors of the graph
 */
list<int> **Bipartite_Graph::get_reaction_colors()
{
  return reaction_colors;
}


int Bipartite_Graph::get_topo()
{
  // print_adjacency_matrix();
  int topo=0;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    topo+=pow(2,i)*pow(3,j);
	  
	}
    }
   // cout<<topo<<endl;
   // exit(0);

  return topo;
}

/*returns true if compounds i and j are on opposite sides of reaction k*/
/*when the side is undefined, we suppose that we have opposite sides*/
bool Bipartite_Graph::opposite_side(int i, int j, int k)
{
  int side1=adjacency_matrix[i][k];
  int side2=adjacency_matrix[j][k];
  //cout <<"side1 "<<side1<<" side2 "<<side2<<endl;
  //if (side1==0 || side2==0 || (side1==side2 && side1!=3))
  if (side1==3 || side2==3) //if one of the compounds is undefined
    return 1;
  else if (side1==1 && side2==2) //if one is on the left and the other on the right
    return 1;
  else if (side1==2 && side2==1) //if one is on the right and the other on the left
    return 1;
  else 
    return 0;

  // if (side1==0 || side2==0 || side1==side2)
//     return 0;
//   else 
//     return 1;
}






/*prints the adjacency matrix*/
void Bipartite_Graph::print()
{
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  cout<<adjacency_matrix[i][j];
	}
      cout<<endl;
    }
}


/**
 * Get the canonical labelling of this graph by calling nauty.
 */
void Bipartite_Graph::set_canonical_labelling()
{
//   cout<<"in set_canonical_labelling"<<endl;  


//   /*declare variables and initialize sizes to 0*/
//   /*all these variables are necessary for Nauty*/
//   DYNALLSTAT(graph,g,g_sz);
//   DYNALLSTAT(int,lab,lab_sz);
//   DYNALLSTAT(int,ptn,ptn_sz);
//   DYNALLSTAT(int,orbits,orbits_sz);
//   static DEFAULTOPTIONS_GRAPH(options);
//   statsblk(stats);
//   setword workspace[100];

//   int n,m;
//   set *gv;

//   /*we don't want nauty to write anything*/
//   options.writeautoms = TRUE;

//   /*m corresponds to the memory taken by one cell of the graph*/
//   /*In Nauty format, a graph is a matrix of bits*/
//   n=nb_reactions+nb_compounds; //n is the number of nodes of the graph
//   m = (n + WORDSIZE - 1) / WORDSIZE; //m is memory parameter
  
//   nauty_check(WORDSIZE,m,n,NAUTYVERSIONID);

//   DYNALLOC2(graph,g,g_sz,m,n,"malloc");
//   DYNALLOC1(int,lab,lab_sz,n,"malloc");
//   DYNALLOC1(int,ptn,ptn_sz,n,"malloc");
//   DYNALLOC1(int,orbits,orbits_sz,n,"malloc");


//   for (int i=0;i<nb_compounds;i++)
//     {
//       gv = GRAPHROW(g,i,m);
//       for (int j=0;j<nb_reactions;j++)
// 	{
// 	  /*add edges from compound to reaction and from reaction to compound*/
// 	  ADDELEMENT(gv,nb_compounds+j);
// 	  ADDELEMENT(GRAPHROW(g,nb_compounds+j,m),i);
// 	}
//     }

//   /*declare and allocate memory for graph with canonical labelling*/
//   DYNALLSTAT(graph,canong,canong_sz);
//   DYNALLOC2(graph,canong,canong_sz,n,m,"dreadnaut");



//   /*call nauty*/
//   nauty(g,lab,ptn,NULL,orbits,&options,&stats,
// 	workspace,100,m,n,canong);



//   // /*compare graphs savedg and canondg*/
// //   for (li = (long)n * (long)m; --li >= 0;)
// //     if (savedg[li] != canong[li]) break;

}


/**
 * Print the bipartite graph to file in dreadnaut format. The first nodes are the compounds, they are the source nodes for each edge. Each time, we want to change the source node we print ";". Finally, we print a "." when the job is finished.
 *
 */
void Bipartite_Graph::print_dreadnaut(ostream &output)
{
  /*print graph*/
  output<<"n="<<nb_reactions+nb_compounds<<" g"<<endl;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    output<<nb_compounds+j<<" ";
	}
      /*end of the graph*/
      if (i==nb_compounds-1)
	output<<"."<<endl;
      /*end of the line*/
      else
	output<<";"<<endl;
    }

  /*print permutation - i.e. vertex colours - for the moment only compounds and reactions are separated*/
  output<<"f=[0:"<<nb_compounds-1<<"]"<<endl;

}


/*Prints to standard output the order of colours, that is the partition of nodes*/
/*This is a copy and paste of print_dreadnaut_ordered*/
void Bipartite_Graph::print_colour_order()
{
  for (int i=0;i<nb_reactions;i++)
    cout<<reaction_colors[i]->front()<<" ";
  cout<<endl;
}


/**
 * Print the bipartite graph with colors attached to the reaction vertices to file in dreadnaut format. The first nodes are the compounds, they are the source nodes for each edge. Each time, we want to change the source node we print ";". Finally, we print a "." when the job is finished. 
 *
 */
void Bipartite_Graph::print_dreadnaut_ordered(ostream &output)
{

  //cerr<<"début de dreadnaut_ordered"<<endl;



  /*print graph*/
  output<<"n="<<nb_reactions+nb_compounds<<" g"<<endl;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    output<<nb_compounds+j<<" ";
	}
      /*end of the graph*/
      if (i==nb_compounds-1)
	output<<"."<<endl;
      /*end of the line*/
      else
	output<<";"<<endl;
    }

  /*print permutation - i.e. vertex colours - for the moment only compounds and reactions are separated*/
  /*You can use the abbreviation \x:y" for the range x; x+1; : : : ; y. Any vertices not
    named are put in a cell of their own at the end.*/
  /*Example: If n = 10, then f=[3:7 | 0,2]" establishes the partition
    [3, 4, 5, 6, 7 | 0, 2 | 1, 8, 9].*/
  /*note that the f=[3:7 | 0,2] and explicitely setting f=[3, 4, 5, 6, 7 | 0, 2 | 1, 8, 9] will not generate the same codes. This problem happened only once and I could not find it again when trying (not so hard).*/
  
  /*In order to tell dreadnaut what the colour groups are, we need to
    build a reverse structure telling us for each colour, which nodes
    have it*/
  /*For this purpose, we first compute the number of unique colors,
    which we store in a tab: unique_colors*/
  /*Second, we fill the tab of lists: colors_reactions which can be
    used directly to tell dreadnaut the color_classes*/


  /*get number of distinct colours*/
  list<int> reactions_colors; //one reaction has only one colour

  /*convert reaction colors into a list, keeping only the first color
    of each node*/
  for (int i=0;i<nb_reactions;i++)
    reactions_colors.push_back(reaction_colors[i]->front());

  //print_list(reactions_colors.begin(),reactions_colors.end());

  reactions_colors.sort();
  reactions_colors.unique();
  int nb_colors=reactions_colors.size();

  /*convert list to tab*/
  int *unique_colors = new int[nb_colors];
  int i=0;
  for (list<int>::iterator it=reactions_colors.begin(); it!= reactions_colors.end(); it++)
    {
      unique_colors[i]=(*it);
      i++;
    }


  // cout<<"le tableau des couleurs est "<<endl;
//   print_tab(unique_colors,nb_colors);

  list<int> *colors_reactions= new list<int>[nb_colors];
  for (int i=0;i<nb_colors;i++)
    {
      /*get nodes with color unique_colors[i]*/
      for (int j=0;j<nb_reactions;j++)
	if (reaction_colors[j]->front()==unique_colors[i])
	  colors_reactions[i].push_back(j);
    }


  // cout<<"à la fin, "<<endl;
//   for (int i=0;i<nb_colors;i++)
//     print_list(colors_reactions[i].begin(), colors_reactions[i].end());



  /*print colors for dreadnaut*/
  output<<"f=[0:"<<nb_compounds-1<<"|";

  for (int i=0;i<nb_colors;i++)
    {
      /*print all nodes sharing the same color*/
      for (list<int>::iterator it=colors_reactions[i].begin(); it!=colors_reactions[i].end(); it++)
	{
	  output<<nb_compounds+*it;
	  it++;
	  if (it!=colors_reactions[i].end())
	    output<<",";
	  it--;
	}
      if (i==nb_colors-1)
	output<<"]"<<endl;
      else
	output<<"|";
    }


  delete[] unique_colors;
  delete[] colors_reactions;
  //exit(0);

}



/*prints the adjacency matrix and the labels*/
void Bipartite_Graph::print_nodes_matrix(ostream &output)
{
  //std::ofstream output;
  //output.open("bipartite_graph.txt", ios_base::app);
  for (int i=0;i<nb_compounds;i++)
    {
      compounds[i]->print(output);
      output<<"\t";
    }
  output<<endl;
  for (int j=0;j<nb_reactions;j++)
    {
      reactions[j]->print(output);
      output<<"\t";
    }
  output<<endl;
  //output<<endl;
  
  for (int i=0;i<nb_compounds;i++)
    {
      //compounds[i]->print(output);
      for (int j=0;j<nb_reactions;j++)
	{
	  output<<adjacency_matrix[i][j];
	}
      output<<endl;
    }
  //output.close();
}

/*prints the adjacency matrix in the other way, reactions as rows and compounds as columns*/
void Bipartite_Graph::print_to_file_reverse()
{
  std::ofstream output;
  output.open("bipartite_graph2.txt", ios_base::app);
  for (int j=0;j<nb_reactions;j++)
    {
      reactions[j]->print(output);
      output<<" ";
    }
  output<<endl;
  for (int i=0;i<nb_compounds;i++)
    {
      compounds[i]->print(output);
      output<<" ";
    }
  output<<endl;
  output<<endl;
  
  for (int j=0;j<nb_reactions;j++)
    {
      //reactions[j]->print(output);
      for (int i=0;i<nb_compounds;i++)
	{
	  output<<adjacency_matrix[i][j];
	}
      output<<endl;
    }
  output.close();
}

void Bipartite_Graph::draw_structure()
{
  std::ofstream output;
  output.open("bidon.res", ios_base::app);
  
  output <<"graph G {"<<endl;
  //output << "node[height=.3, width=.3];"<<endl;
  output << "{node[shape=box, style=filled, fontsize=10,height=.3, width=.3 ]";
  for (int j=0;j<nb_reactions;j++)
	{
	  //reactions[j]->draw(output);
	  output<<j;
	  output<<" ";
	}
  output<<"}"<<endl;

  output << "{node[shape=ellipse, style=filled, fontsize=10,height=.3, width=.3 ]";
  for (int i=0;i<nb_compounds;i++)
	{
	  output<<1000+i;
	  //compounds[i]->draw(output);
	  output<<" ";
	}
  output<<"}"<<endl;


  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    {
	      output<<1000+i;
	      //compounds[i]->draw(output);
	      output <<" -- ";
	      output<<j;
	      //reactions[j]->draw(output);
	      output <<";"<<endl;
	    }
	}
    }
  output<<"}"<<endl;
  output.close();

  /*compile with neato*/
  if (system("neato bidon.res -Tps -o draw.ps")!=0)
    {
      cerr<<"unable to execute command: neato bidon.res -Tps -o draw.ps"<<endl;
      exit(0);
    }
  
  if (system("rm bidon.res")!=0)
    {
      cerr<<"unable to execute command: rm bidon.res"<<endl;
      exit(0);
    }
  
  /*draw with gv*/
  if (system("gv draw.ps &")!=0)
    {
      cerr<<"unable to execute command: gv draw.ps &"<<endl;
      exit(0);
    }

}



void Bipartite_Graph::draw()
{
  std::ofstream output;
  output.open("bidon.res", ios_base::app);
  
  output <<"graph G {"<<endl;
  //output << "node[height=.3, width=.3];"<<endl;
  output << "{node[shape=box, style=filled, fontsize=10,height=.3, width=.3 ]";
  for (int j=0;j<nb_reactions;j++)
	{
	  reactions[j]->draw(output);
	  output<<" ";
	}
  output<<"}"<<endl;

  output << "{node[shape=ellipse, style=filled, fontsize=10,height=.3, width=.3 ]";
  for (int i=0;i<nb_compounds;i++)
	{
	  compounds[i]->draw(output);
	  output<<" ";
	}
  output<<"}"<<endl;


  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    {
	      compounds[i]->draw(output);
	      output <<" -- ";
	      reactions[j]->draw(output);
	      output <<";"<<endl;
	    }
	}
    }
  output<<"}"<<endl;
  output.close();

  /*compile with neato*/
  if (system("neato bidon.res -Tps -o draw.ps")!=0)
    {
      cerr<<"unable to execute command: neato bidon.res -Tps -o draw.ps"<<endl;
      exit(0);
    }
  
  if (system("rm bidon.res")!=0)
    {
      cerr<<"unable to execute command: rm bidon.res"<<endl;
      exit(0);
    }
  
  /*draw with gv*/
  if (system("gv draw.ps &")!=0)
    {
      cerr<<"unable to execute command: gv draw.ps &"<<endl;
      exit(0);
    }
}


void Bipartite_Graph::print_nodes_edges(std::ostream& output)
{

  /*print nodes*/
  for (int i=0;i<nb_compounds;i++)
    {
      compounds[i]->print(output);
      output<<"\t";
    }
  output<<endl;
  output<<endl;
  for (int j=0;j<nb_reactions;j++)
    {
      reactions[j]->print(output);
      output<<"\t";
    }
  output<<endl;
  output<<endl;
  /*print edges*/
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    {
	      compounds[i]->print(output);
	      output <<"\t";
	      reactions[j]->print(output);
	      output <<"\t";
	      output<<adjacency_matrix[i][j];
	      output <<endl;
	    }
	}
    }
}

/**
 *Prints bipartite graph to file using simple format. Each line is an edge (node node).
 *
 * @param output device where to print the graph
 */
void Bipartite_Graph::print_edges(std::ostream& output)
{
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  if (adjacency_matrix[i][j])
	    {
	      compounds[i]->print_id(output);
	      output <<"\t";
	      reactions[j]->print(output);
	      // output <<" ";
// 	      output<<adjacency_matrix[i][j];
	      output <<endl;
	    }
	}
    }
}

void Bipartite_Graph::print_adjacency_matrix()
{
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  cout<<adjacency_matrix[i][j];
	}
      cout<<endl;
    }
}

void Bipartite_Graph::print_identity_card()
{
  int nb_edges=0;
  for (int i=0;i<nb_compounds;i++)
    {
      for (int j=0;j<nb_reactions;j++)
	{
	  nb_edges += (adjacency_matrix[i][j] != 0);
	}
    }
  cout<<"the bipartite graph contains "<<nb_compounds+nb_reactions<<" nodes and "<<nb_edges<<" edges"<<endl;

}
