/* Filtered_Graph.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "FilteredGraph.h"
#include "my_library.h"
#include <iostream>
#include <queue>
#include <algorithm>
#include <stack>

/** Constructor
 * 
 * @param graph_file file where graph is written.
 * @param labels_file file where color of node are written.
 * @param EC_numbers  identification tab of Enzyme.
 * @param nb_EC
 * @param colors_to_filter colors to keep.
 * @param motif_size number of color to keep.
 * @param local_gap local gap.
 * @param global_gap global gap value.
 */
FilteredGraph::FilteredGraph(char *graph_file,
			     char *labels_file,
			     char **EC_numbers,
			     int nb_EC,
			     int* colors_to_filter,
			     int color_size,
			     int local_gap,
			     int global_gap):Graph(graph_file,
						  labels_file,
						  EC_numbers,
						  nb_EC){
  
  int i;
  list<int>::iterator it;
  int initial_nb_nodes = Graph::get_nb_nodes();
  this->local_gap = local_gap;
  this->global_gap = global_gap;
  //gives indices of nodes you gotta keep since they have the right color

  parent_nodes=new int[initial_nb_nodes]; 
  
  //gives the correspondance between indices in parent graph and the new graph
  int *matchingTab = new int[initial_nb_nodes];
  
  int ** tmpDistanceMatrix;//matrix of distance for transitive closure
  list<int>** tmpAdjacencyList; //AdjacencyList representation of distanceMatrix
  list<int>** tmp_color = Graph::get_colors();

  /*get size of the filtered graph*/
  nb_node=0;
  for (i=0;i<initial_nb_nodes;i++){
    if(is_list_in_tab(Graph::get_colors()[i],colors_to_filter,color_size)){
      cout << " parent_nodes[" << nb_node<< "] = "<<i<<endl;;
      parent_nodes[nb_node]=i;
      matchingTab[i]=nb_node;
      nb_node++;
    }
    else 
      matchingTab[i]=-1;
  }
  
  cout << "dans le constructor de filtered Graph : " << endl;
  cout << "size of graph : "<< nb_node << endl;
  
  /*allocate memory (with the right size)*/  
  
  distance_list = new list<pair<int,int> > *[nb_node];
  distance_matrix = new int*[nb_node];
  colors = new list<int> *[nb_node];
  for (int i=0;i<nb_node;i++){
    distance_list[i] = new list<pair<int,int> >;
    distance_matrix[i] = new int[nb_node];
    colors[i] = new list<int>;
  }
  
  if(local_gap>0){
    
    ///////////////////////////////////////////
    // Construction of the local_gap-closure //
    
    tmpDistanceMatrix = floyd(local_gap,Graph::get_adjacency_matrix(),initial_nb_nodes);
    
    cout << "matrix of : "<<local_gap << "-transitive closure :"<<endl;
    printMatrix(tmpDistanceMatrix,initial_nb_nodes);
    cout << endl;
    
    tmpAdjacencyList = distanceMatrixToAdjacencylist(tmpDistanceMatrix,initial_nb_nodes);
    
    //////// end of transitive closure ////////
    ///////////////////////////////////////////
  }
  else{
    tmpAdjacencyList = Graph::get_adjacency_list();
    tmpDistanceMatrix = boolMtrixToIntMatrix(Graph::get_adjacency_matrix(),initial_nb_nodes);
  }

  /*copy and filter adjacency matrix, list and colors*/
  for(i=0 ; i<nb_node ; i++){
    for(int j = i ; j<nb_node ; j++){
      distance_matrix[i][j] = tmpDistanceMatrix[parent_nodes[i]][parent_nodes[j]];
      distance_matrix[j][i] = distance_matrix[i][j];
      if (distance_matrix[i][j]>0){

	//insert in order of distance
	list<pair<int,int> >::iterator it1 = distance_list[i]->begin();   
	do{ 
	  if(it1==distance_list[i]->end() || distance_matrix[i][j] <= (*it1).second){
	    distance_list[i]->insert(it1,make_pair(j,distance_matrix[i][j]));
	    break;
	  }
	  it1++;
	}
	while(true);
	
	it1=distance_list[j]->begin();
	do{ 
	  if(it1==distance_list[j]->end() || distance_matrix[j][i] <= (*it1).second){
	    distance_list[j]->insert(it1,make_pair(i,distance_matrix[j][i]));
	    break;
	  }
	  it1++;
	}
	while(true);
      }
    }
   *colors[i]=*tmp_color[parent_nodes[i]];
  }


  //deleting temporary variables
  if(local_gap>0) {
    //deleting matrice distance
    deleteMatrix(tmpDistanceMatrix,initial_nb_nodes);
    
    //deleteting tmpAdjacencyList
      for (int i=initial_nb_nodes-1;i>=0;i--)
	delete tmpAdjacencyList[i];
      delete[]tmpAdjacencyList;
    }
  delete[]matchingTab;
}


//destructor

FilteredGraph::~FilteredGraph(){
  if(distance_list!=NULL)
    {
      for (int i=nb_node-1;i>=0;i--)
	delete distance_list[i];
      delete[]distance_list;
    }
  
  /*Already done by Graph 
  if (colors!=NULL){
    for (int i=0;i<nb_node;i++)
      delete(colors[i]);
    delete[]colors;
  }
  */

  if(distance_matrix!=NULL)
    deleteMatrix(distance_matrix,nb_node);
  delete[] parent_nodes;
}

/**
 * Print the distance matrix.
 */
void FilteredGraph::print_distance_list(){
  list<pair<int,int> >::iterator it;
  cout <<endl<< "start -> (destination - distance)"<<endl ;
  for(int i=0 ; i < nb_node ; i++){
    cout << i << " -> ";
    for(it = distance_list[i]->begin()  ; it!=distance_list[i]->end(); it++)
      cout<< it->first <<" - "<< it->second << " || ";
    cout << endl;
  }
  cout << endl;
}





/**
 *Calculate the frequency of the color of the graph and fill the given tab with the node who has this color.
 *
 * @param tabToFill a non allocate pointer on integer, it will be allocate here with the size  of only node which will start the search algorithm
 * @return number of node wich color is less frequent
 */
int FilteredGraph::frequency(int **tabToFill){
 
cout << "Frequence :" << endl;

  int* nodeWithLowFrequency = new int[nb_node];
  //map of color / frequency 
  map<int,int> colorFrequency;
  int tmpMin;
  int color;
  list<int>::iterator it;
  //first, count for each color its frequency
  for(int i = 0; i< nb_node ;i++){
    for(it = colors[i]->begin() ; it != colors[i]->end() ; it++){
      colorFrequency[*it]++;
      cout << "it(color) = " << *it << " val(frequency) = " << colorFrequency[*it]<< endl;
    }
  }
  map<int,int>::iterator itmap;
  itmap = colorFrequency.begin();
  tmpMin = (*itmap).second;
  color = (*itmap).first;

  //second, for each color select the one with minimal frequency 
  for(itmap = colorFrequency.begin() ; itmap != colorFrequency.end(); itmap++){
    if((*itmap).second < tmpMin){
      color = (*itmap).first;
      tmpMin = (*itmap).second;
     
    }
  }
 
  // third, construction of tab of node with the color found before
  int j=0;
  for(int i = 0; i< nb_node ;i++){
    for(it = colors[i]->begin() ; it != colors[i]->end() ; it++){
      if(*it == color){
	nodeWithLowFrequency[j++]=i;
      }
    }
  }
  (*tabToFill) = new int[j];
  //fourth, return the table with low frequency color node wich has the right size
  for(int i =0 ;i<j ; i++){
    (*tabToFill)[i]=nodeWithLowFrequency[i];
  }
  delete[]nodeWithLowFrequency;
  return j;
}

/** 
 * Print the list of color of each node of the graph.
 */
void FilteredGraph::printNodeColor(){
 list<int>::iterator it;
  for(int i=0 ; i < nb_node; i++){
    cout << "node " << i << "color : ";
    for(it = colors[i]->begin() ; it != colors[i]->end() ; it++ )
      cout << *it << " ";
    cout << endl;
  }
}

/**
 *Transform the  solution of the filtered graph to the solution in the original graph.
 * @param finalSolution the pointer to the table to put the solution of the original graph 
 * @param tmpSolution the solution in the filtered graph
 * @param motif_size size of the motif
 */
void FilteredGraph::filterOneSolution(int** finalSolution,
				      list<int> tmpSolution,
				      int motif_size){
  list<int>::iterator it; 
  int i;
  for(i = 0, it= tmpSolution.begin();
      i < motif_size ;
      i++, ++it){
    (*finalSolution)[i] = this->parent_nodes[*it];
  }
}

/**
 * Backtracking consist of 
 * (1) unmarking the vertex pointed by p,
 * (2) updating the number of used solution,
 * (3) removing all neighbor of actual node from the list that were inserted when actual node was marked
 * (4) updating gap contribution for neighbor of unmarked node
 */
void FilteredGraph::backtrack(
			      list<int>* markedNode,
			      int* partOfMotifMarked,
			      int* actualNode,
			      list<int>*** addedNeighbor,
			      list<int>* neighborList2,
			      int*** tabGap,
			      unsigned int* color_count_coef,
			      bool ***usedEdgeForGap
			      ){
  list<int>::iterator it;
  list<pair<int,int> >::iterator it_pair;
  list<int*>::iterator it_tab;

  int tmpNode,tmpGap;

  /*
    (1) unmark the vertex pointed by p.
  */
  tmpNode =(*markedNode).back();
  (*markedNode).pop_back();
  
  /*
    (2)updating the number of used solution
   */
  *color_count_coef /= colors[tmpNode]->size();
  (*partOfMotifMarked)--;
  
  /*
    (3)
    removing all neighbor of actual node from the list that were
    inserted when actual node was marked
  */
  for(it = ((*addedNeighbor)[*actualNode])->begin();
      it!= ((*addedNeighbor)[*actualNode])->end();
      ){
    (*neighborList2).remove(*it);
    it=((*addedNeighbor)[*actualNode])->erase(it);
  }
  /*
    (4)
    updating gap contribution for neighbor of unmarked node
  */
  for(it_pair = distance_list[*actualNode]->begin();
      it_pair!= distance_list[*actualNode]->end();
      it_pair++){
    tmpNode = (*it_pair).first;
    tmpGap = (*it_pair).second -1;
    if((*usedEdgeForGap)[*actualNode][tmpNode]==true){
      (*tabGap)[tmpNode][tmpGap]--;
      (*usedEdgeForGap)[*actualNode][tmpNode] = false;
    }
    //cout << " pour le sommet " << tmpNode <<", gap retir = " << tmpGap  <<endl;
  }
}

/** Launch the search of the given motif colors for the filtered graph.
 *
 * @param solutions
 * @param motif_size
 * @param motif_colors
 */
int FilteredGraph::enumerate(list<int*>*solutions,
			      int motif_size,
			      int *motif){
  
  int *nodeToVisite;
  int nb_node_to_visite;
  int tmpNeighbor ; 
  list<pair<int,int> >::iterator it_pair;
  list<int>::iterator it;
  list<int*>::iterator it_tab;
  //pre-treatment :
  //calculate node frequency and fill a tab of node to visit
  nb_node_to_visite = frequency(&nodeToVisite);

 
  
  //the queue of the node to visit
  queue<int> startingNodeQueue;

  for(int i=0 ; i<nb_node_to_visite ; i++){
    startingNodeQueue.push(nodeToVisite[i]);
    cout << "node to start the algorithm : " <<nodeToVisite[i]<< endl;
  }
  
  //eventualy solutions during the course
  list<int*> tmpSolution;
  
  //to test the solution
  bool* matchedSolution = new bool[motif_size];
  unsigned int color_count_coef=1; //number of solution coeficient during a course

  //node not use for the matching of current research
  bool *eliminatedNode = new bool[nb_node];

  //node in treatment
  int actualNode;
 
  // count the number of the node that are a part of the motif
  int partOfMotifMarked=0; 

  //remind marked node
  list<int> markedNode;

  //list of neighbore - the pair represent node and the distance with the node wich added it
  //list<pair<int,int> > neighborList;
  
  list<int> neighborList2;

  //to know which node add one node to the queue - each node has its list of added neighbore
  list<int> **addedNeighbor; 

  //the node which one the search start
  int startingNode;
  /*
    global gap contribution reminder
    each node has its map of global gap contribution
    first int correspond to the global gap that the nodes in the list give.
  */
 
  bool **usedEdgeForGap = new bool*[nb_node];
  
  int **tabGap = new int*[nb_node]; // for each node : counter for each level of gap 
  addedNeighbor = new list<int> *[nb_node];
  for(int i=0 ; i <nb_node ; i++){
    eliminatedNode[i]=false;
    addedNeighbor[i] = new list<int>;
    tabGap[i] = new int[local_gap+1];
    usedEdgeForGap[i] = new bool[nb_node];
    for(int j = 0 ; j <= local_gap ; j++){
      tabGap[i][j]=0;
    }
    for(int j = 0 ; j < nb_node ; j++){
      usedEdgeForGap[i][j] = false;
    }
  }  

  cout << "Research Start"<<endl <<endl;

  while(startingNodeQueue.size() > 0){
      
    /*
      removing the first node
    */     
    startingNode = startingNodeQueue.front();
    actualNode = startingNode;
    startingNodeQueue.pop();

    //test
    cout << endl << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << endl<<"debut : actualNode = "<< actualNode << endl<< endl;
    
    //put the starting node into the neighbore list
    neighborList2.push_back(actualNode);
    
    /*
      mark it
    */
    markedNode.push_back(actualNode);
    color_count_coef *=  colors[actualNode]->size();
     //it is the first node of a eventual solution
    cout << "color_count_coef = "<< color_count_coef <<endl; 
    /*
      create one temporary solution per color
    */
    
    if(tmpSolution.size() < color_count_coef){
      for(it = colors[actualNode]->begin() ;
	  it != colors[actualNode]->end();
	  it++ ){     
	int *tmp = new int[motif_size];
	tmp[0] = *it;
	tmpSolution.push_back(tmp);
      }
    }
 
    partOfMotifMarked++;
    
    /*
      adding all neighbore to the queue
    */
    //first -> destination node, second -> distance
    for(it_pair = distance_list[actualNode]->begin();
	it_pair!=distance_list[actualNode]->end();
	it_pair++){
      
      tmpNeighbor = (*it_pair).first;
      /*
	add neighbor if it not a starting node of a precedent research
      */      
      if(eliminatedNode[tmpNeighbor]==false){
	neighborList2.push_back(tmpNeighbor);
      
      /*
	tell which neighbore was added by actual node
      */
      addedNeighbor[actualNode]->push_back(tmpNeighbor);
      
      /*
	updating gap for neighbore of first node
       */
      tabGap[tmpNeighbor][((*it_pair).second)-1]++;
      usedEdgeForGap[actualNode][tmpNeighbor] = true;
      }
    }
    //breaking condition : while queue is not empty 
    while(true){
      
      //if we have marked the same number of node as the size of motif
      if(partOfMotifMarked == motif_size){
	
	/*
	  colouring test condition
	*/
	cout << "*colouring testcondition start*"<< endl;
       	/*
	  for each solution build in this course do the test
	*/
	unsigned int sol_counter;
	bool oneTest = true; // if there is multiple color for node stop when a good match
	for(sol_counter = 0, it_tab = tmpSolution.begin();
	    sol_counter < color_count_coef;/* it_tab != tmpSolution.end();*/
	    sol_counter++, it_tab++ ){
	  int* tmp = *it_tab;
	  
	  int t;
	  cout << "\tnode <-> color : "<< endl<<"\t";
	  for(it = markedNode.begin(),t = 0 ; t< motif_size ; it++, t++)
	    cout << *it << "<->" << tmp[t] <<" | ";
	 
	  cout << endl;
	  cout << "\ttest this ? ";
	  if(oneTest) cout << "yes" << endl;
	  else  cout << "no" << endl << endl;
	  /**test**/
	  /*	  for(int i=0 ; i<nb_node ; i++){
	    for(int j = 0 ; j<local_gap+1 ; j++)
	      cout << tabGap[i][j];
	    cout << endl;
	  }
	  */
	  if(oneTest && color_test(tmp,motif_size,motif,motif_size,matchedSolution)== 1){  
	    cout << endl << "\t----------------------------------------" << endl;
	    
	    
	    //ajouter la solution a l'ensemble des solutions
	    cout << endl<< "\tcolor matched for nodes : ";
	    int gb = 0;
	    //valeur du global gap :
	    for(it = markedNode.begin();
		it!=markedNode.end();
		it++){
	      tmpNeighbor = (*it);
	      cout << tmpNeighbor << " ";
	      if(tmpNeighbor == startingNode )
		continue;
	      int j=0;
	      while(j<=local_gap){
		if(tabGap[tmpNeighbor][j]>0){
		  gb+=j;
		  break;
		}
		j++;
	      }
	      oneTest = false;
	    }
	    if(gb <=  global_gap){
	      int *finalSolution = new int[motif_size];
	      filterOneSolution(&finalSolution, markedNode, motif_size);
	      (*solutions).push_back(finalSolution);	      
	    }
	    cout << endl << "\tgap global pour le motif : " << gb << endl;
	    cout <<  "\t----------------------------------------" << endl << endl;
	  } 
	}
	cout << "*colouring testCondition finish*"<< endl << endl << endl;
	/*
	  backtracking
	*/
	backtrack(&markedNode,
		  &partOfMotifMarked,
		  &actualNode,
		  &addedNeighbor,
		  &neighborList2,
		  &tabGap,
		  &color_count_coef,
		  &usedEdgeForGap);
      }
      if(partOfMotifMarked < motif_size){
	if(actualNode != neighborList2.back()){
	  /*
	    getting the next node of actual node in the list
	  */
	  it = find(neighborList2.begin(),neighborList2.end(),actualNode);
	  
	  ++it;
	  actualNode = (*it);
	  
	  /*
	    mark it
	  */
	  markedNode.push_back(actualNode);
	  
	  /*
	    adding color to the solution
	  */
	  int nb_color =  colors[actualNode]->size();
	  color_count_coef *= nb_color ;
	  unsigned int initial_size = tmpSolution.size();
	  /*
	    if node has multiple color and it doesn't exist a tab to store it
	  */
	  if(color_count_coef > initial_size){   
	    /*
	      get others tabs
	    */
	    unsigned int k = initial_size;
	    for(it_tab = tmpSolution.begin() ;
		k<color_count_coef;
		it_tab++,k++ ){
	      /*
		copy others tabs into a new one and add it to the pool of solution
	      */
	      int* tmp2 = *it_tab;
	      int* newTab = new int[motif_size];
	      memcpy(newTab,tmp2,partOfMotifMarked*sizeof(int));
	      tmpSolution.push_back(newTab);
	    }
	  }
	  
	  /*
	    for each tab of solution, adding corresponding colors
	  */
	  int* tmp;
	  it_tab = tmpSolution.begin();
	  for(it = colors[actualNode]->begin();
	      it != colors[actualNode]->end();
	      it++ ){
	    int color = *it;
	    
	    for (unsigned int m = 0 ; m<initial_size ; m++){
	      tmp = *it_tab;
	      tmp[partOfMotifMarked] = color;
	      it_tab++;
	    }	     
	  }
	  
	  partOfMotifMarked++;
	  
	  /*
	    adding neighbors of actual node which are not in the list
	  */
	  for(it_pair = distance_list[actualNode]->begin();
	      it_pair!=distance_list[actualNode]->end();
	      it_pair++){
	    tmpNeighbor = (*it_pair).first;

	    /*
	      if it is not a eliminated node
	    */
	    if(eliminatedNode[tmpNeighbor]==false){	    
	      if(find(neighborList2.begin(),neighborList2.end(),tmpNeighbor)==neighborList2.end()){
		neighborList2.push_back(tmpNeighbor);
		/*
		  tell which neighbore was added by actual node
		*/
		addedNeighbor[actualNode]->push_back(tmpNeighbor);
	      }
	      /*
		actualNode gap contribution for precedent marqued node
	      */
	      if(usedEdgeForGap[actualNode][tmpNeighbor]==false 
		 && usedEdgeForGap[tmpNeighbor][actualNode]==false){
		tabGap[tmpNeighbor][((*it_pair).second)-1]++;
		usedEdgeForGap[actualNode][tmpNeighbor] = true;
	      }
	    }
	    //cout << " pour le voisin " << tmpNeighbor <<", gap ajout = " << (*it_pair).second-1  << endl <<endl;   
	  }
	}  
	else{
	  /*
	    if exploration is finished for the starting node
	  */
	  if(markedNode.size()==0){
            for(int i=0;i< nb_node;i++){
	      addedNeighbor[i]->clear();
	    }

	    eliminatedNode[startingNode] = true;
            neighborList2.clear(); //pas necessaire
            break;
	  }
	  actualNode = markedNode.back();
	  /*
	    backtracking
	  */
	  backtrack(&markedNode,
		    &partOfMotifMarked,
		    &actualNode,
		    &addedNeighbor,
		    &neighborList2,
		    &tabGap,
		    &color_count_coef,
		    &usedEdgeForGap);
	}
      }  
    }
  }
  cout << "Research End"<<endl <<endl;
  
  //free memory
  
  delete []nodeToVisite;
  for (int i=0;i<nb_node;i++)
    delete(addedNeighbor[i]);
  delete []addedNeighbor;
  //delete []tmpSolution;
  delete []matchedSolution;
  for(int i=0 ; i <nb_node ; i++){
    delete []tabGap[i];
    delete []usedEdgeForGap[i];
  }
  delete []usedEdgeForGap;
  delete []tabGap;
  delete []eliminatedNode;

  int *clean;
  while(!tmpSolution.empty()){
    clean = tmpSolution.back();
    delete []clean;
    tmpSolution.pop_back();
  }
  return solutions->size();
}


