/* Io.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../include/Io.h"
#include <iostream>
//#include <iomanip>
#include <fstream>
#include <sstream>
#include <list>
#include <ctime>
#include <sys/times.h>
#include <math.h>
#include "../include/Metabolic_Motif.h"
#include "../include/Data.h"
#include "../include/Reaction_Graph.h"
#include "../include/Graph.h"
#include "../include/Bipartite_Graph.h"
#include "../include/Compound_Graph.h"
#include "../include/Occurrence.h"
#include "../include/Occurrence_Group.h"



#include "../../melangeur/include/melangeur.h"

using namespace std;

Io::Io()
{
}

Io::Io(int argc, char **argv)
{
  // argc is the number of arguments
  // argv are the arguments...
  
  /*initialize settings -- default settings*/
  
  /*obsolete parameters*/
  global_gap=0;
  local_gap=0;
  quorum=100; 
  sigma_C=100;

  /*output parameters*/
  append=0;
  mode=2;  //on Anne's request
  verbose=1;

  /*motif definition*/
  topology=0;  //default is 0 : 1 if I want to group my results according to their topology
  order=0;
  motif_size=3;
  sigma_E=3;

  this->enzymes=new char[MAX];
  this->enzymes[0]='\0'; //sets the string to an empty string
  this->output_compound=new char[MAX];
  this->input_compound=new char[MAX];
  this->input_compound[0]='\0';
  this->output_compound[0]='\0';

  data_file=new char[MAX];
  graph_file=new char[MAX];
  labels_file=new char[MAX];
  results_file=new char[MAX];
  motifs_file=new char[MAX];
  enzymes_file=new char[MAX];
  write_path=new char[MAX];

  data_file[0]='\0';
  graph_file[0]='\0';
  labels_file[0]='\0';
  results_file[0]='\0';
  motifs_file[0]='\0';
  enzymes_file[0]='\0';
  write_path[0]='\0';

  strcpy(write_path,".");
  strcpy(results_file,"motus.res"); 
  //strcpy(data_file,"data/primCpdsSmmReactionsCompounds.col"); //default file : small molecule metabolism of E. coli (only primary compounds are kept)

  nb_frequent_compounds=0;
  nb_simulations=100;
  int digits=4; //number of digits after the comma in p-values

  ec_list=0;
  stat=0;
  fabien_jourdan=0;
  stephane_robin=0;
  alberto_marchetti=0;
  etienne_birmele=0;
  mfinder=0;
  sif=0;
  permutability=0;
  no_p_value=0;

  p_only=0; 

  /*Global variable storing the permutability of the current graph*/
  perm=NULL;

  /*confirms if the memory has been allocated*/
  if (output_compound==NULL || input_compound==NULL || enzymes==NULL)
    cout<<"error memory allocating"<<endl;
   
  /* welcome screen */
  cout<<endl;
  cout << "               mo-mo-motus \n";
  cout << "                   Version:  " << VERSION << "\n";
  cout<<endl;
  
  /* parsing of parameters */ 
  parse(argc, argv);
  
     
  /*data capture*/
  Data *myData=NULL;
  Reaction_Graph *whole_reaction_graph=NULL;
  Reaction_Graph **simulations=NULL;

  /*Data in the case of general graphs*/
  Graph *users_graph=NULL;
  Graph **randomized_graphs = NULL;

  /*Metabolic graphs*/
  if (data_file[0]!='\0')
    {
      /*EcoCyc data*/
      myData = new Data(data_file);
      /*Preprocessing the data: compounds considered as ubiquitous (involved in too many reactions) are withdrawn from the dataset*/
      myData->remove_frequent_compounds(nb_frequent_compounds);


      /*Creation of graphs*/
      whole_reaction_graph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      /*array of graphs - all created in the begining*/
      simulations = new Reaction_Graph *[nb_simulations];  
      /*creates 1000 random graphs and store them in this array*/
      shuffle_graphs(whole_reaction_graph,simulations, nb_simulations);
    }

  /*Data in the case of general graphs*/
  else if (graph_file[0] != '\0' && labels_file[0] != '\0')
	{
	  users_graph = new Graph(graph_file, labels_file);
	  randomized_graphs = new Graph *[nb_simulations]; 
	  shuffle_graphs(users_graph,randomized_graphs, nb_simulations);
	}

  else
    {
      cout<<"There was no input data file, you should either input (a metabolic data file) OR (a graph file AND a labels file)"<<endl;
      exit(0);
    }

  // std::ofstream output;
//   output.open(results_file);
//   whole_reaction_graph->print_degrees(output);
//   output.close();
//   exit(0);

  /******************STATISTICS MODE*******************************/
  if (ec_list)
    {
      std::ofstream output;
      output.open(results_file);
      myData->print_possible_ecs(output);
      output.close();
      exit(0);
    }

  /*I should clean this function*/
  if (stat)
    {
      //myData->print_identity_card();
      //myData->print_enzyme_arity(sigma_E,"tmp/enzyme_arity");
      std::ofstream output;
      output.open(results_file);
      output<<"STATISTICS ON THE DATA"<<endl;
      myData->print_identity_card(output);
      output<<"STATISTICS ON THE REACTION NETWORK"<<endl;
      Reaction_Graph *myReactionGraph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      myReactionGraph->print_identity_card(output);
      
      cout<<"Statistics printed in file: "<<results_file<<endl;
      output.close();
      delete myReactionGraph;
      // myData->print_compound_arity("compound_arity");
      exit(0);
    }

  /*Transform input file to sif format*/
  if (sif)
    {
      std::ofstream output;
      output.open(results_file);
      Reaction_Graph *myReactionGraph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      myReactionGraph->print_sif(output);
      cout<<"sif file printed in "<<results_file<<endl;
      output.close();
      exit(0);
    }

  if (fabien_jourdan)
    {
      /*Write to file some data to give them to other people*/
      std::ofstream output;
      char *file_name=new char[MAX];
      // file_name[0]='\0';
      myData->print_identity_card();

      strcpy(file_name,write_path);
      file_name=strcat(file_name,"/reaction_enzyme.txt");
      output.open(file_name);
      myData->print_reaction_enzyme(output);
      output.close();

      strcpy(file_name,write_path);
      file_name=strcat(file_name,"/enzyme_reaction.txt");
      output.open(file_name);
      myData->print_enzyme_reaction(output);
      output.close();

      strcpy(file_name,write_path);
      file_name=strcat(file_name,"/reaction_pathway.txt");
      output.open(file_name);
      myData->print_reaction_pathway(output);
      output.close();

      strcpy(file_name,write_path);
      file_name=strcat(file_name,"/pathway_reaction.txt");
      output.open(file_name);
      myData->print_pathway_reaction(output);
      output.close();

      Bipartite_Graph *myBipartiteGraph;
      myBipartiteGraph = new Bipartite_Graph(myData->tab_reactions, myData->nb_reactions, myData->tab_compounds, myData->nb_compounds);
      output.open(strcat(write_path,"/bipartite_graph.txt"));
      myBipartiteGraph->print_nodes_edges(output);
      output.close();
      delete[] file_name;
      delete myBipartiteGraph;
      exit(0);
    }

  if (stephane_robin)
    {
      /*Write to file some data to give them to other people*/
      std::ofstream output;
      Bipartite_Graph *myBipartiteGraph;
      myBipartiteGraph = new Bipartite_Graph(myData->tab_reactions, myData->nb_reactions, myData->tab_compounds, myData->nb_compounds);
      if (system("rm bipartite_graph.txt")!=0)
	{
	  cerr<<"unable to delete file";
	  exit(0);
	}

      output.open("bipartite_graph.txt", ios_base::app);
      // myBipartiteGraph->print_nodes_matrix(output);
      myBipartiteGraph->print_edges(output);
      myBipartiteGraph->print_identity_card();
      output.close();
      delete myBipartiteGraph;
      exit(0);
    }

  if (alberto_marchetti)
    {
      std::ofstream output;
      Bipartite_Graph *myBipartiteGraph;
      myBipartiteGraph = new Bipartite_Graph(myData->tab_reactions, myData->nb_reactions, myData->tab_compounds, myData->nb_compounds);
      output.open("bipartite_graph.txt");
      myBipartiteGraph->print_nodes_matrix(output);
      output.close();

      Compound_Graph *myCompoundGraph;
      myCompoundGraph = new Compound_Graph(myBipartiteGraph);
      output.open("compound_graph.txt");
      myCompoundGraph->print_nodes_matrix(output);
      output.close();

      Reaction_Graph *myReactionGraph;
      myReactionGraph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      output.open("reaction_graph.txt");
      myReactionGraph->print_nodes_matrix(output);
      output.close();
      output.open("labels_reaction_graph.txt");
      myReactionGraph->print_labels_reaction_graph(output,sigma_E);
      output.close();
      // cout<<"nb de noeuds"<<myReactionGraph->get_nb_nodes()<<endl;
//       cout<<"nb d'aretes"<<myReactionGraph->get_nb_edges()<<endl;
      delete myBipartiteGraph;
      delete myCompoundGraph;
      delete myReactionGraph;
      exit(0);
    }

  if (etienne_birmele)
    {
      /*temporary for whole graph statistics*/
      //myData->print_nb_compounds_per_reaction();
      myData->print_nb_reactions_per_compound();
      Reaction_Graph *myReactionGraph;
      myReactionGraph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      std::ofstream output;
      output.open("reaction_graph.txt");
      myReactionGraph->print_edges(output);
      output.close();
      output.open("labels_reaction_graph_threshold3.txt");
      myReactionGraph->print_labels_reaction_graph(output,3);
      output.close();

      output.open("labels_reaction_graph_threshold2.txt");
      myReactionGraph->print_labels_reaction_graph(output,2);
      output.close();

      output.open("labels_reaction_graph_threshold1.txt");
      myReactionGraph->print_labels_reaction_graph(output,1);
      output.close();

      output.open("labels_reaction_graph_threshold4.txt");
      myReactionGraph->print_labels_reaction_graph(output,4);
      output.close();


      output.open("labels_reaction_graph_all_tresholds.txt");
      myReactionGraph->print_colors(output);
      output.close();
      exit(0);
    }

  if (mfinder)
    {
      /*Note that Bipartite_Graph already has a method called "print_edges" which does not actually print edges in mfinder format*/
      /*Note that we should use the write path, to specify the place where we want to write the file*/
      std::ofstream output;
      Reaction_Graph *myReactionGraph;
      myReactionGraph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      output.open("reaction_graph.txt");
      myReactionGraph->print_edges(output);
      output.close();
      cout<<"The graph has been stored in reaction_graph.txt"<<endl;
      output.open("labels.txt");
      myReactionGraph->print_colors(output);
      output.close();
      cout<<"The labels have been stored in labels.txt"<<endl;
      delete myReactionGraph;
      exit(0);
    }

  /************************INFERENCE MODE*******************************/

  /*if no enzymes are specified, then we are doing inference*/
  if (enzymes[0] == '\0')
    {

      /*if a motifs_file is specified, then you search for these motifs*/
      if (motifs_file[0] != '\0')
	refine_motifs(myData,whole_reaction_graph,simulations);

      /*if an enzymes_file is specified, then you infer all the motifs of given size that can be formed with this list*/
      if (enzymes_file[0] != '\0')
	motif_inference_from_enzymes(myData,whole_reaction_graph,motif_size,simulations);

      /*if you specify your own graph, then the inference is performed on it*/
      if (graph_file[0] != '\0' && labels_file[0] != '\0')
	{
	  //Graph *random_graph = new Graph(graph_file, labels_file, myData->colours_str, myData->nb_colours);

	  //Graph *users_graph = new Graph(graph_file, labels_file);
	  motif_inference(users_graph,motif_size,randomized_graphs);
	  //motif_inference(myData,random_graph,sigma_E,motif_size);
	  delete users_graph;
	  exit(0);
	}
      
      /*We want to investigate the permutability of a metabolic network*/
      if (permutability)
	{
	  std::ofstream output;
	  output.open("permutations3");
	  perm=new int[3];

	  /*Real Network*/
	  for (int j=0;j<3;j++)
	    perm[j]=0;
	  motif_inference(myData,whole_reaction_graph,sigma_E,motif_size,simulations);
	  // cerr<<"after"<<endl;
	  /*Print the permutability matrix*/
	  cout<<"For the real network the permutability is"<<endl;
	  for (int j=0;j<3;j++)
	    cout<<perm[j]<<" ";
	  cout<<endl;

	  for (int i=0;i<nb_simulations;i++)
	    {
	      /*Initialisation*/
	      for (int j=0;j<3;j++)
		perm[j]=0;
	      // cerr<<"before"<<endl;
	      motif_inference(myData,simulations[i],sigma_E,motif_size,simulations);
	      // cerr<<"after"<<endl;
	      /*Print the permutability matrix*/
	      for (int j=0;j<3;j++)
		output<<perm[j]<<" ";
	      output<<endl;
	    }
	  output.close();
	  exit(0);
	}

      /*otherwise, it is performed on the real network, given as data_file*/
      else
	{

	  cout<<"inferring motifs "<<endl;
	  /*the bipartite graph is missing for the moment*/
	  motif_inference(myData,whole_reaction_graph,sigma_E,motif_size,simulations);
	  
	  if (verbose==2)
	    {
	      cout<<"sorting the solutions"<<endl;
	      sort_by_block();
	    }
	}
    }


  /*****************************SEARCH MODE**********************************/

  /*otherwise, we are doing search*/
  else
    {

      /*********************SYNTHETIC DATA************************************/
      /*if you specify your own graph, then the search is performed on it*/
      if (graph_file[0] != '\0' && labels_file[0] != '\0')
	{
	  /*parsing enzymes into motif*/
	  /*maybe we should use another name since motif may not
	    necessarily represent enzymes*/

	  /*we assume that colours are separated by white spaces*/	
	  motif_size=count_char(enzymes,' '); 
	  //cout<<"motif size is "<<motif_size<<endl;
	  int *motif=new int[motif_size];
	  split(enzymes,motif,motif_size);

	  Graph *filtered_graph = new Graph(users_graph,motif,motif_size);
	  int nb_solutions=0;
	  list<int *> *occurrences = new list<int *>;
	  filtered_graph->enumerate(occurrences,motif_size,motif);
	  nb_solutions=occurrences->size();

	  // cout<<"Number of solutions: "<<nb_solutions<<endl;

	  if (topology==0)
	    {
	      /*group by overlap*/
	      int nb_clumps=get_nb_clumps(occurrences,motif_size);
	      float p,c;
	      calculate_p_c_value(randomized_graphs,motif,motif_size,nb_solutions,nb_clumps,&p,&c);

	      /*Printing results to standard output*/
	      cout<<"Colours: "<<enzymes<<endl;
	      cout<<"Number of occurrences: "<<nb_solutions<<endl;
	      cout<<"p-value: "<<p<<endl;
	      cout<<"Number of clumps: "<<nb_clumps<<endl;
	      cout<<"p-value (clumps): "<<c<<endl;

	      /*We need to create occurrences because the ids of the nodes are not the same in the original graph and in the filtered graph*/
	      cout<<"List of occurrences:"<<endl;
	      Occurrence_Group *all_occurrences=new Occurrence_Group;
	      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end();it++)
		all_occurrences->add(new Occurrence(*it,motif_size,filtered_graph->get_parent_nodes()));
	      
	      all_occurrences->print_occurrences();

	      //There is a mistake when outputing the list of occurrences, I output the node ids of the filtered graph
	      // cout<<"List of occurrences:"<<endl;
// 	      print_list_tab(occurrences->begin(),occurrences->end(),motif_size);

	      /*Printing results to file*/
	      /*Should we print occurrences ?*/ 
	      
	      std::ofstream output;
	      if (append==1)
		output.open(results_file,ios_base::app);
	      else
		output.open(results_file);
	      //fprint_nb_solutions(enzymes, nb_solutions,output);
	      output<<enzymes<<nb_solutions<<" "<<p<<" "<<nb_clumps<<" "<<c<<endl;
	      output.close();

	    }

	  if (topology==1)

	    {
	      /*group by topology*/
	      /*create occurrences*/
	      Occurrence_Group *all_occurrences=new Occurrence_Group;
	      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end();it++)
		all_occurrences->add(new Occurrence(*it,motif_size,filtered_graph->get_parent_nodes()));
	      
	      all_occurrences->set_adj_matrix(users_graph);
	      all_occurrences->compute_canonical_labelling(order);
	      list<Occurrence_Group *> *topology_occurrences=new list<Occurrence_Group *>;
	      all_occurrences->group_by_topology(topology_occurrences);
	      
	      // cout<<"nb of groups is "<<topology_occurrences->size()<<endl;
	      cout<<"Colours: "<<enzymes<<endl;
	      cout<<"Number of groups of topology is : "<<topology_occurrences->size()<<endl;
	      cout<<endl;
	      for (list<Occurrence_Group *>::iterator it=topology_occurrences->begin(); it!=topology_occurrences->end();it++)
		(*it)->print();
	      /*we should delete occurrences after printing*/

	      /*Printing results to file*/
	      std::ofstream output;
	      if (append==1)
		output.open(results_file,ios_base::app);
	      else
		output.open(results_file);
	      //fprint_nb_solutions(enzymes, nb_solutions,output);
	      output<<"Colours: "<<enzymes<<endl;
	      output<<"Number of groups of topology is : "<<topology_occurrences->size()<<endl;
	      output<<endl;
	      for (list<Occurrence_Group *>::iterator it=topology_occurrences->begin(); it!=topology_occurrences->end();it++)
		(*it)->print(output);
	      // output<<enzymes<<nb_solutions<<" "<<topology_occurrences->size()<<endl;
	      // output<<enzymes<<nb_solutions<<" "<<p<<" "<<nb_clumps<<" "<<c<<endl;
	      output.close();

	    }
	      
	  
	        
	  delete filtered_graph;
	  delete users_graph;
	  delete[] motif;
	  /*clean list of occurrences */
	  //cerr<<"before cleaning occurrences"<<endl;
	  for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	    {
	      delete[] (*it);
	      it=occurrences->erase(it);
 	      it--;
	    }
	  delete occurrences;
	  // cerr<<"before cleaning occurrences"<<endl;
	  for (int k=0;k<nb_simulations;k++)
	    delete randomized_graphs[k];
	  delete[] randomized_graphs;
	  if (append)
	    cout<<"Results have been appended to file : "<<results_file<<endl;
	  else
	    cout<<"Results have been printed to file : "<<results_file<<endl;

	  exit(0);
	}
      
      
      /*******************BIOLOGICAL DATA************************************/
      
      /*creation of motif*/
      Metabolic_Motif *myMotif = new Metabolic_Motif(myData, local_gap, global_gap, quorum, sigma_E, enzymes, input_compound, output_compound, results_file);
      
      
      /*motif_search*/
      
      /*STEP 1: build the filtered graph (graph containing nodes coloured with the same colours from the motif)*/
      
      /*new way, first build the whole graph and then filter it using the colours you need.*/
      /*note that in this point, colours have already been assigned to
	reactions (this has been done in the Data constructor*/
      //Reaction_Graph *whole_reaction_graph = new Reaction_Graph(myData->tab_reactions, myData->nb_reactions);
      /*filter the graph using the colours of the motif*/
      Reaction_Graph *filtered_graph = new Reaction_Graph(whole_reaction_graph, myMotif->get_colours(), myMotif->size());
      
      /*STEP 2: enumerate solutions inside the filtered graph*/
      Solution_Group *all_solutions = new  Solution_Group();
      filtered_graph->enumerate(all_solutions, myMotif);
      
      /*old way: do not use the whole graph but build a filtered version of it*/
        // int nb_reactions=0;
//         Reaction **tab_reactions=NULL;
//         tab_reactions = select_reactions(tab_reactions, &nb_reactions, myData, myMotif);
//         Reaction_Graph *myReactionGraph = new Reaction_Graph(tab_reactions, nb_reactions);
      
//         /*STEP 2: enumerate solutions inside the filtered graph*/
//         Solution_Group *all_solutions = new  Solution_Group();
//         myReactionGraph->enumerate(all_solutions, myMotif);
      
//         /*free memory for tab_reactions*/
//         unselect_reactions(tab_reactions,nb_reactions);
      
      //cerr<<"avant p-value"<<endl;	 
     
      /*STEP 3: do the same job on random graphs*/
      float p=calculate_p_value(simulations,myMotif->get_colours(), myMotif->size(),all_solutions->size());


      /*STEP STAT: store results for Stphane Robin*/
      //fprint_stats_for_count(simulations,myMotif,myData->tab_reactions);
      
      //cerr<<"aprs p-value"<<endl;

      /***********End of calculation, print results***************/
      /*short code for making good guesses on the number of simulations*/
      if (p_only)
	{
	  std::ofstream output;
	  output.open(results_file,ios::app);
	  output<<p<<endl;
	  output.close();
	  exit(0);
	}
      
      /*print results of STEP 1 (useful for debugging)*/
      /*This step does not work anymore - you need to recode the draw method*/
      if (mode == 0)
	{
	  filtered_graph->draw();
	}
      
      /*print results of STEP 2*/
      if (mode==1 || mode==2 || mode==3)
	{
	  /*print solutions in standard output (no grouping)*/
	  cout<<"Number of solutions: "<<all_solutions->size()<<endl;
	  cout<<"p-value: ";
	  print_p_value(p,cout,digits);
	  //cout<<setw(1)<<p<<endl;
	  cout<<endl;
	  cout<<"Confidence Interval at 95% on p-value is:";
	  float error = 1.96*sqrt(p*(1-p)/nb_simulations);
	  float approx_error = (float)((int)(pow((float)10,(float)digits)*error))/(float)pow((float)10,(float)digits);
	  float approx_p = (float)((int)(pow((float)10,(float)digits)*p))/(float)pow((float)10,(float)digits);
	  if (nb_simulations*p<5 || nb_simulations*(1-p)<5)
	    cout<<" Not enough simulations"<<endl;
	  else
	    cout<< "["<<approx_p-approx_error<<";"<<approx_p+approx_error<<"]"<<endl;
	  //cout<<endl;

	  cout<<"Confidence Interval at 99% on p-value is:";
	  error = 2.64*sqrt(p*(1-p)/nb_simulations);
	  approx_error = (float)((int)(pow((float)10,(float)digits)*error))/(float)pow((float)10,(float)digits);
	  // float approx_p = (float)((int)(pow((float)10,(float)digits)*p))/(float)pow((float)10,(float)digits);
	  if (nb_simulations*p<5 || nb_simulations*(1-p)<5)
	    cout<<" Not enough simulations"<<endl;
	  else
	    cout<< "["<<approx_p-approx_error<<";"<<approx_p+approx_error<<"]"<<endl;
	  cout<<endl;

	  cout<<"The whole list of solutions is:"<<endl;
	  all_solutions->print(verbose);
	  cout<<endl;
	  cout<<endl;
	  
	  /*print solutions in a file (no grouping)*/
	  if (mode==1)
	    {
	      std::ofstream output;
	      output.open(results_file);
	      output<<"!This file is structured as follows:"<<endl;
	      output<<"!The first line gives the number of occurrences of a motif and its p-value, separated by tab"<<endl;
	      output<<"!Every line is a solution of the search for motif "<<enzymes<<endl;
	      output<<"!Every solution is a list of reaction IDs"<<endl;
	      if (verbose>0)
		{
		  output<<"!Moreover, EC numbers of each reaction are indicated between brackets"<<endl;
		  output<<"!Finally, in the case where all reactions of an occurrence belong to the same pathway, the name of this pathway is indicated at the end of the line"<<endl;
		}
	      output<<all_solutions->size()<<"\t"<<p<<endl;
	      all_solutions->print(output,verbose);
	      output.close();
	    }
	  
	  /*FILTERING OF SOLUTIONS*/
	  /*if I only want to get linear occurrences : occurrences with n-1 inner compounds (i. e. linear in the bipartite graph)*/
	  //  if (topology==0)
	  // 	all_solutions->withdraw_non_linear_solutions();
	  
	  /*filter with compounds*/
	  if (input_compound[0] != '\0' && output_compound[0] != '\0')
	    {
	      Compound *input = myMotif->get_input();
	      Compound *output = myMotif->get_output();
	      all_solutions->filter_by_compounds(input, output, sigma_C);
	    }
	  
	  /*keep only occurrences that are intra-pathway*/
	  // if (only_intra_pathway)
// 	    {
// 	      all_solutions->filter_by_intra_pathway();
// 	    }



	  /*GROUPING OF SOLUTIONS*/
	  /*overlapping solutions are grouped together (simple link procedure)*/
	  if (mode==2)
	    {
	      cout<<"Grouping 1: overlapping solutions are grouped together (simple-link clustering)"<<endl;
	      
	      // cout<<"a group is a connected component in the graph (an occurrence is put in a group if it has at least one reaction in common with at least one occurrence from this group)"<<endl;
	      list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
	      all_solutions->group_by_reaction(disjoint_solutions);
	      
	      /*calculate the p-value corresponding to the over-representation of the number of connected components*/
	      int nb_connected_components = (int)disjoint_solutions->size();  
	      float c=calculate_c_value(simulations,myMotif,all_solutions->size(),nb_connected_components,myData->tab_reactions);
	      
	      cout<<"Number of connected components: "<<nb_connected_components<<endl;
	      //cout<<"p-value: "<<c<<endl;
	      cout<<"p-value: ";
	      print_p_value(c,cout,digits);
	      //print_confidence_interval(c,cout,digits);
	      cout<<endl;
	      
	      /*print groups to standard output*/
	      list<Solution_Group *>::iterator iter= disjoint_solutions->begin();
	      while (iter != disjoint_solutions->end())
		{
		  (*iter)->print(verbose);
		  cout<<endl;
		  iter++;
		}
	      
	      /*print groups of solutions to file*/
	      std::ofstream output;
	      output.open(results_file);
	      output<<"!This file is structured as follows:"<<endl;
	      output<<"!The first line gives the number of occurrences of a motif and its p-value, separated by tab"<<endl;
	      output<<"!Every line is a solution of the search for motif "<<enzymes<<endl;
	      output<<"!Every solution is a list of reaction IDs"<<endl;
	      output<<"!Solutions are organised into groups (blocks of lines separated by empty lines) and a group is a collection of solutions that overlap"<<endl;
	      if (verbose>0)
		{
		  output<<"!Moreover, EC numbers of each reaction are indicated between brackets"<<endl;
		  output<<"!Finally, in the case where all reactions of an occurrence belong to the same pathway, the name of this pathway is indicated at the end of the line"<<endl;
		}
	      
	      output<<all_solutions->size()<<"\t"<<p<<endl;
	      iter= disjoint_solutions->begin();
	      while (iter != disjoint_solutions->end())
		{
		  (*iter)->print(output,verbose);
		  output<<endl;
		  iter++;
		}
	      output.close();
	      
	      /*delete everything*/
	      for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
		delete(*it);
	      delete(disjoint_solutions);
	    }
	  
	  if (mode==3)
	    {
	      // cout<<""<<endl;	   
	      
	      Bipartite_Graph *myBipartiteGraph;
	      myBipartiteGraph = new Bipartite_Graph(myData->tab_reactions, myData->nb_reactions, myData->tab_compounds, myData->nb_compounds);
	      all_solutions->set_bipartite_graph(myBipartiteGraph, myMotif->get_colours());
	      
	      all_solutions->compute_canonical_labelling(order);
	      list<Solution_Group *> *topology_solutions=new list<Solution_Group *>;
	      all_solutions->group_by_topology(topology_solutions);
	               
	      /*assessing over-representation of topological motifs*/
	      
	      int nb_observed_topologies = (int)topology_solutions->size();
	      
	      int observed_topologies_counts[nb_observed_topologies];
	      
	      
	      long int observed_topologies1[nb_observed_topologies];
	      long int observed_topologies2[nb_observed_topologies];
	      
	      /*fill these arrays*/
	      int g=0;
	      for (list<Solution_Group *>::iterator iter = topology_solutions->begin(); iter != topology_solutions->end(); iter++)
		{
		  observed_topologies1[g] = (*iter)->get_solutions()->front()->get_topo1();
		  observed_topologies2[g] = (*iter)->get_solutions()->front()->get_topo2();
		  observed_topologies_counts[g] = (*iter)->size();
		  g++;
		}
	      
	      /*array storing the p-value for each sub-motif i.e. defined with a given topology*/
	      float *p_s = new float[nb_observed_topologies]; 
	      
	      // time_t *tim=NULL; //to be able to say how long it takes
	      
	      //cout<<time(tim)<<endl;
	      cout<<"estimating p-values for each of the topological motifs, this may take some time"<<endl;
	      cout<<endl;
	      calculate_p_values(simulations, myMotif, nb_observed_topologies, observed_topologies_counts, observed_topologies1, observed_topologies2, p_s, myData->tab_reactions, myBipartiteGraph);
	      
	      //cout<<time(tim)<<endl;
	      
	      // cout<<"the p_values are "<<endl;
// 	      print_tab(p_s,nb_observed_topologies);
	      

	      /*print groups of solutions to standard output*/
	      cout<<"Grouping 2: occurrences are grouped together if they have the same topology"<<endl;
	      list<Solution_Group *>::iterator iter= topology_solutions->begin();
	      int d=0;
	      while (iter != topology_solutions->end())
		{
		  (*iter)->print(verbose);
		  cout<<"topology matrix of the group"<<endl;
		  (*iter)->print_bipartite_graph();
		  if (order)
		    {
		      cout<<"order of colours w.r.t. topology"<<endl;
		      (*iter)->print_colour_order();
		    }
		  cout<<"p-value of the group: "<<p_s[d]<<endl;
		  // (*iter)->print_bipartite_graphs();
		  cout<<endl;
		  iter++;
		  d++;
		}

	      /*print groups of solutions to file*/
	      std::ofstream output;
	      output.open(results_file);
	      output<<"!This file is structured as follows:"<<endl;
	      output<<"!The first line gives the number of occurrences of a motif and its p-value, separated by tab"<<endl;
	      output<<"!Every line is a solution of the search for motif "<<enzymes<<endl;
	      output<<"!Every solution is a list of reaction IDs"<<endl;
	      output<<"!Solutions are organised into groups (blocks of lines separated by empty lines) and a group is a collection of solutions that have the same topology"<<endl;
	      if (verbose>0)
		{
		  output<<"!Moreover, EC numbers of each reaction are indicated between brackets"<<endl;
		  output<<"!Finally, in the case where all reactions of an occurrence belong to the same pathway, the name of this pathway is indicated at the end of the line"<<endl;
		}
	      
	      output<<all_solutions->size()<<"\t"<<p<<endl;
	      iter= topology_solutions->begin();
	      while (iter != topology_solutions->end())
		{
		  (*iter)->print(output,verbose);
		  output<<endl;
		  iter++;
		}
	      output.close();

	      /*delete everything*/
	      delete[] p_s;
	      for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
		delete(*it);
	      delete(topology_solutions);
	      delete myBipartiteGraph;
	      
	      
	      // all_solutions->withdraw_non_linear_solutions(); // you got to withdraw non linear solutions before grouping them (for the moment)
	      // 	  list<Solution_Group *> *ordered_solutions=new list<Solution_Group *>;
	      // 	  all_solutions->group_by_order(ordered_solutions);
	      // 	  //cout<<"aprs group_by_order"<<endl;
	      // 	  list<Solution_Group *>::iterator iter= ordered_solutions->begin();
	      // 	  while (iter != ordered_solutions->end())
	      // 	    {
	      // 	      (*iter)->print(verbose);
	      // 	      cout<<endl;
	      // 	      iter++;
	      // 	    }
	      
	      // 	  /*delete everything*/
	      // 	  for(list<Solution_Group *>::iterator it=ordered_solutions->begin();it!=ordered_solutions->end();it++)
	      // 	    delete(*it);
	      // 	  delete(ordered_solutions);
	    }
	  
	  /*delete all the solutions and the group that contains them*/
	  all_solutions->delete_solutions();
	  delete(all_solutions);
	  
	}
      delete(myMotif);
      delete(filtered_graph);
      /*end of search loop*/
    }
  delete(myData);
  delete whole_reaction_graph;
  for (int k=0;k<nb_simulations;k++)
    delete simulations[k];
  delete[] simulations;

  cout<<"Results are printed in file : "<<results_file<<endl;
}

Io::~Io()
{
  delete[](enzymes);
  delete[](input_compound);
  delete[](output_compound);
  delete[](graph_file);
  delete[](labels_file);
  delete[](results_file);
  delete[](data_file);
  delete[](write_path);
  delete[](motifs_file);
  delete[](enzymes_file);
}


void Io::printHelpShort()
{
  cout<<"INFERENCE MODE"<<endl;
  cout<<"Usage : mo-mo-motus -d <Network input file name> -n <motif size> -ns <no. of random networks> [-r <output file name>] [more flags]"<<endl;
  cout<<endl;
  cout<<"SEARCH MODE"<<endl;
  cout<<"Usage : mo-mo-motus -d <Network input file name> -e <colors> -ns <no. of random networks> [-r <output file name>] [more flags]"<<endl;
  cout<<endl;
  // cout<<"STATISTICS MODE"<<endl;
//   cout<<"Usage : mo-mo-motus -d <Network input file name> -st 1 [-r <output file name>] [more flags]"<<endl;
//   cout<<endl;
//   cout<<"Advanced help, use mo-mo-motus -h"<<endl;  

  exit(0);
}

void Io::printHelp() {
  cout<<"Usage : mo-mo-motus -d <Network input file name> -n <motif size> -ns <no. of random networks> [-r <output file name>] [more flags]"<<endl;



  cout <<"You can use this program in three modes: SEARCH, INFERENCE and STATISTICS"<<endl;
  cout<<"Some options are specific to a given mode"<<endl;
  cout<<"GENERAL OPTIONS"<<endl;
  //cout << "Command-line options.\n";
  cout << "If not specified, default value will be used\n";
  cout << "    help      -h : displays this help screen\n";
  cout << "    data_file -d: name of the file containing the data needed to build the network" <<endl;
  cout << "                      Default: data/primCpdsSmmReactionsCompounds.col\n";
  cout << "    ubiquitous      -ub : number of frequent compounds you want to remove from the dataset" <<endl;
  cout << "    graph_file -g: name of the file containing the topology of your own graph" <<endl;
  cout << "                      Default: graph_file\n";
  cout << "    labels_file -l: name of the file containing the labels of your own graph" <<endl;
  cout << "                      Default: labels_file\n";
  cout << "    results_file -r: name of the file in which the results of the query are stored" <<endl;
  cout << "                      Default: results/motus.res\n";
  cout<<endl;
  cout<<"SEARCH OPTIONS:"<<endl;
  cout << "    enzymes  -e : EC numbers of the enzymes you want to find"<<endl;
  // cout << "    input       -i :  name of the input compound"<<endl;
//   cout << "    output    -o : name of the output compound"<<endl;
//   cout << "    sigma_C   -sc : threshold used to compare two compounds "<<endl;
//   cout << "    the comparison is done between chemical fingerprints : every compound is seen as a set of chemical fragments (-OH + -COOH + NH2 +...) summarized in one array of integers : the fingerprint"<<endl;
//   cout << "    the threshold indicates the percentage similarity you want between those two fingerprints"<<endl;
//   cout<< "the score function used "<<endl;
//   cout << "                      Default: 100 (identity).\n";
  cout << "    simulations  -ns : number of random graphs to generate to assess over-representation of motif"<<endl;
  cout << "                      Default: 100\n";
  cout << "    mode     -m : quantity of information computed"<<endl;
  //cout << "      0: perform only first step of computation (filtering phase) -- requires neato and gv to be installed"<<endl;
  cout << "      1: whole list of occurrences of your motif"<<endl;
  cout << "      2: group by overlap: split your solutions into disjoint (non-overlapping) groups"<<endl;
  cout << "      3: group by topology (only for metabolic bipartite graphs): group solutions sharing the same topology in the bipartite graph"<<endl;
  //cout << "      3: group by topology: group together solutions which have the EC numbers appearing in the same order"<<endl;
  cout << "                      Default: 2\n";
//   cout << "    global gap -gg : size of global gap accepted\n";
//   cout<< "     this is useful in case you want to detect occurrences larger than your motif"<<endl;
//   cout << "                      Default: 0.\n";
//   cout << "    local gap  -lg : size of local gap accepted \n";
//   cout << "    in addition to the global gap, here comes the local gap, useful to control the position of the gaps in your occurrences"<<endl;
//   cout<< "     It specifies the number of consecutive unmatched enzymes you accept in your occurrence"<<endl;
//   cout << "                      Default: 0.\n";
//   cout << "    quorum  -q : percentage of enzymes from the motif to be found in the occurrences \n";
//   cout<< "     This is useful if you want to detect occurrences smaller than your motif"<<endl;
//   cout << "                      Default: 100.\n";
//   cout << "    sigma_E   -s : threshold used to compare two enzymes "<<endl;
//   cout << "    the comparison is done between EC numbers"<<endl;
//   cout << "    example : 1.1.1.2 and 1.1.1.3 are considered similar when the threshold is set to 3 "<<endl;
//   cout << "            they are considered non similar if the threshold is set to 4 (the 4th number is different)"<<endl;
//   cout << "                      Default: 4.\n";
  cout << "    verbose   -v :  quantity of information printed"<<endl;
  cout << "      0: set of reaction IDs"<<endl;
  cout << "      1: set of reaction IDs + EC numbers of the enzymes that catalyse these reactions"<<endl;
  cout << "      2: set of reaction IDs + EC numbers of the enzymes that catalyse these reactions + IDs of the compounds that link the reactions"<<endl;
  // cout << "    topology  -t :  set this to 0 if you only want linear occurrences of your motif (linear means linear in the bipartite graph)"<<endl;
//   cout << "                      Default: 1.\n";
  //cout << "    set this option to 2 if you want to split your solutions into disjoint groups, this can be useful to analyse your results"<<endl;
  //cout << "                      Default: 2.\n";
  cout<<endl;

    
  cout<<"INFERENCE OPTIONS:"<<endl;
  cout << "     motif_size -n: size of the motifs you want to infer" <<endl;
  cout << "                      Default: 3\n";
  cout << "     motif_threshold -s: threshold considered for EC number comparisons"<<endl;
  cout << "                      Default: 3\n";
  cout << "     simulations  -ns : number of random graphs to generate to assess over-representation of motif"<<endl;
  cout << "                      Default: 100\n";
  cout << "     mode     -m : calculation of properties on the detected motifs "<<endl;
  cout << "        1: no properties are calculated"<<endl;
  cout << "        2: properties are calculated (overlap of occurrences and "<<endl;
  cout<<  "distribution of occurrences among pathways)"<<endl;
  cout << "        3: properties are used to filter the solutions"<<endl;
  cout << "                      Default: 2\n";
  cout<<  "     verbose      -v :  quantity of information printed"<<endl;
  cout<<  "        1: print motif + number of occurrences + p-value"<<endl;
  cout << "        2: print motif + properties + list of occurrences"<<endl;
  cout<<  "        3: print motif + list of occurrences"<<endl;
  cout<<  "        4: print motif + some properties"<<endl;
  cout << "                      Default: 2\n";
  //cout<<  "        5: print motif + properties (including p-value)"<<endl;
  cout<<endl;

  cout<<endl;
  cout<<"STAT OPTIONS:"<<endl;
  cout<<"      stat -st:  1 : print basic statistics on dataset"<<endl;
  exit(0);
}


void Io::printSettings() {
  
  //cout<<endl;
  //cout << "Your favorite program was launched with the following settings:\n";

  
  if (enzymes[0]!='\0')
    {
      cout<<"SEARCH MODE"<<endl;
      cout << "    Data_file: " <<  data_file << "\n";
      //cout << "    Mode : " <<mode <<endl;
      //cout << "    Local gap: " <<  local_gap << "\n";
      //cout << "    Global gap: " <<  global_gap << "\n";
      //cout << "    Quorum: " <<  quorum << "%\n";
      cout << "    Colours: " <<  enzymes << endl;
      cout<<  "    Simulations: "<<nb_simulations<<endl;
      cout << "    Verbose: " <<  verbose << endl;
      cout << "    Topology: " <<  topology << endl;
      
      //cout << "    Input Compound:  "<<input_compound <<"\n";
      //cout << "    Output Compound:  "<<output_compound <<"\n";
      //cout << "    Sigma_C: " <<  sigma_C << "%\n";
    }
  else if (stat || alberto_marchetti || fabien_jourdan || stephane_robin || ec_list || etienne_birmele)
    {
      cout<<"STAT MODE"<<endl;
      cout << "    Data_file: " <<  data_file << "\n";
      cout << "    Results_file: " <<  results_file << "\n";
    }
  else
    {
      cout<<"INFERENCE MODE"<<endl;
      cout << "    Data_file: " <<  data_file << "\n";
      cout << "    Motif Size: " <<  motif_size << "\n";
      cout << "    Sigma_E: " <<  sigma_E << "\n";
      cout<<  "    Simulations: "<<nb_simulations<<endl;
      // cout << "    Graph_file: " <<  graph_file << "\n";
//       cout << "    Labels_file: " <<  labels_file << "\n";
      cout << "    Results_file: " <<  results_file << "\n";
    }


  if (graph_file[0]!='\0')
    {
      if (labels_file[0]!='\0')
	{
	  cout << "    Graph_file: " <<  graph_file << "\n";
	  cout << "    Labels_file: " <<  labels_file << "\n";
	}
      else
	{
	  cerr << "ERROR: You should specify a labels file\n";
	  exit(0);
	}
    }

  cout<<endl;
}



/**
 * parses string passed as argument and correspondingly sets the class variables.
 * If specified arguments aren't complete, default values are set.
 */
void Io::parse(int argc, char** argv) 
{
  // cerr<<"before parsing"<<endl;
//   cerr<<argv[0]<<endl;
//   cerr<<argc<<endl;

  if (argc<=1) 
    {
      //cerr << "ERROR: no command-line arguments were given\n";
      //cout<<endl;
      printHelpShort();
    }
  for (int i=1; i<argc; i++) 
    {
      /**
       * strcmp compares two strings and returns:
       * <0 if string1 is less than string2
       *  0 if string1 is the same as string2
       * >0 string1 is greater than string2
       *
       * So.... 
       **/
      if       (!strcmp("-h",    argv[i])) 
	{
	  printHelp();
	}
      else if (!strcmp("-e",    argv[i]))
	{
	  while (argv[i+1][0] != '-')
	    {
	      strcat(enzymes, argv[++i]);
	      strcat(enzymes, " ");
	      /* in case -e is the last argument*/
	      if (i+1==argc)
		break;
	    }
	}
      else if (!strcmp("-o",    argv[i]))
	{
	  strcpy(output_compound, argv[++i]);
	}
      else if (!strcmp("-i",    argv[i]))
	{
	  strcpy(input_compound, argv[++i]);
	}
      else if  (!strcmp("-m",    argv[i]))   mode    = atoi(argv[++i]);
      else if  (!strcmp("-gg",    argv[i]))   global_gap    = atoi(argv[++i]);
      else if  (!strcmp("-lg",    argv[i]))   local_gap  = atoi(argv[++i]); 
      else if  (!strcmp("-q",    argv[i]))   quorum = atoi(argv[++i]);
      else if  (!strcmp("-s",    argv[i]))   sigma_E = atoi(argv[++i]);
      else if  (!strcmp("-sc",    argv[i]))   sigma_C    = atoi(argv[++i]);    
      else if  (!strcmp("-v",    argv[i]))   verbose     = atoi(argv[++i]);    
      else if  (!strcmp("-t",    argv[i]))   topology     = atoi(argv[++i]);
      else if  (!strcmp("--order",    argv[i]))   order     = atoi(argv[++i]);
      else if  (!strcmp("-n",    argv[i]))   motif_size     = atoi(argv[++i]);
      else if  (!strcmp("-ns",    argv[i]))   nb_simulations = atoi(argv[++i]);
      else if  (!strcmp("-d",    argv[i]))   strcpy(data_file, argv[++i]);
      else if  (!strcmp("-g",    argv[i]))   strcpy(graph_file, argv[++i]);
      else if  (!strcmp("-l",    argv[i]))   strcpy(labels_file, argv[++i]);
      else if  (!strcmp("-r",    argv[i]))   strcpy(results_file, argv[++i]);
      else if  (!strcmp("--append",    argv[i]))   append=atoi(argv[++i]);
      else if  (!strcmp("--motifs",    argv[i]))   strcpy(motifs_file, argv[++i]);
      else if  (!strcmp("--enzymes",    argv[i]))   strcpy(enzymes_file, argv[++i]);
      else if  (!strcmp("-wp",    argv[i]))   strcpy(write_path, argv[++i]);

      else if  (!strcmp("-fj",    argv[i]))   fabien_jourdan = atoi(argv[++i]);
      else if  (!strcmp("-sr",    argv[i]))   stephane_robin = atoi(argv[++i]);
      else if  (!strcmp("-am",    argv[i]))   alberto_marchetti = atoi(argv[++i]);
      else if  (!strcmp("-eb",    argv[i]))   etienne_birmele = atoi(argv[++i]);
      else if  (!strcmp("--mfinder",    argv[i]))   mfinder = atoi(argv[++i]);
      else if  (!strcmp("--sif",    argv[i]))   sif = atoi(argv[++i]);
      else if  (!strcmp("-st",    argv[i]))   {stat = atoi(argv[++i]);}
      else if  (!strcmp("-ec",    argv[i]))   ec_list = atoi(argv[++i]);
      else if  (!strcmp("-ub",    argv[i]))   nb_frequent_compounds = atoi(argv[++i]);
      else if  (!strcmp("-p",    argv[i]))   p_only = atoi(argv[++i]);
      else if  (!strcmp("-perm",    argv[i]))   permutability = atoi(argv[++i]);
      else if  (!strcmp("-nop",    argv[i]))   no_p_value = atoi(argv[++i]);

            else 
	{
	  cerr << argv[0] << ": unrecognized argument: " << argv[i] << "\n";
	}
    }

  //#warning default settings are not yet implemented
 //  cerr<<"hop"<<endl;  


  //if (verbose >0) {
  printSettings();
  //}
}


// //this is not very speedy, it adds a list traversal
// bool in_list5(Reaction * reac, list<Reaction *> *reac_list)
// {
//   list<Reaction *>::iterator iter = reac_list->begin();
//     while (iter != reac_list->end())
//       {
// 	if (reac == *iter)
// 	  return 1;
// 	iter++;
//       }
//   return 0;
// }


/**
 *Search motif in nb_simulations random graphs (say 1000) and compare
 *the number of occurrences to the observed number of occurrences. We
 *perform 2 unilateral tests, one for over-representation and one for
 *under-representation. We calculate the proportion of graphs for
 *which the number of occurrences is larger or equal to the number of
 *observed occurrences. If this proportion is very small, the motif is
 *over-represented, if it is very large, the motif is
 *under-represented. Since, we deal with discrete distributions,
 *ex-aequos are possible. We deal with them in a very simple way, they
 *account half for under and half for over-representations.
 *
 * @param simulations array of random graphs
 * @param myMotif motif to search for
 * @param nb_occurrences number of occurrences of the motif in the real graph
 * @return p-value
 */
float Io::calculate_p_value(Reaction_Graph **simulations, int *motif, int motif_size, int nb_occurrences)
{
  /*in case no random graphs are generated, we return -1 as p-value.*/
  if (nb_simulations==0)
    return -1.0;

  if (no_p_value==1)
    return -1.0;

  int *solutions = new int[nb_simulations];
  list<int *> *occurrences = new list<int *>;
  int nb_sup=0;
  int nb_inf=0;
  int nb_eq=0;

  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],motif, motif_size);
      filtered_graph->enumerate(occurrences,motif_size,motif);
      solutions[i]=occurrences->size();

      if (solutions[i]<nb_occurrences)
	nb_inf++;
      else if (solutions[i]>nb_occurrences)
	nb_sup++;
      else
 	nb_eq++;
     
      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;
    }
 
  //print_tab(solutions, nb_simulations);
  //cout<<endl;

  delete occurrences;
  delete[] solutions;

  /*calculate the p-value*/
  float p_more = ((float)nb_sup + ((float)nb_eq / 2)) / (float)nb_simulations;

  return p_more;

}


/**
 *Search motif in 1000 random graphs and compare the number of
 *occurrences to the observed number of occurrences. Also does the
 *same work for the number of connected components formed by these
 *occurrences. We perform 2 unilateral tests, one for
 *over-representation and one for under-representation. We calculate
 *the proportion of graphs for which the number of occurrences is
 *larger or equal to the number of observed occurrences. If this
 *proportion is very small, the motif is over-represented, if it is
 *very large, the motif is under-represented. Since, we deal with
 *discrete distributions, ex-aequos are possible. We deal with them in
 *a very simple way, they account half for under and half for
 *over-representations.
 *
 * @param simulations array of random graphs
 * @param myMotif motif to search for
 * @param nb_occurrences number of occurrences of the motif in the real graph
 * @param nb_connected_components number of connected components formed by the occurrences in the real graph
 * @return c-value p-value corresponding to the number of connected components
 */
float Io::calculate_c_value(Reaction_Graph **simulations,Metabolic_Motif *myMotif, int real_nb_occurrences, int real_nb_connected_components, Reaction **tab_reactions)
{
  if (nb_simulations==0)
    return -1.0;

  if (no_p_value==1)
    return -1.0;

  int *solutions = new int[nb_simulations];
  int *connected_components = new int[nb_simulations];
  list<int *> *occurrences = new list<int *>;
  int nb_sup=0;
  int nb_inf=0;
  int nb_eq=0;
  int c_nb_sup=0;
  int c_nb_inf=0;
  int c_nb_eq=0;

  
  /*additional structure needed to build solution objects to tell the
    constructor that we already have the good indices*/
  int *identity_array = new int[simulations[0]->get_nb_nodes()];
  for (int k=0;k<simulations[0]->get_nb_nodes();k++)
    identity_array[k]=k;


  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],myMotif->get_colours(), myMotif->size());
      
      /*get occurrences*/
      filtered_graph->enumerate(occurrences,myMotif->size(),myMotif->get_colours());
      
      // for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
// 	print_tab(*it,myMotif->size());cout<<endl;

      /*transform occurrences with integers corresponding to filtered
	reactions to integers corresponding to Reactions of the
	original complete graph*/
      int *parent_nodes= filtered_graph->get_parent_nodes();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	for (int j=0;j<myMotif->size();j++)
	  (*it)[j]=parent_nodes[(*it)[j]];


//       for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
// 	print_tab(*it,myMotif->size());cout<<endl;
      
      // /*creates a simple identity array useful to create Solution*/
//       parent_nodes = new int[myMotif->size()];
//       for (int k=0;k<myMotif->size();k++)
// 	parent_nodes[k]=k;


      /*create Solution objects with these occurrences*/
      //cout<<"dbut de cration de st ofbqmdj"<<endl;
      Solution_Group *group = new Solution_Group();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	group->add(new Solution(*it,myMotif->size(), tab_reactions, identity_array));
      //cout<<"fin cration des objets solution"<<endl;

      /*group the solutions into connected components*/
      list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
      group->group_by_reaction(disjoint_solutions);
      int nb_connected_components = (int)disjoint_solutions->size();      
      for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	delete(*it);
      delete disjoint_solutions;

      /*store obtained values for this random graph (#i)*/
      connected_components[i]=nb_connected_components;
      solutions[i]=occurrences->size();

      /*compare these values to the real values*/
      if (solutions[i]<real_nb_occurrences)
	nb_inf++;
      else if (solutions[i]>real_nb_occurrences)
	nb_sup++;
      else
 	nb_eq++;
     
      //cout<<"connected_components[i] "<<connected_components[i]<<endl;

      if (connected_components[i]<real_nb_connected_components)
	c_nb_inf++;
      else if (connected_components[i]>real_nb_connected_components)
	c_nb_sup++;
      else
 	c_nb_eq++;


      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;

      /*clean solutions from the last call*/
      group->delete_solutions();
      delete group;
    }
 

  // /*NEW VERSION USING A POISSON APPROXIMATION*/
  /*first calculate the lambda of the poisson approximation: this is estimated by the mean of the distribution*/
  // double mean_connected_components = mean(connected_components,nb_simulations);  
//   double variance_connected_components = variance(connected_components,nb_simulations,mean_connected_components); 

//   cout<<"mean = "<<mean_connected_components<<endl;
//   cout<<"variance = "<<variance_connected_components<<endl;

  // std::ofstream output;
//   output.open("connected_components");
//   for (int i=0;i<nb_simulations;i++)
//     output<<connected_components[i]<<endl;
//   output.close();

//   exit(0);

  // double ccdf_compute;
//   double cdf_compute;
//   double lambda=10;
//   double x_double=10;
//   cumpoi ( &x_double, &lambda, &cdf_compute, &ccdf_compute );

  delete occurrences;
  delete[] identity_array;
  delete[] solutions;
  delete[] connected_components;
  /*calculate the p-value*/
  //float p_more = ((float)nb_sup + ((float)nb_eq / 2)) / (float)nb_simulations;

  float c_more = ((float)c_nb_sup + ((float)c_nb_eq / 2)) / (float)nb_simulations;

  return c_more;
}


/**
 *Calculate p-values for each of the subgroups of occurrences sharing the same topology.
 *
 * @param simulations array of random graphs
 * @param myMotif motif to search for
 * @param nb_observed_topologies number of distinct topologies observed among the occurrences of the motif in the real graph
 * @param observed_topologies_counts counts for each of these topologies
 * @param observed_topologies1 first hex number characterising each of these topologies
 * @param observed_topologies2 second hex number characterising each of these topologies
 * @param p_s resulting p_values
 * @param tab_reactions array of Reaction * corresponding to the whole list of reactions of the dataset. Useful to make the correspondance between filtered and whole graph.
 * @param myBipartiteGraph whole bipartite graph, useful to group solutions by topologies
 * @return p-value
 */
int Io::calculate_p_values(Reaction_Graph **simulations, Metabolic_Motif * myMotif, int nb_observed_topologies, int *observed_topologies_counts, long int *observed_topologies1, long int *observed_topologies2, float *p_s, Reaction **tab_reactions, Bipartite_Graph *myBipartiteGraph)
{
  /*if there is no simulation, then no p-value is calculated*/
  if (nb_simulations==0)
    return -1.0;

  if (no_p_value==1)
    return -1.0;

  //int *solutions = new int[nb_simulations];
  //int *connected_components = new int[nb_simulations];
  list<int *> *occurrences = new list<int *>;
  // int nb_sup=0;
//   int nb_inf=0;
//   int nb_eq=0;
//   int c_nb_sup=0;
//   int c_nb_inf=0;
//   int c_nb_eq=0;

/*declaration of necessary structures to calculate p-values*/
  int nb_sup[nb_observed_topologies];
  int nb_inf[nb_observed_topologies];
  int nb_eq[nb_observed_topologies];

  int *simulated_topologies_counts = new int[nb_observed_topologies];

  /*initialisation*/
  for (int i=0;i<nb_observed_topologies;i++)
    {
      nb_sup[i]=0;
      nb_inf[i]=0;
      nb_eq[i]=0;
      simulated_topologies_counts[i]=0;
    }


  /*additional structure needed to build solution objects to tell the
    constructor that we already have the good indices*/
  int *identity_array = new int[simulations[0]->get_nb_nodes()];
  for (int k=0;k<simulations[0]->get_nb_nodes();k++)
    identity_array[k]=k;


  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],myMotif->get_colours(), myMotif->size());
      
      /*get occurrences*/
      filtered_graph->enumerate(occurrences,myMotif->size(),myMotif->get_colours());
      
      /*transform occurrences with integers corresponding to filtered
	reactions to integers corresponding to Reactions of the
	original complete graph*/
      int *parent_nodes= filtered_graph->get_parent_nodes();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	for (int j=0;j<myMotif->size();j++)
	  (*it)[j]=parent_nodes[(*it)[j]];

      /*create Solution objects with these occurrences*/
      //cout<<"dbut de cration de st ofbqmdj"<<endl;
      Solution_Group *group = new Solution_Group();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	group->add(new Solution(*it,myMotif->size(), tab_reactions, identity_array));

      /*group the solutions according to the topology*/
      /*first compute their canonical labelling*/
      group->set_bipartite_graph(myBipartiteGraph, myMotif->get_colours());
      //cout<<"before canonical labelling"<<endl;
      group->compute_canonical_labelling(order);
      //cout<<"after canonical labelling"<<endl;
      group->count_observed_topologies(observed_topologies1, observed_topologies2, nb_observed_topologies, simulated_topologies_counts);

      // print_tab(observed_topologies1, nb_observed_topologies);
//       print_tab(observed_topologies2, nb_observed_topologies);
//       print_tab(simulated_topologies_counts, nb_observed_topologies);
//       cout<<endl;
 


      /*compare the simulated values to the observed values*/
      for (int j=0;j<nb_observed_topologies;j++)
	{
	  if (simulated_topologies_counts[j] > observed_topologies_counts[j])
	    nb_sup[j]++;
	  else if (simulated_topologies_counts[j] == observed_topologies_counts[j])
	    nb_eq[j]++;
	}

      /*reinitialize simulated_topologies_count*/
      for (int i=0;i<nb_observed_topologies;i++)
         simulated_topologies_counts[i]=0;
   
      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;

      /*clean solutions from the last call*/
      group->delete_solutions();
      delete group;
    }
 

  delete occurrences;
  delete[] identity_array;
  delete[] simulated_topologies_counts;


  /*calculate the p-values*/
  for (int j=0;j<nb_observed_topologies;j++)
    p_s[j] = ((float)nb_sup[j] + ((float)nb_eq[j] / 2)) / (float)nb_simulations;
  
  /*the true return item is p_s*/
  return 1;

}


/**
 *Same as previous calculate_p_value but for general graphs
 *
 * @param simulations array of random graphs
 * @param myMotif motif to search for
 * @param nb_occurrences number of occurrences of the motif in the real graph
 * @return p-value
 */
float Io::calculate_p_value(Graph **simulations, int *motif, int motif_size, int nb_occurrences)
{
  /*in case no random graphs are generated, we return -1 as p-value.*/
  if (nb_simulations==0)
    return -1.0;

  if (no_p_value==1)
    return -1.0;

  int *solutions = new int[nb_simulations];
  list<int *> *occurrences = new list<int *>;
  int nb_sup=0;
  int nb_inf=0;
  int nb_eq=0;

  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],motif, motif_size);
      filtered_graph->enumerate(occurrences,motif_size,motif);
      solutions[i]=occurrences->size();
      
      if (solutions[i]<nb_occurrences)
	nb_inf++;
      else if (solutions[i]>nb_occurrences)
	nb_sup++;
      else
 	nb_eq++;

      

     
      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;
    }
 
  //print_tab(solutions, nb_simulations);
  //cout<<endl;

  delete occurrences;
  delete[] solutions;

  /*calculate the p-value*/
  float p_more = ((float)nb_sup + ((float)nb_eq / 2)) / (float)nb_simulations;

  return p_more;

}
/**
 *Same as previous calculate_p_c_value but for general graphs
 *
 * @param simulations array of random graphs
 * @param myMotif motif to search for
 * @param nb_occurrences number of occurrences of the motif in the real graph
 * @return p-value
 */

int Io::calculate_p_c_value(Graph **simulations, int *motif, int motif_size, int nb_occurrences, int real_nb_connected_components, float *p, float *c)
{
  /*in case no random graphs are generated, we return -1 as p-value.*/
  // if (nb_simulations==0)
//     {}    
//return -1.0;
 //  &p=-1.0;
//   &c=-1.0;
  if (no_p_value==1)
    return -1;

  int *solutions = new int[nb_simulations];
  int *connected_components = new int[nb_simulations];
  list<int *> *occurrences = new list<int *>;
  int nb_sup=0;
  int nb_inf=0;
  int nb_eq=0;
  int c_nb_sup=0;
  int c_nb_inf=0;
  int c_nb_eq=0;

  

  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],motif, motif_size);
      filtered_graph->enumerate(occurrences,motif_size,motif);
      solutions[i]=occurrences->size();
      connected_components[i]=get_nb_clumps(occurrences,motif_size);
 


      if (solutions[i]<nb_occurrences)
	nb_inf++;
      else if (solutions[i]>nb_occurrences)
	nb_sup++;
      else
 	nb_eq++;

      if (connected_components[i]<real_nb_connected_components)
	c_nb_inf++;
      else if (connected_components[i]>real_nb_connected_components)
	c_nb_sup++;
      else
 	c_nb_eq++;

     
      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;
    }
 
  //print_tab(solutions, nb_simulations);
  //cout<<endl;

  delete occurrences;
  delete[] solutions;
  delete[] connected_components;

  /*calculate the p-value*/
  if (nb_simulations > 0)
    {
      float hp = ((float)nb_sup + ((float)nb_eq / 2)) / (float)nb_simulations;
      *p=hp;
      *c = ((float)c_nb_sup + ((float)c_nb_eq / 2)) / (float)nb_simulations;
    }

  else
    {
      *p = -1.0;
      *c = -1.0;
    }

  return 1;

}




/**
 *Print p-value to output. If the number of graphs in which the number
 *of occurrences is greater than the number of solutions is 0, then
 *the p-value is said to be lower than the minimum measurable
 *(1/ns). Additionnally, the p-value is rounded 4 numbers after the
 *comma (this should be enough).
 *
 * @param p p-value
 * @param output device where to print
 * @param digits number of digits after the comma for rounding results
 */
void Io::print_p_value(float p, std::ostream& output, int digits)
{
  float approx_p = (float)((int)(pow((float)10,(float)digits)*p))/(float)pow((float)10,(float)digits);
  float p_min = 1.0/(float)nb_simulations;
  float approx_p_min = (float)((int)(pow((float)10,(float)digits)*p_min))/(float)pow((float)10,(float)digits);
  //float approx_result = (float)((int)(100*result))/(float)100;

  /*in case no graph has more occurrences*/
  if (p==0.0)
    //output<<"<"<<1.0/(float)nb_simulations;
    output<<"<"<<approx_p_min;

  /*in case no random graph has fewer occurrences*/
  if (p==1.0)
    //output<<">"<<1.0 - 1.0/(float)nb_simulations;
    output<<">"<<1.0 - approx_p_min;

  else
    output<<approx_p;
}

/**
 *Sorts the output file by ascending p-values when verbose is set to
4. It performs a sort using 'awk' and the gnu sort algorithm. It
outputs a modified version of the inference file. Namely, the
occurrences are not present anymore and the properties are written on
the same line as the motif.
 */
void Io::sort_by_p_value()
{
  //cerr<<"hio"<<endl;
  char *command=new char[7*MAX];
  command[0]='\0';

  /*string telling you which column is to be sorted*/
  ostringstream oss; //useful to turn int to string
  oss<< motif_size+3;
  int i=0;
  char *column=new char[MAX];
  while (oss.str()[i]!='\0')
    {
      column[i]=oss.str()[i];
      i++;
    }
  column[i]='\0';


  /*turn points into commas (get rid of !!!s)*/
  //system("awk '/^#/ {gsub(/\\./,\",\",$5); print $1 \" \" $2 \" \" $3 \"\t\" $4 \"\t\" $5;}' results/motus.res > hop");
  command = strcat(command,"awk '/^#/ {gsub(/\\./,\",\",$");
  command = strcat(command, column);
  command = strcat(command, "); print $0;}' ");
  command = strcat(command, results_file);
  command = strcat(command," > hop");
  cerr<<command<<endl;
  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  
  /*sort by p-value*/
  //system("sort -k 5 -n hop -o hop");
  command[0]='\0';
  command=strcat(command,"sort -k ");
  command = strcat(command, column);
  command = strcat(command, " -n hop -o hop");
  //system(command);
  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  
  cerr<<command<<endl;


  
  /*turn commas into points*/
  //  system("awk '/^#/ {gsub(/,/,\".\",$5); print $1 \" \" $2 \" \" $3 \"\t\" $4 \"\t\" $5;}' hop > hop2");
  command[0]='\0';
  command = strcat(command,"awk '/^#/ {gsub(/,/,\".\",$");
  command = strcat(command, column);
  command = strcat(command,"); print $0;}' hop > hop2");
  // command = strcat(command,"); print $0;}' hop > ");
//   command = strcat(command,results_file);
  cerr<<command<<endl;
  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  //system(command);

  /*add the header to the file*/
  std::ofstream output;
  output.open("hop4");
  //fprint_header(output,verbose, mode);
  fprint_header(output);
  output.close();

  /*concatenate files*/
  command[0]='\0';
  // command = strcat(command,"cat ");
//   command = strcat(command, results_file);
//   command = strcat(command, " hop2 > ");
  command = strcat(command, "cat hop4 hop2 > ");
  command = strcat(command, results_file);
  cerr<<command<<endl;
  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  //system(command);
  // command = strcat(command,"); print $0;}' hop > hop2");
//   system("cat ");
  if (system("rm -f hop hop2 hop4")!=0)
    {
      cerr<<"unable to execute command: rm -f hop hop2 hop4"<<endl;
      exit(0);
    }
  delete []command;
  delete []column;
}

/**
 *Sorts the output file by block according to ascending p-values, when
 *verbose is set to 2. A block starts with # and is composed of a
 *motif (set of EC numbers) followed by a line of properties, followed
 *by a list of occurrences. This format is used for Fabien Jourdan's
 *vizualizer.  This method is very slow due to the great number of
 *file traversal. It should be optimizable.
 */
void Io::sort_by_block()
{
  char str[MAX];
  bool start_block=0;
  char p[10];  //p-value as string
  int i,j=0;
  int nb_motifs=0;
  //double p_values[10*MAX];

  /*get the number of motifs*/
  std::ifstream file_op0(results_file,ios::in);
  while(!file_op0.eof())
    {
      file_op0.getline(str,MAX);
      if (str[0]=='#')
	nb_motifs++;
    }
  file_op0.close();

  /*allocate memory for p-values*/
  double p_values[nb_motifs];
  nb_motifs=0;

  /*fill p_values*/
  std::ifstream file_op(results_file,ios::in);
  while(!file_op.eof()) 
    {
      file_op.getline(str,MAX);

      /*if it is a new block, then we look for the p-value (second field)*/     
      if (start_block)
	{
	  /*pass through the first word*/
	  i=0;
	  while(str[i]!='\t')
	    i++;
	    
	  i++;
	  /*copy the second word to p*/
	  j=0;
	  while (str[i]!='\t')
	    {
	      p[j]=str[i];
	      j++;
	      i++;
	    }
	  p[j]='\0';
	  p_values[nb_motifs]=atof(p);

	  /*say that the block is treated*/
	  start_block=0;
	  nb_motifs++;
	}
 
      /*detection of new block*/
      if (str[0]=='#')
	start_block=1;
    }
      
  file_op.close();

  /*get the order in which the blocks should be to be in ascending p-values*/
  /*this order is stored in the array permutation*/
  int *permutation=new int[nb_motifs];
  for (int i=0;i<nb_motifs;i++)
    permutation[i]=i;

  sort(p_values,nb_motifs,permutation);

  // print_tab(permutation,nb_motifs);
//   print_tab(p_values, nb_motifs);


  /*reorder the file to write the blocks according to the new order*/
  /*this part of the method is very SLOW*/
  /*get motif # k*/
  std::ofstream output;
  char *file_name=new char[MAX];
  file_name[0]='\0';
  strcpy(file_name,write_path);
  file_name=strcat(file_name,"/hop");
  //  output.open("tmp/hop");
  output.open(file_name);

  for (int i=0;i<nb_motifs;i++)
    {
      std::ifstream  file_op2(results_file,ios::in);
      int k=0;
      while(!file_op2.eof())
	{
	  file_op2.getline(str,MAX);
	  
	  /*detection of new block*/
	  if (str[0]=='#')
	    {
	      
	      //cout<<"k "<<k<<endl;
	      /*detection of the block we are looking for*/
	      if (k==permutation[i]) //we found the right motif, we print it to file
		{
		  //cout<<"on est dans le bon cas"<<endl;
		  /*print the motif line*/
		  output<<str<<endl;
		  file_op2.getline(str,MAX);
		  //output<<str<<endl;
		  /*print the rest of the block until the next block*/
		  while (str[0]!='#' && !file_op2.eof())
		    {
		      output<<str<<endl;
		      file_op2.getline(str,MAX);
		      //output<<str<<endl;
		    }
		  break;
		}
	      k++;
	    }
	}
      file_op2.close();
    }
  output.close();


  /*add the header to the file*/
  //std::ofstream output;
  strcpy(file_name,write_path);
  file_name=strcat(file_name,"/hop4");
  //output.open("tmp/hop4");
  output.open(file_name);

  fprint_header(output);
  output.close();

  /*concatenate files*/
  char *command=new char[MAX];
  command[0]='\0';
 
  command = strcat(command, "cat ");
  command = strcat(command, write_path);
  command = strcat(command, "/hop4 ");
  command = strcat(command, write_path);
  command = strcat(command, "/hop > ");
  command = strcat(command, results_file);

 //  command = strcat(command, "cat tmp/hop4 tmp/hop > ");
//   command = strcat(command, results_file);
  
  
  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  //system(command);

  strcpy(command,"rm -f ");
  command = strcat(command, write_path);
  command = strcat(command, "/hop ");
  command = strcat(command, write_path);
  command = strcat(command, "/hop4 ");

  if (system(command)!=0)
    {
      cerr<<"unable to execute command: "<<command<<endl;
      exit(0);
    }
  //system(command);
  //system("rm -f tmp/hop tmp/hop4");

  /*free memory*/
  delete[] permutation;
  delete[] command;
  delete[] file_name;

  /*copy file to results_file*/
}


/**
 *Search motif in random graphs and print to file the results: number
 *of occurrences, number of connected components and size of the
 *connected components. This function looks very much like the c-value
 *calculation.
 */
void Io::fprint_stats_for_count(Reaction_Graph **simulations,Metabolic_Motif *myMotif, Reaction **tab_reactions)
{
  // if (nb_simulations==0)
//     return -1.0;


  int *solutions = new int[nb_simulations];
  int *connected_components = new int[nb_simulations];
  //list<int> **connected_components_size = new list<int>*[nb_simulations];
  int **connected_components_size = new int*[nb_simulations];

  list<int *> *occurrences = new list<int *>[nb_simulations];

  
  /*additional structure needed to build solution objects to tell the
    constructor that we already have the good indices*/
  int *identity_array = new int[simulations[0]->get_nb_nodes()];
  for (int k=0;k<simulations[0]->get_nb_nodes();k++)
    identity_array[k]=k;


  for (int i=0;i<nb_simulations;i++)
    {
      Graph *filtered_graph = new Graph(simulations[i],myMotif->get_colours(), myMotif->size());
      
      /*get occurrences*/
      filtered_graph->enumerate(occurrences,myMotif->size(),myMotif->get_colours());
      
      /*transform occurrences with integers corresponding to filtered
	reactions to integers corresponding to Reactions of the
	original complete graph*/
      int *parent_nodes= filtered_graph->get_parent_nodes();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	for (int j=0;j<myMotif->size();j++)
	  (*it)[j]=parent_nodes[(*it)[j]];


      /*create Solution objects with these occurrences*/
      Solution_Group *group = new Solution_Group();
      for (list<int *>::iterator it=occurrences->begin(); it!= occurrences->end(); it++)
	group->add(new Solution(*it,myMotif->size(), tab_reactions, identity_array));

      /*group the solutions into connected components*/
      list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
      group->group_by_reaction(disjoint_solutions);
      int nb_connected_components = (int)disjoint_solutions->size(); 
    
      /*store information on sizes of connected components*/
      // if (connected_components==0)
// 	{
// 	  connected_components_size[i] = new int[1];
// 	  connected_components_size[i][0]=0;
// 	}

//       else
// 	{
	  connected_components_size[i] = new int[nb_connected_components];
	  int j=0;
	  for (list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	    {
	      connected_components_size[i][j]=(int)(*it)->size();
	      j++;
	    }
	  //}

      /*delete connected_components*/
      for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	delete(*it);
      delete disjoint_solutions;

      /*store obtained values for this random graph (#i)*/
      connected_components[i]=nb_connected_components;
      solutions[i]=occurrences->size();
      //connected_components_size[i]=connected_components_size_list;


      /*clean list of occurrences from the last call*/
      for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	{
	  delete[] (*it);
	  it=occurrences->erase(it);
	  it--;
	} 
      delete filtered_graph;

      /*clean solutions from the last call*/
      group->delete_solutions();
      delete group;
    }
 
  /*print to file the occurrences count*/
  std::ofstream output;
  output.open("nb_occurrences");
  for (int i=0;i<nb_simulations;i++)
    output<<solutions[i]<<endl;
  output.close();

  /*print to file the number of connected components*/
  output.open("nb_trains");
  for (int i=0;i<nb_simulations;i++)
    output<<connected_components[i]<<endl;
  output.close();

  /*print to file the number of connected components*/
  output.open("taille_des_trains1");
  for (int i=0;i<nb_simulations;i++)
    for (int j=0;j<connected_components[i];j++)
      output<<connected_components_size[i][j]<<endl;
  output.close();

  /*print to file the number of connected components*/
  output.open("taille_des_trains2");
  for (int i=0;i<nb_simulations;i++)
    {
      for (int j=0;j<connected_components[i];j++)
	output<<connected_components_size[i][j]<<" ";
      output<<endl;
    }
  output.close();

  delete[] occurrences;
  delete[] identity_array;
  delete[] solutions;
  delete[] connected_components;
  for (int i=0;i<nb_simulations;i++)
    delete[] connected_components_size[i];
  delete[] connected_components_size;
  exit(0);
}



/**
 *Count the number of occurrences of a motif in a graph and calculate
 *a p-value, or several p-values if occurrences are grouped into
 *subclasses.
 *
 * @param myData dataset used (useful to create the motif)
 * @param enzymes string defining the enzymes from the motif
 * @param all_solutions Solution_Group where all solutions of the search are stored
 * @param topology_solutions list of Solution_Groups to store the solutions of each topological motif
 * @param simulations array of shuffled graphs to assess motif over-representation
 * @param whole_reaction_graph graph in which the motif is sought
 * @param p p-value of the coloured motif (passed as a pointer since it is an output of the method)
 * @param c p-value of the number of connected components of the coloured motif (passed as a pointer since it is an output of the method)
 * @param p_s array of p-values corresponding to each sub-topological motif inside the colored motif
 * @param nb_observed_topologies size of the p_s array, it is used as a return value (which explains why it is a pointer)
 * 
 * @return p-value p-value of the motif
 */
float *Io::motif_search(Data *myData, char *enzymes, Solution_Group *all_solutions, list<Solution_Group *> *topology_solutions, Reaction_Graph **simulations, Reaction_Graph *whole_reaction_graph, float *p, float *c, float *p_s, int *nb_observed_topologies)
{
  Metabolic_Motif *myMotif = new Metabolic_Motif(myData, local_gap, global_gap, quorum, sigma_E, enzymes, input_compound, output_compound, results_file);
  
  /*initialisation of p-value*/
  *p=-1;



  /*motif search - new version*/
  /*filter the graph using the colours of the motif*/
  Reaction_Graph *filtered_graph = new Reaction_Graph(whole_reaction_graph, myMotif->get_colours(), myMotif->size());

  /*STEP 2: enumerate solutions inside the filtered graph*/
  //Solution_Group *all_solutions = new  Solution_Group();
  filtered_graph->enumerate(all_solutions, myMotif);

  /*STEP 3: calculation of p-values when the motif is found*/
  if (all_solutions->size() > 0)
    {
      /*in case you need to calculate p-values for topological motifs*/
      if (topology==1)
	{
	  /*first group solutions by topology, then do it for shuffled graphs*/
	  Bipartite_Graph *myBipartiteGraph;
	  myBipartiteGraph = new Bipartite_Graph(myData->tab_reactions, myData->nb_reactions, myData->tab_compounds, myData->nb_compounds);
	  all_solutions->set_bipartite_graph(myBipartiteGraph,myMotif->get_colours());

	  all_solutions->compute_canonical_labelling(order);
	  all_solutions->group_by_topology(topology_solutions);

	  /*assessing over-representation of toplogical motifs*/
	  // cerr<<"dbut de calcul de sur-reprsentation"<<endl;
// 	  cerr<<"avant la mise  jour du nombre de topologies, nb_obs_top = "<<*nb_observed_topologies<<endl;
	  *nb_observed_topologies = (int)topology_solutions->size();
	  // cerr<<"aprs la mise  jour du nombre de topologies, nb_obs_top = "<<*nb_observed_topologies<<endl;

	  int observed_topologies_counts[*nb_observed_topologies];


	  long int observed_topologies1[*nb_observed_topologies];
	  long int observed_topologies2[*nb_observed_topologies];

	  /*fill these arrays*/
	  int g=0;
	  for (list<Solution_Group *>::iterator iter = topology_solutions->begin(); iter != topology_solutions->end(); iter++)
	    {
	      observed_topologies1[g] = (*iter)->get_solutions()->front()->get_topo1();
	      observed_topologies2[g] = (*iter)->get_solutions()->front()->get_topo2();
	      observed_topologies_counts[g] = (*iter)->size();
	      g++;
	    }
 
	  /*array storing the p-value for each sub-motif i.e. defined with a given topology*/
	  // if (p_s != NULL)
// 	    delete[] p_s;
	  p_s = new float[*nb_observed_topologies]; 
	  for (int i=0;i<*nb_observed_topologies;i++)
	    p_s[i]=0.0;	  

	  // time_t *tim=NULL; //to be able to say how long it takes
	  
	  // cout<<time(tim)<<endl;
// 	  cout<<"estimating p-values for each of the topological motifs"<<endl;

          /*this is the costly step*/
	  calculate_p_values(simulations, myMotif, *nb_observed_topologies, observed_topologies_counts, observed_topologies1, observed_topologies2, p_s, myData->tab_reactions, myBipartiteGraph);

	  
	  /*delete everything*/
	  delete myBipartiteGraph;

	}

      else
	{
	  *p=calculate_p_value(simulations,myMotif->get_colours(), myMotif->size(),all_solutions->size());
	  /*first divide the solutions into groups of overlapping solutions*/
	  /*WATCH OUT: this is done twice, also when printing solutions, this should be optimized*/
	  list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
	  all_solutions->group_by_reaction(disjoint_solutions);
	      
	  /*calculate the p-value corresponding to the over-representation of the number of connected components*/
	  int nb_connected_components = (int)disjoint_solutions->size();
	  *c=calculate_c_value(simulations,myMotif,all_solutions->size(),nb_connected_components,myData->tab_reactions);
	  
	  /*free space*/
	  for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	    delete(*it);
	  delete(disjoint_solutions);

	}
    }

  else 
    p_s=NULL;

  delete filtered_graph;
  delete myMotif;
  
  return p_s;
}


/**
 *Count the number of occurrences of a motif in a graph
 *
 * @param random_graph graph in which the motif is sought
 * @param motif_colors array giving the colors composing the motif
 * @param motif_size number of colors in the motif
 * @return number of occurrences of the motif in the graph
 */
//int Io::motif_search(Graph *random_graph, int *motif_colors, int motif_size)
int Io::motif_search(Data *myData, Graph *random_graph, char *enzymes)
{
  /*motif_search*/
  Metabolic_Motif *myMotif = new Metabolic_Motif(myData, local_gap, global_gap, quorum, sigma_E, enzymes, input_compound, output_compound, results_file);
  
  Graph *filtered_graph = new Graph(random_graph,myMotif->get_colours(), myMotif->size());
  int nb_solutions=0;
  nb_solutions = filtered_graph->enumerate(myMotif->size(),myMotif->get_colours());
  delete filtered_graph;
  delete myMotif;
  return nb_solutions;



/*we use this method of enumerate in Graph.cpp because the lighter one (the one that only returns the number of solutions is bugged, it does not use topology_matrix)*/
  // list<int *> *occurrences = new list<int *>;
//   Graph *filtered_graph = new Graph(random_graph,myMotif->get_colours(), myMotif->size());
//   filtered_graph->enumerate(occurrences,myMotif->size(),myMotif->get_colours());
//   int nb_solutions=occurrences->size();
  
//   /*clean list of occurrences from the last call*/
//   for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
//     {
//       delete[] (*it);
//       it=occurrences->erase(it);
//       it--;
//     } 
//   delete occurrences;
//   delete myMotif;
//   delete filtered_graph;
  
//   return nb_solutions;


  // Graph *filtered_graph = new Graph(random_graph,motif_colors,motif_size);
//   int nb_solutions=0;
//   nb_solutions = filtered_graph->enumerate(motif_size, motif_colors);
//   delete filtered_graph;
//   return nb_solutions;
}



/**Generate random graphs from the original graph and store them in an
   array. Note that this way is costly in memory.*/
void Io::shuffle_graphs(Reaction_Graph *original_graph, Reaction_Graph **simulations, int nb_simulations)
{
  /*declare necessary structures*/
  int nb_nodes=original_graph->get_nb_nodes();
  int *tab = new int[nb_nodes];

  /*initialisation of tab as the identity permutation*/
  for (int i=0;i<nb_nodes;i++)
    tab[i]=i;

  for (int k=0;k<nb_simulations;k++)
    {
      /*submit the identity permutation to "melangeur" and collect a
	random permutation in tab*/
      generateseqInt2(nb_nodes,tab,write_path);
      Reaction_Graph *shuffledReactionGraph = new Reaction_Graph(original_graph,tab);
      simulations[k]=shuffledReactionGraph;
    }

  delete[] tab;
}

/**Generate random graphs from the original graph and store them in an
   array. Note that this way is costly in memory.*/
void Io::shuffle_graphs(Graph *original_graph, Graph **simulations, int nb_simulations)
{
  /*declare necessary structures*/
  int nb_nodes=original_graph->get_nb_nodes();
  int *tab = new int[nb_nodes];

  /*initialisation of tab as the identity permutation*/
  for (int i=0;i<nb_nodes;i++)
    tab[i]=i;

  for (int k=0;k<nb_simulations;k++)
    {
      /*submit the identity permutation to "melangeur" and collect a
	random permutation in tab*/
      generateseqInt2(nb_nodes,tab,write_path);
      Graph *shuffledReactionGraph = new Graph(original_graph,tab);
      simulations[k]=shuffledReactionGraph;
    }

  delete[] tab;
}



// /*we print the enzyme EC numbers, the number of disjoint groups of
//   solutions found by the prog, the number of solutions for each group,
//   the total number of solutions, the number of intra-pathway
//   solutions, the mean (for all solutions) of the arity of the edge
//   compound which is a measure of the ubiquitarity of the edge
//   compound, and finally the number of soltions whose size is the same
//   as the motif's*/
// void Io::fprint_useful_info(Metabolic_Motif *motif, list<Solution_Group *> *groups, list<Solution_Group *> *ordered_groups)
// {
//   /*note that we use a ofstream object here, and it is considered as a ostream object later on (ofstream inherits from ostream), apparently, you need to do that if you want to use the << operator*/
//   //cerr<<"fprint_usefulinfo"<<endl;
//   std::ofstream output;
//   output.open("systematic.res", ios_base::app);
//   motif->print_enzymes(output);
//   //cerr<<"aprs print_enzymes"<<endl;
//   output<<groups->size()<<" ";
//   //cerr<<"aprs size"<<endl;
//   int nb_sol=0;
//   int nb_intra_pathway=0;
//   //float cpd_arity=0.0;
//   //int nb_max_sol=0;
//   list<Solution_Group *>::iterator group_it;
//   group_it=groups->begin();
//   while (group_it != groups->end())
//     {
//       //cerr<<"boucle groueps"<<endl;
//       output<<(*group_it)->size()<<" ";
//       //cerr<<"aprs size"<<endl;
//       nb_sol+=((*group_it)->size());
//       //cerr<<"avant intra_pat"<<endl;
//       nb_intra_pathway+=(*group_it)->nb_intra_pathway();
//       //cerr<<"aprs intra_pat"<<endl;
//       //cpd_arity+=(*group_it)->cpd_arity();
//       //nb_max_sol+=(*group_it)->nb_max_sol(motif);
//       group_it++;
//     }

//   output<<nb_sol<<" ";
//   //cerr<<"aprs nb_sol"<<endl;
//   output<<nb_intra_pathway<<" ";
//   //output<<cpd_arity<<" ";


//   /*group by order*/
//   //cerr<<"avant nb_groups_expected()"<<endl;
//   output<<motif->nb_groups_expected()<<" "; //nb groups expected
//   //cerr<<"aprs nb_groups_expected()"<<endl;
//   output<<ordered_groups->size()<<" ";  //nb groups observed
//   group_it=ordered_groups->begin();
//   int nb_linear_sol=0;
//   while (group_it != ordered_groups->end())
//     {
//       output<<(*group_it)->size()<<" ";  //distribution of sizes for groups
//       nb_linear_sol+=((*group_it)->size());
//       group_it++;
//     }
//   output<<nb_linear_sol<<" ";  //total number of linear solutions
//   output<<endl;
//   output.close();
// }


// /*we print the enzyme EC numbers, and the number of occurrences*/
// void Io::fprint_nb_occurrences(Metabolic_Motif *motif, list<Solution_Group *> *groups)
// {
//   std::ofstream output;
//   output.open("nb_occurrences.res", ios_base::app);
//   motif->print_enzymes(output);

//   int nb_sol=0;
//   list<Solution_Group *>::iterator group_it;
//   group_it=groups->begin();
//   while (group_it != groups->end())
//     {
//       nb_sol+=((*group_it)->size());
//       group_it++;
//     }
//   output<<nb_sol;
//   output<<endl;
//   output.close();
// }

int Io::fprint_header(std::ostream& output)
{

  /*special case where we print topological motifs*/
  if (topology==1)
    {  
      output<<"!This file is structured as follows:"<<endl;
      output<<"!The first line after the \"!\"s indicates the names of the properties of the motif. This line begins with the keyword PROPERTIES, then properties are separated by tabs"<<endl;
      output<<"!Each bloc, starting with # gives:"<<endl;
      output<<"!   - the motif found (multiset of EC numbers)"<<endl;
      output<<"!   - properties of the occurrences (separated by tab):"<<endl;
      output<<"!           - number of occurrences"<<endl;
      output<<"!           - the p-value (to assess over-representation of the motif):"<<endl;
      output<<"!           - the number of connected components formed by the occurrences (occurrences may overlap)"<<endl;
      output<<"!           - the mean number of pathways per occurrence"<<endl;
      output<<"!           - the variance of the number of pathways per occurrence"<<endl;
      output<<"!           - the number of occurrences totally included in a pathway"<<endl;
      output<<"!   - the complete list of the occurrences (sets of reactions), one per line"<<endl;
      output<<"PROPERTIES\tnumber of occurrences\tp-value\tnumber of connected components\tnumber of pathways per occurrence (mean)\tnumber of pathways per occurrence (variance)\tnumber of intra-pathway occurrences"<<endl;
      return 0;
    }

  //cout<<"je suis dans fprint header"<<endl;
  output<<"!This file is structured as follows:"<<endl;
  if (mode==3)
	output<<"!You have been using the m3 option : your results have been FILTERED"<<endl;

  switch(verbose)
	{
	case 1 : 
	  output<<"!Each line gives the motif found (multiset of EC numbers), the number of occurrences and the p-value (to assess over-representation of the motif):"<<endl;
	  break;
	case 2 : 
	  output<<"!The first line after the \"!\"s indicates the names of the properties of the motif. This line begins with the keyword PROPERTIES, then properties are separated by tabs"<<endl;
	  output<<"!Each bloc, starting with # gives:"<<endl;
	  output<<"!   - the motif found (multiset of EC numbers)"<<endl;
	  output<<"!   - properties of the occurrences (separated by tab):"<<endl;
	  output<<"!           - number of occurrences"<<endl;
	  output<<"!           - the p-value (to assess over-representation of the motif):"<<endl;
	  output<<"!           - the number of connected components formed by the occurrences (occurrences may overlap)"<<endl;
	  output<<"!           - the c-value (to assess over-representation of the number of connected components of the motif):"<<endl;
	  output<<"!           - the mean number of pathways per occurrence"<<endl;
	  output<<"!           - the variance of the number of pathways per occurrence"<<endl;
	  output<<"!           - the number of occurrences totally included in a pathway"<<endl;
	  output<<"!   - the complete list of the occurrences (sets of reactions), one per line"<<endl;
	  output<<"PROPERTIES\tnumber of occurrences\tp-value\tnumber of connected components\tc-value\tnumber of pathways per occurrence (mean)\tnumber of pathways per occurrence (variance)\tnumber of intra-pathway occurrences"<<endl;
	  break;
	case 3 : 
	  output<<"!Each bloc, starting with # gives:"<<endl;
	  output<<"!   - the motif found (multiset of EC numbers)"<<endl;
	  output<<"!   - the complete list of the occurrences (sets of reactions), one per line"<<endl;
	  break;
	case 4 : 
	  // output<<"!The first line after the \"!\"s indicates the names of the properties of the motif. This line begins with the keyword PROPERTIES, then properties are separated by tabs"<<endl;
	  output<<"!Each line, starting with # gives the following items separated by tabs:"<<endl;
	  output<<"!   - the motif found (multiset of EC numbers)"<<endl;
	  output<<"!   - the number of occurrences"<<endl;
	  output<<"!   - the p-value"<<endl;
	  output<<"!   - the number of connected components formed by the occurrences (occurrences may overlap)"<<endl;
	  output<<"!   - the c-value (p-value for connected components)"<<endl;
	  output<<"!   - the mean number of pathways per occurrence"<<endl;
	  output<<"!   - the variance of the number of pathways per occurrence"<<endl;
	  output<<"!   - the number of occurrences totally included in a pathway"<<endl;
	  break; 
	}
  return 1; 

}





/**
 *Print to result_file a motif and its occurrences
 * mode can take values in [1,2,3]
 * m=1 : properties are not calculated
 * m=2 : properties are calculated
 * m=3 : properties are used to filter the solutions (the motif is printed only if the number of groups of non-overlapping occurrences is greater than one and if the number of intra-pathway occurrences in not 0)

 * verbose can take values in [1,2,3,4]
 * v=1 : print motif + number of occurrences
 * v=2 : print motif + properties + list of occurrences (Fabien)
 * v=3 : print motif + list of occurrences
 * v=4 : print motif + properties, including p-value (Patricia + website)
 
 * @param enzymes: string corresponding to the EC numbers
 * @param group: Solution_Group containing all solutions to print
 * @param p p-value of the coloured motif
 * @param c p-value for the number of connected components of the coloured motif
 * @param output: any output (preferably open in append mode)
  */
void Io::fprint_solutions(char *enzymes, Solution_Group *group, float p, float c, std::ostream& output)
{

  /*if mode is set to 1, then we don't calculate the properties on the motifs*/
  if (mode==1)
    {
      /*print motif + number of occurrences + p-value*/
      if (verbose==1)
	{
	  output<<enzymes<<group->size()<<" "<<p;
	  //print_p_value(p,output);
	  output<<endl;
	}
      
      /*print motif + list of occurrences (useful for old version of Fabien's drawing tool*/
      else if (verbose==3)
	{
	  output<<"# "<<enzymes<<endl; 
	  group->print(output);
	}
      
      else 
	{
	  cerr<<"wrong value for verbose (option -v), most probably not compatible with mode value"<<endl; 
	  exit(0);
	}
    }
  
  /*properties are calculated*/
  else if (mode==2 || mode==3 || mode==4)
    {
      /*calculation of properties*/
      /*calculation of the number of connected components*/
      list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
      group->group_by_reaction(disjoint_solutions);
      int nb_connected_components = (int)disjoint_solutions->size();      
      for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	delete(*it);
      delete disjoint_solutions;
      
      /*calculation of the number of pathways per occurrence*/
      float mean_nb_pathways = group->mean_nb_pathway_per_occurrence();
      float var_nb_pathways = group->var_nb_pathway_per_occurrence();
      int nb_intra_pathways = group->nb_intra_pathway();
      
      /*formatting the output*/
      enzymes[strlen(enzymes)-1]='\0'; //the string enzymes used to end with an extra space
      
      
      /*properties are printed but not used as filter*/
      if (mode==2)
	{
	  /*print motif + number of occurrences + p-value*/
	  if (verbose==1)
	    {
	      output<<enzymes<<" "<<group->size()<<" "<<p;
	      //print_p_value(p,output);
	      output<<endl;
	      //output<<enzymes<<" "<<group->size()<<" "<<p<<endl;
	    }
	  
	  /*print motif + number of occurrences + properties + list of occurrences (Fabien, will add the p-value later)*/
	  else if (verbose==2)
	    {
	      output<<"# "<<enzymes<<endl; 
	      output<<group->size()<<"\t"<<p<<"\t"<<nb_connected_components<<"\t"<<c<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<"\t"<<endl;
	      group->print(output);
	    }
	  
	  /*print motif + list of occurrences (useful for old version of Fabien's tool*/
	  else if (verbose==3)
	    {
	      output<<"# "<<enzymes<<endl; 
	      group->print(output);
	    }
	  
	  /*print motif + properties including p-value (useful for Patricia + website)*/
	  else if (verbose==4)
	    {
	      output<<"# "<<enzymes<<" "<<group->size()<<" "<<p<<" "<<nb_connected_components<<" "<<c<<" "<<mean_nb_pathways<<" "<<var_nb_pathways<<" "<<nb_intra_pathways<<endl;
	      //output<<"# "<<enzymes<<"\t"<<group->size()<<"\t"<<p<<"\t"<<nb_connected_components<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<endl;
	      
	    }
	  
	      
	  else 
	    {
	      cerr<<"wrong value for verbose (option -v), authorized values are 1,2,3 and 4"<<endl; 
	      exit(0);
	    }
	}
      
      /*properties are used to filter the solutions*/
      if (mode==3)
	{
	  if (nb_connected_components>1 && nb_intra_pathways>0)
	    {
	      
	      /*print motif + number of occurrences*/
	      if (verbose==1)
		{
		  output<<enzymes<<group->size()<<" "<<p<<endl;
		}
	      
	      /*print motif + number of occurrences + properties + list of occurrences*/
	      else if (verbose==2)
		{
		  output<<"# "<<enzymes<<endl; 
		  output<<group->size()<<"\t"<<p<<"\t"<<nb_connected_components<<"\t"<<c<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<endl;
		  group->print(output);
		}
	      
	      /*print motif + list of occurrences*/
	      else if (verbose==3)
		{
		  output<<"# "<<enzymes<<endl; 
		  group->print(output);
		}
	      
	      /*print motif + properties including p-value (useful for Patricia + website)*/
	      else if (verbose==4)
		{
		  output<<"# "<<enzymes<<"\t"<<group->size()<<"\t"<<p<<"\t"<<nb_connected_components<<"\t"<<c<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<endl;
		}
	      // /*print motif + some properties*/
	      // 	      else if (verbose==4)
	      // 		{
	      // 		  output<<"# "<<enzymes<<"\t"<<group->size()<<"\t"<<nb_connected_components<<"\t"<<nb_intra_pathways<<endl;
	      // 		}
	      
	      // 	      /*print motif + properties (including p-value)*/
	      // 	      else if (verbose==5)
	      // 		{
	      // 		  output<<"# "<<enzymes<<endl; 
	      // 		  output<<group->size()<<"\t"<<p<<"\t"<<nb_connected_components<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<"\t"<<endl;
	      // 		  //output<<"# "<<enzymes<<"\t"<<group->size()<<"\t"<<nb_connected_components<<"\t"<<nb_intra_pathways<<endl;
	      //	}
	      
	      else 
		{
		  cerr<<"wrong value for verbose (option -v), authorized values are 1,2,3 and 4"<<endl; 
		  exit(0);
		}
	    }
	}
      
      /*we do not print occurrences that are not intra-pathway (useful to test operons)*/
      if (mode==4)
	{
	  output<<"# "<<enzymes<<endl; 
	  output<<nb_intra_pathways<<" "<<p<<" "<<nb_connected_components<<" "<<mean_nb_pathways<<" "<<var_nb_pathways<<" "<<nb_intra_pathways<<endl;
	  group->print_intra_pathway(output);
	}


    }
  
  /*wrong value for mode*/
  else 
    {
      cerr<<"wrong value for mode (option -m), authorized values are 1,2,3, start again"<<endl; 
      exit(0);
    }

}


/**
 *Print to result_file topological motifs and its occurrences. By default, all properties are calculated and printed.
 * @param enzymes: string corresponding to the EC numbers
 * @param topology_solutions: list of Solution_Group containing solutions for each topology to print
 * @param p_s p-values of the topological coloured motif
 * @param output: any output (preferably open in append mode)
  */
void Io::fprint_solutions(char *enzymes, list<Solution_Group *> *topology_solutions, float *p_s, std::ostream& output)
{
  int i=0;
  /*formatting the output*/
  enzymes[strlen(enzymes)-1]='\0'; //the string enzymes used to end with an extra space

  // /*getting the topology of the subgraph, independent of the colour order*/
//   long int topo=topology_solutions->front()->get_topo(0);

  
/*This is to assess what is the number of clumps, for the coloured motif, prior to clustering by topology*/
/*First gather all solutions in the same structure*/
  int global_nb_connected_components=0;
  Solution_Group *all_solutions=new Solution_Group();
  for (list<Solution_Group *>::iterator it=topology_solutions->begin(); it!=topology_solutions->end(); it++)
    {
      all_solutions->copy_group(*it);
    }
  // cout<<all_solutions->size()<<endl;

  /*Then group them by clumps*/
  list<Solution_Group *> *global_disjoint_solutions=new list<Solution_Group *>;
  all_solutions->group_by_reaction(global_disjoint_solutions);
  global_nb_connected_components = (int)global_disjoint_solutions->size();      
  /*Then clean data structures*/
  for(list<Solution_Group *>::iterator it2=global_disjoint_solutions->begin();it2!=global_disjoint_solutions->end();it2++)
    delete(*it2);
  delete global_disjoint_solutions;
  delete all_solutions;
  

  for (list<Solution_Group *>::iterator it=topology_solutions->begin(); it!=topology_solutions->end(); it++)
    {
      
      /*calculation of properties*/
      /*calculation of the number of connected components*/
      list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
      (*it)->group_by_reaction(disjoint_solutions);
      int nb_connected_components = (int)disjoint_solutions->size();      
      for(list<Solution_Group *>::iterator it2=disjoint_solutions->begin();it2!=disjoint_solutions->end();it2++)
	delete(*it2);
      delete disjoint_solutions;

     
      /*calculation of the number of pathways per occurrence*/
      float mean_nb_pathways = (*it)->mean_nb_pathway_per_occurrence();
      float var_nb_pathways = (*it)->var_nb_pathway_per_occurrence();
      int nb_intra_pathways = (*it)->nb_intra_pathway();

      /*Order of the colours in the motif*/
      
      long int topo=(*it)->get_topo(0);
      
      long int topo_order=0;
      if (order)
	topo_order=(*it)->get_topo(1);
      else
	topo_order=topo;
  

      /*I have a group of topology and I want to know how many groups of order it has*/
      if (permutability==1)
	{
	  if ((*it)->size()>2)//more than 3 occurrences
	    {
	      if (topo==213444222) //linear path of 3 reactions
		{
		  (*it)->compute_canonical_labelling(1);
		  list<Solution_Group *> *order_solutions=new list<Solution_Group *>;
		  (*it)->group_by_topology(order_solutions);
		  int nb_order_groups = (int)order_solutions->size();  
		  perm[nb_order_groups-1]++;
		  // cerr<<"in the loop"<<endl;
		  delete order_solutions;
		}
	    }
	}

    
      /*For studying the groups of order for linear paths*/
      if(verbose==0)
	{
	  // output<<enzymes<<" "<<(*it)->size()<<" "<<topo<<" "<<topo_order<<" "<<p_s[i]<<" "<<nb_intra_pathways<<endl;
	  output<<enzymes<<" "<<(*it)->size()<<" "<<topo<<" "<<topo_order<<" "<<p_s[i]<<" "<<global_nb_connected_components<<" "<<nb_intra_pathways<<endl;
	}

      if (verbose==1)
	{
	  output<<enzymes<<" "<<(*it)->size()<<" "<<p_s[i]<<endl;
	}

      if (verbose==2)
	{
	  output<<"# "<<enzymes<<endl; 
	  output<<(*it)->size()<<"\t"<<p_s[i]<<"\t"<<nb_connected_components<<"\t"<<-1<<"\t"<<mean_nb_pathways<<"\t"<<var_nb_pathways<<"\t"<<nb_intra_pathways<<"\t"<<endl;
	  (*it)->print(output);
	}
    }
}

void Io::fprint_nb_solutions(char *enzymes, int nbsol, std::ostream& output)
{
  // std::ofstream output;
//   output.open(results_file, ios_base::app);
  output<<enzymes<<nbsol<<endl;
  // output.close();
}

void Io::fprint_nb_solutions(int *motif, int motif_size, int nbsol, std::ostream& output)
{
  // std::ofstream output;
//   output.open(results_file, ios_base::app);
  print_tab(motif,motif_size,output);
  output<<nbsol<<endl;  
  //output<<motif<<nbsol<<endl;
  //output<<enzymes<<nbsol<<endl;
  // output.close();
}


/*able to detect if topological motifs are in disjoint parts of the
  network*/
void Io::refine_motifs(Data *myData, Reaction_Graph *whole_reaction_graph, Reaction_Graph **simulations)
{
  cout<<"je suis dans refine motifs"<<endl;
  
  /*other structures needed on the way*/
  char *enzymes=(char*)malloc((MAX)*sizeof(char));

  /*needed to store intermediate solutions before printing*/
  //int nbsol=0;
  float p=-1;
  float c=-1;
  float *p_s=NULL;
  int nb_observed_topologies=0;
  Solution_Group *all_solutions=new Solution_Group();

  list<Solution_Group *> *topology_solutions=new list<Solution_Group *>;

  /*file writing*/
  std::ofstream output;
  output.open(results_file);

  /*get the motifs*/
  char str[MAX];
  std::ifstream file_op(motifs_file,ios::in);
  while(!file_op.eof())
    {
      /*get the motif*/
      file_op.getline(str,MAX);
      strcpy(enzymes,str);

      /*special tricks so that everything goes smooth*/
      if (enzymes[0]=='\0')
	break;
      strcat(enzymes," ");

      /*search for the coloured motif (and group by topology if t=1)*/
      //cerr<<enzymes<<endl;
      p_s=motif_search(myData, enzymes,all_solutions, topology_solutions, simulations, whole_reaction_graph,&p,&c, p_s,&nb_observed_topologies);

      /*group topology_solutions into disjoint_solutions*/
      
      for(list<Solution_Group *>::iterator top_it=topology_solutions->begin();top_it!=topology_solutions->end();top_it++)
	{
	  list<Solution_Group *> *disjoint_solutions=new list<Solution_Group *>;
	  (*top_it)->group_by_reaction(disjoint_solutions);
	  int nb_connected_components = (int)disjoint_solutions->size(); 
	  //cout<<nb_connected_components<<endl;
	  if (nb_connected_components>1) 
	    {
	      // cout<<nb_connected_components<<endl;
// 	      cout<<"hiphip"<<endl;

	      output<<enzymes<<" "<<nb_connected_components<<endl;
	    }
	  for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
	    delete(*it);
	  delete disjoint_solutions;
	}
      // group->group_by_reaction(disjoint_solutions);
//       int nb_connected_components = (int)disjoint_solutions->size();      
//       for(list<Solution_Group *>::iterator it=disjoint_solutions->begin();it!=disjoint_solutions->end();it++)
// 	delete(*it);
//       delete disjoint_solutions;

      /*print results*/
      //output<<enzymes<<" "<<all_solutions->size()<<" "<<nb_observed_topologies<<" "<<endl;

      /*delete previous results*/
      for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
	delete(*it);
      topology_solutions->clear(); //to clear out all these pending deleted groups
      all_solutions->delete_solutions();
      if (p_s!=NULL)
	delete[] p_s;


      // cout<<"j'ai cherch le motif "<<enzymes<<endl;
//       cout<<endl;

    }
  file_op.close();

  output.close();
  
  exit(0);
}


/**
 *performs inference of motifs of a given size but not from the whole
  list of enzymes. It uses the list provided with the --enzymes
  option. No pruning technique is applied here. This function is used
  when the --enzymes option is used.
 * @param myData: string corresponding to the EC numbers
 * @param whole_reaction_graph the reaction graph corresponding to the whole network
 * @param motif_size the size of the motif
 * @param simulations an array of randomized reaction_graphs that are used to calculate p-values
  */

void Io::motif_inference_from_enzymes(Data *myData, Reaction_Graph *whole_reaction_graph, int motif_size, Reaction_Graph **simulations)
{

  /*file writing*/
  std::ofstream output;
  output.open(results_file);

  /*structures needed to traverse all motifs from a list of enzymes*/
  int *color_choice=(int*)malloc(motif_size*sizeof(int));
  for (int i=0;i<motif_size;i++)
    color_choice[i]=0;
  bool last_multichoice = 0;

  /*needed to store intermediate solutions before printing*/
  int nbsol=0;
  float p=-1;
  float c=-1;
  float *p_s=NULL;
  int nb_observed_topologies=0;
  Solution_Group *all_solutions=new Solution_Group();

  list<Solution_Group *> *topology_solutions=new list<Solution_Group *>;

  /*declaration, memory allocation and initialisation of the tab of
    enzymes I'm going to use*/
  char **tab=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab[i]=(char*)malloc((MAX)*sizeof(char));
 
  int nb_enzymes=0;

  /*other structures needed on the way*/
  char *enzymes=(char*)malloc((MAX)*sizeof(char));

  /*get the enzymes*/
  char str[MAX];
  std::ifstream file_op(enzymes_file,ios::in);
  while(!file_op.eof())
    {
      /*get the motif*/
      file_op.getline(str,MAX);
      strcpy(tab[nb_enzymes],str);
      
      if (str[0]=='\0')
 	break;

      nb_enzymes++;
      // /*special tricks so that everything goes smooth*/
//       if (enzymes[0]=='\0')
// 	break;
//       strcat(enzymes," ");
    

    }

  print_tab(tab,nb_enzymes);

  while(!last_multichoice)
    {
      enzymes[0]='\0';  //reinitialize enzymes
      for (int i=0;i<motif_size;i++)
	{
	  enzymes = strcat(enzymes, tab[color_choice[i]]);
	  enzymes = strcat(enzymes, " ");
	}
      p_s=motif_search(myData, enzymes,all_solutions, topology_solutions, simulations, whole_reaction_graph,&p,&c, p_s,&nb_observed_topologies);
      nbsol=all_solutions->size();
      
      
      /*print results*/
      if (topology==0)
	fprint_solutions(enzymes, all_solutions,p,c,output);
      
      else
	fprint_solutions(enzymes, topology_solutions, p_s, output);
      
      /*delete previous results*/
      for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
	delete(*it);
      topology_solutions->clear(); //to clear out all these pending deleted groups
      all_solutions->delete_solutions();
      if (p_s!=NULL)
	delete[] p_s;
      
      /*go to the next motif*/
      last_multichoice = !next_multichoice(color_choice,motif_size,nb_enzymes);
    }


  /*free memmory*/
  free(color_choice);
  free(enzymes);
  for (int i=0; i<MAX_NB_EC; i++)
    free(tab[i]);
  free(tab);
  output.close();

  delete all_solutions;
  
  delete topology_solutions;

  exit(0);
}

void Io::motif_inference(Data *myData, Reaction_Graph *whole_reaction_graph, int threshold, int motif_size, Reaction_Graph **simulations)
{
  //cerr<<"on est dans systematic analysis"<<endl;

  /*file writing*/
  std::ofstream output;
  output.open(results_file);

  /*headers for the file*/
  //fprint_header(output, verbose, mode);
  fprint_header(output);

  /*time counting*/
  clock_t begin, end;
  double cpu_time_used;
    
  begin = clock();
  //... /* Do the work. */
  end = clock();
  cpu_time_used = ((double) (end - begin)) / CLOCKS_PER_SEC;
  
  time_t strt_time, curr_time;
  time(&strt_time);
  int nb_motifs_done=0;
  int nb_motifs_done_old=0;
  int nb_motifs_total=0;
  double elapsed_time, estimated_time;
  int time_display_flag=0;

  /*to look at the time for each motif -- this is test*/
  // std::ofstream output2;
//   output2.open("hop");
  


  /*declaration, memory allocation and initialisation of the tab of enzymes I'm going to use*/
  char **tab1=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab1[i]=(char*)malloc((MAX)*sizeof(char));
 
  char **tab2=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab2[i]=(char*)malloc((MAX)*sizeof(char));

  char **tab3=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab3[i]=(char*)malloc((MAX)*sizeof(char));

  char **tab4=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab4[i]=(char*)malloc((MAX)*sizeof(char));

  /*fills tab with every partial EC number cut at given threshold in
    the network*/
  int nb_enzymes1 = myData->fill_tab_enzymes(1,tab1);
  int nb_enzymes2 = myData->fill_tab_enzymes(2,tab2);
  int nb_enzymes3 = myData->fill_tab_enzymes(3,tab3);
  myData->fill_tab_enzymes(4,tab4);
  
  /*declaration and memory allocation of set of colors*/
  int *color_choice1=(int*)malloc(motif_size*sizeof(int));
  int *color_choice2=(int*)malloc(motif_size*sizeof(int));
  int *color_choice3=(int*)malloc(motif_size*sizeof(int));
  int *color_choice4=(int*)malloc(motif_size*sizeof(int));
  for (int i=0;i<motif_size;i++)
    {
      color_choice1[i]=0;
      color_choice2[i]=0;
      color_choice3[i]=0;
      color_choice4[i]=0;
    }
  bool last_multichoice1 = 0;
  bool last_multichoice2 = 0;
  bool last_multichoice3 = 0;
  bool last_multichoice4 = 0;

  /*declaration and memory allocation of nb_of_children per partial EC
    numbers useful for skipping the motifs we know don't occur*/
  int *nb_children2=(int*)malloc(nb_enzymes2*sizeof(int));
  int *nb_children3=(int*)malloc(nb_enzymes3*sizeof(int));

  /*intermediate data structure needed cos easier to manipulate: it gives the correspondance between the indices of two consecutive tabs of partial EC numbers*/
  int *cum_sum_nb_children2=(int*)malloc((nb_enzymes2+1)*sizeof(int));
  int *cum_sum_nb_children3=(int*)malloc((nb_enzymes3+1)*sizeof(int));

  int *start=(int*)malloc(motif_size*sizeof(int)); //gives the start position of the block you want to traverse
  int *stop=(int*)malloc(motif_size*sizeof(int));
  bool startstop; //true if one of the start position is equal to its corresponding stop position

  int *start3=(int*)malloc(motif_size*sizeof(int)); //gives the start postion of the block you want to traverse
  int *stop3=(int*)malloc(motif_size*sizeof(int));
  bool startstop3; //true if one of the start position is equal to its corresponding stop position

  myData->fill_nb_children(2,nb_enzymes2,nb_children2);
  myData->fill_nb_children(3,nb_enzymes3,nb_children3);

  cum_sum_nb_children2[0]=0;
  for (int i=1;i<nb_enzymes2+1;i++)
    cum_sum_nb_children2[i]=cum_sum_nb_children2[i-1]+nb_children2[i-1];

  cum_sum_nb_children3[0]=0;
  for (int i=1;i<nb_enzymes3+1;i++)
    cum_sum_nb_children3[i]=cum_sum_nb_children3[i-1]+nb_children3[i-1];

  /*other structures needed on the way*/
  char *enzymes=(char*)malloc((MAX)*sizeof(char));
  
  /*needed to store intermediate solutions before printing*/
  int nbsol=0;
  float p=-1;
  float c=-1;
  float *p_s=NULL;
  int nb_observed_topologies=0;
  Solution_Group *all_solutions1=new Solution_Group();
  Solution_Group *all_solutions2=new Solution_Group();
  Solution_Group *all_solutions3=new Solution_Group();
  Solution_Group *all_solutions4=new Solution_Group();

  list<Solution_Group *> *topology_solutions=new list<Solution_Group *>;

  // /*In case we just need to perform a search from the end of a previous file*/
//   /*We need two parameters: the number of digits that stay fixed and the initial colours of the next digit: Example: fix=2 init=1 means that the two first digits*/
  
  // color_choice2[1]=1;
//   for (int i=2;i<motif_size;i++)
//     color_choice2[i]=5;

//   enzymes[0]='\0';  //reinitialize enzymes
//   for (int i=0;i<motif_size;i++)
//     {
//       enzymes = strcat(enzymes, tab2[color_choice2[i]]);
//       enzymes = strcat(enzymes, " ");
//     }
  
//   cout<<enzymes<<endl;
  //exit(0);

  /*If the threshold is 1, then we simply search for all possible
    motifs formed with EC numbers cut at threshold 1*/
  if (threshold==1)
    {
      while(!last_multichoice1)
	{
	  enzymes[0]='\0';  //reinitialize enzymes
	  for (int i=0;i<motif_size;i++)
	    {
	      enzymes = strcat(enzymes, tab1[color_choice1[i]]);
	      enzymes = strcat(enzymes, " ");
	    }
	  p_s=motif_search(myData, enzymes,all_solutions1, topology_solutions, simulations, whole_reaction_graph,&p,&c, p_s,&nb_observed_topologies);
	  nbsol=all_solutions1->size();
	  

	  /*print results*/
	  if (topology==0)
	    fprint_solutions(enzymes, all_solutions1,p,c,output);

	  else
	    fprint_solutions(enzymes, topology_solutions, p_s, output);

	  /*delete previous results*/
	  for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
	    delete(*it);
	  topology_solutions->clear(); //to clear out all these pending deleted groups
	  all_solutions1->delete_solutions();
	  if (p_s!=NULL)
	    delete[] p_s;

	  /*go to the next motif*/
	  last_multichoice1 = !next_multichoice(color_choice1,motif_size,nb_enzymes1);
	}
    }


  

  /*If the threshold is 2, 3 or 4, then we use a trick based on the EC
    numbers to speed up the process, namely, if 1.1 2.3 4.2 has no
    occurrence, then there is no need to search for 1.1.1 2.3.3 4.2.1
    or any of its brothers (block)*/
  /*start with s=2, then if it gives solution, traverse the
    corresponding block of s=3, do the same from s=3 to s=4*/
  /*additionally, we don't calculate useless p-values, i.e. when we
    are at threshold 2 and the final threshold is 3, we just need to
    know how many solutions there are, not the p-value*/
  else
    {
      while(!last_multichoice2)
	{
	  enzymes[0]='\0';  //reinitialize enzymes
	  for (int i=0;i<motif_size;i++)
	    {
	      enzymes = strcat(enzymes, tab2[color_choice2[i]]);
	      enzymes = strcat(enzymes, " ");
	    }

	  if (threshold==2)
	    {
	      //cerr<<enzymes<<endl;
	      p_s=motif_search(myData,enzymes,all_solutions2,topology_solutions, simulations, whole_reaction_graph,&p, &c,p_s, &nb_observed_topologies);
	      nbsol=all_solutions2->size();


	      //print_tab(p_s,nb_observed_topologies);

              /*print results*/
	      if (nbsol>0)
		{
		  /*coloured motifs*/
		  if (topology==0)
		    fprint_solutions(enzymes, all_solutions2,p,c,output);
		  /*topological motifs*/
		  else
		    fprint_solutions(enzymes, topology_solutions, p_s, output);
		}

	      /*delete previous results before moving to next (sub)-motif*/
	      for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
		delete(*it);
	      topology_solutions->clear(); //to clear out all these pending deleted groups
	      all_solutions2->delete_solutions();
	      if (p_s!=NULL)
		delete[] p_s;
	    }
	  
	  else  //we go on for threshold 3
	    {
	      /*if this is not the final threshold, we only count occurrences*/
	      nbsol=motif_search(myData, whole_reaction_graph, enzymes);

	      if (nbsol!=0)  //the nb of solutions is ok for threshold 2, let's try threshold 3
		{
		  
		  last_multichoice3=0;
		  startstop3=0;
		  for (int i=0;i<motif_size;i++)
		    {
		      start3[i]=cum_sum_nb_children2[color_choice2[i]];
		      stop3[i]=cum_sum_nb_children2[color_choice2[i]+1];
		      startstop3=(startstop3 || (start3[i]==stop3[i])); //in case there are no EC numbers with 4 numbers in the class considered
		      color_choice3[i]=start3[i];
		    }

		  while(!last_multichoice3 && !startstop3)  
		    {
		      enzymes[0]='\0';  //reinitialize enzymes
		      for (int i=0;i<motif_size;i++)
			{
			  enzymes = strcat(enzymes, tab3[color_choice3[i]]);
			  enzymes = strcat(enzymes, " ");
			}

		      
		      if (threshold==3)
			{
			  p_s=motif_search(myData,enzymes,all_solutions3, topology_solutions, simulations, whole_reaction_graph,&p,&c,p_s,&nb_observed_topologies);
			  nbsol=all_solutions3->size();
			  if(nbsol>0)
			    {
			      if (topology==0)
				fprint_solutions(enzymes, all_solutions3,p,c,output);
			      else
				fprint_solutions(enzymes, topology_solutions, p_s, output);
			    }


			  /*delete previous results before moving to next (sub)-motif*/
			  for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
			    delete(*it);
			  topology_solutions->clear(); //to clear out all these pending deleted groups
			  all_solutions3->delete_solutions();
			  if (p_s !=NULL)
			    delete[] p_s;
			}
		      else
			{
			  nbsol=motif_search(myData,whole_reaction_graph,enzymes);
			  if (nbsol!=0)  //the nb of solutions is ok for threshold 3, let's try threshold 4
			    {
			      last_multichoice4=0;
			      startstop=0;
			      for (int i=0;i<motif_size;i++)
				{
				  start[i]=cum_sum_nb_children3[color_choice3[i]];
				  stop[i]=cum_sum_nb_children3[color_choice3[i]+1];
				  startstop=(startstop || (start[i]==stop[i])); //in case there are no EC numbers with 4 numbers in the class considered
				  color_choice4[i]=start[i];
				}
			      
			      while(!last_multichoice4 && !startstop)  
				{
				  enzymes[0]='\0';  //reinitialize enzymes
				  for (int i=0;i<motif_size;i++)
				    {
				      enzymes = strcat(enzymes, tab4[color_choice4[i]]);
				      enzymes = strcat(enzymes, " ");
				    }
				  
				  p_s=motif_search(myData,enzymes,all_solutions4, topology_solutions, simulations, whole_reaction_graph,&p, &c, p_s,&nb_observed_topologies);
				  nbsol=all_solutions4->size();
				  				  
				  /*print results*/				  
				  if(nbsol>0)
				    {
				      if (topology==0)
					fprint_solutions(enzymes, all_solutions4,p,c,output);
				      else
					fprint_solutions(enzymes, topology_solutions, p_s, output);
				    }


				  /*delete previous results before moving to next (sub)-motif*/
				  for(list<Solution_Group *>::iterator it=topology_solutions->begin();it!=topology_solutions->end();it++)
				    delete(*it);
				  topology_solutions->clear(); //to clear out all these pending deleted groups
				  all_solutions4->delete_solutions();
				  if (p_s !=NULL)
				    delete[] p_s;

				  last_multichoice4 = !next_multichoice_in_block(color_choice4,motif_size,start,stop);
				}
			    }
			  else //jump to next class of EC numbers , level 3
			    {
			    }
			}
		      
		      //whatever if threshold is 3 or 4, we move to the next choice at level 3
		      // all_solutions3->delete_solutions();
// 		      if (p_s !=NULL)
// 			delete[] p_s;
		      last_multichoice3=!next_multichoice_in_block(color_choice3,motif_size,start3,stop3);
		      
		    }
		}
	      
	      else //jump level 2
		{
		}
	    }
	  // all_solutions2->delete_solutions();
// 	  if (p_s !=NULL)
// 	    delete[] p_s;
	  time_display_flag++;
	  nb_motifs_done_old = nb_motifs_done;
	  nb_motifs_done = get_nb_motifs_done(color_choice2, motif_size, nb_enzymes2);
	  nb_motifs_total = multichoose(motif_size,nb_enzymes2);

	  // if (nb_motifs_done != nb_motifs_done_old)
// 	    cerr<<".";
	  if (time_display_flag == nb_motifs_total/10)
	    {
	      time(&curr_time);
	      elapsed_time = difftime( curr_time, strt_time );
	      //cout<<"elapsed time "<<elapsed_time<<endl;
	      //output2<<nb_motifs_done<<" "<<elapsed_time<<endl;
	      //cout<<"nb motifs total "<<nb_motifs_total<<endl;
	      //cout<<"nb motifs done "<<nb_motifs_done<<endl;
	      estimated_time = elapsed_time * (nb_motifs_total - nb_motifs_done) / nb_motifs_done;
	      
	      if (estimated_time > 18000) {
		//display message in hours
		estimated_time /=3600;
		printf("\n Estimated run time left : %6.0f hours.\n", estimated_time);
	      }
	      else if (estimated_time > 300) {
		//display message in minutes
		estimated_time /=60;
		printf("\n Estimated run time left : %6.0f minutes.\n", estimated_time);
	      }
	      else {
		//display message in seconds
		printf("\n Estimated run time left : %6.0f seconds.\n", estimated_time );
	      }
	      time_display_flag=0;
	    }
	      
	  last_multichoice2 = !next_multichoice(color_choice2,motif_size,nb_enzymes2);
	}
    }

  /*free memory*/
  free(color_choice1);
  free(color_choice2);
  free(color_choice3);
  free(color_choice4);
  free(enzymes);
  for (int i=0; i<MAX_NB_EC; i++)
    {
      free(tab1[i]);
      free(tab2[i]);
      free(tab3[i]);
      free(tab4[i]);
    }
  free(tab1);
  free(tab2);
  free(tab3);
  free(tab4);
  free(nb_children2);
  free(nb_children3);
  free(cum_sum_nb_children2);
  free(cum_sum_nb_children3);
  free(start);
  free(stop);
  free(start3);
  free(stop3);
  output.close();

  delete all_solutions1;
  delete all_solutions2;
  delete all_solutions3;
  delete all_solutions4;

  delete topology_solutions;
}

/*New function for motif inference in any vertex-labelled graph*/
/*If colours are not derived form a hierarchy, no trick can be applied*/
/*Inference algorithm is simply a series of search*/
void Io::motif_inference(Graph *graph, int motif_size, Graph **randomized_graphs)
{
  /*file writing*/

  std::ofstream output;
  output.open(results_file);

  output<<"!Each line gives the motif, the number of occurrences, the p-value for the number of occurrences, the number of clumps of occurrences, the p-value for the number of clumps."<<endl;

  /*needed structures*/
  bool last_multichoice=0;
  int *color_choice=new int[motif_size];
  for (int i=0;i<motif_size;i++)
    color_choice[i]=0;
  int *motif=new int[motif_size];
  int nb_colors=graph->get_nb_colors();
  int *all_colors=new int[nb_colors];
  int nbsol=0;  

  /*first get all labels*/
  graph->get_all_colors(all_colors);

  // cout<<"all colours are ";
//   print_tab(all_colors,nb_colors);

  /*then generate all motifs and search for them*/
  while(!last_multichoice)
	{

	  for (int i=0;i<motif_size;i++)
	    motif[i]=all_colors[color_choice[i]];

       	  /*I could store this in a function named motif_search*/
	  Graph *filtered_graph = new Graph(graph,motif,motif_size);
	  // nbsol = filtered_graph->enumerate(motif_size,motif);
	  list<int *> *occurrences = new list<int *>;
	  filtered_graph->enumerate(occurrences,motif_size,motif);
	  nbsol=occurrences->size();
	  int nb_clumps=get_nb_clumps(occurrences,motif_size);

	  //float p=calculate_p_value(randomized_graphs,motif,motif_size,nbsol);
	 
	  float p,c;
	  calculate_p_c_value(randomized_graphs,motif,motif_size,nbsol,nb_clumps,&p,&c);

	  /*print solutions if any*/
	  if (nbsol>0)
	    {
	      print_tab(motif,motif_size,output);
	      output<<nbsol<<" "<<p<<" "<<nb_clumps<<" "<<c<<endl;
	    }

	  /*delete objects*/
	  delete filtered_graph;
	  for (list<int *>::iterator it=occurrences->begin(); it!=occurrences->end(); it++)
	    {
	      delete[] (*it);
	      it=occurrences->erase(it);
	      it--;
	    } 
	  delete occurrences;
	  
	  last_multichoice = !next_multichoice(color_choice,motif_size,nb_colors);
	}

  cout<<"Results are printed in file "<<results_file<<endl;
  output.close();
  delete[] color_choice;
  delete[] motif;
  delete[] all_colors;
}


/*This function is very much similar to the first function
  motif_inference. The main difference is that the computation is done
  here on a general graph and not on a reaction graph. Therefore, the
  only thing we do is to calculate the number of occurrences, we do
  not output the occurrences.*/
/*One should verify if this works for every threshold*/
void Io::motif_inference(Data *myData, Graph *graph, int threshold, int motif_size)
{
  /*file writing*/
  std::ofstream output;
  output.open(results_file);

  /*time counting*/
  clock_t begin, end;
  double cpu_time_used;
    
  begin = clock();
  //... /* Do the work. */
  end = clock();
  cpu_time_used = ((double) (end - begin)) / CLOCKS_PER_SEC;
  
  /*declaration, memory allocation and initialisation of the tab of enzymes I'm going to use*/
  char **tab1=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab1[i]=(char*)malloc((MAX)*sizeof(char));

  char **tab2=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab2[i]=(char*)malloc((MAX)*sizeof(char));

  char **tab3=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab3[i]=(char*)malloc((MAX)*sizeof(char));

  char **tab4=(char**)malloc((MAX_NB_EC)*sizeof(char*));
  for (int i=0; i<MAX_NB_EC; i++)
    tab4[i]=(char*)malloc((MAX)*sizeof(char));


  /*fills tab with every partial EC number cut at given threshold in the network*/
  int nb_enzymes1 = myData->fill_tab_enzymes(1,tab1);
  int nb_enzymes2 = myData->fill_tab_enzymes(2,tab2);
  int nb_enzymes3 = myData->fill_tab_enzymes(3,tab3);
  myData->fill_tab_enzymes(4,tab4);
  //int nb_enzymes4 = myData->fill_tab_enzymes(4,tab4);

  /*declaration of Graphs, one for each threshold*/
  //Graph *graph2 = new Graph(graph, tab2, nb_enzymes2);
  //Graph *graph3 = new Graph(graph_file, labels_file, tab3, nb_enzymes3);
  

  /*declaration and memory allocation of set of colors*/
  int *color_choice1=(int*)malloc(motif_size*sizeof(int));
  int *color_choice2=(int*)malloc(motif_size*sizeof(int));
  int *color_choice3=(int*)malloc(motif_size*sizeof(int));
  int *color_choice4=(int*)malloc(motif_size*sizeof(int));
  for (int i=0;i<motif_size;i++)
    {
      color_choice1[i]=0;
      color_choice2[i]=0;
      color_choice3[i]=0;
      color_choice4[i]=0;
    }
  bool last_multichoice1 = 0;
  bool last_multichoice2 = 0;
  bool last_multichoice3 = 0;
  bool last_multichoice4 = 0;

  /*declaration and memory allocation of nb_of_children per partial EC numbers useful for skipping the motifs we know don't occur*/
  int *nb_children2=(int*)malloc(nb_enzymes2*sizeof(int));
  int *nb_children3=(int*)malloc(nb_enzymes3*sizeof(int));

  /*intermediate data structure needed cos easier to manipulate: it gives the correspondance between the indices of two consecutive tabs of partial EC numbers*/
  int *cum_sum_nb_children2=(int*)malloc((nb_enzymes2+1)*sizeof(int));
  int *cum_sum_nb_children3=(int*)malloc((nb_enzymes3+1)*sizeof(int));

  /*reverse structure gives the correspondance between the indices of two consecutive tabs of partial EC numbers*/
  int *father_index3=new int[nb_enzymes3+1];

  int *start=(int*)malloc(motif_size*sizeof(int)); //gives the start postion of the block you want to traverse
  int *stop=(int*)malloc(motif_size*sizeof(int));
  bool startstop; //true if one of the start position is equal to its corresponding stop position

  int *start3=(int*)malloc(motif_size*sizeof(int)); //gives the start postion of the block you want to traverse
  int *stop3=(int*)malloc(motif_size*sizeof(int));
  bool startstop3; //true if one of the start position is equal to its corresponding stop positio

  myData->fill_nb_children(2,nb_enzymes2,nb_children2);
  myData->fill_nb_children(3,nb_enzymes3,nb_children3);

  cum_sum_nb_children2[0]=0;
  for (int i=1;i<nb_enzymes2+1;i++)
    cum_sum_nb_children2[i]=cum_sum_nb_children2[i-1]+nb_children2[i-1];

  cum_sum_nb_children3[0]=0;
  for (int i=1;i<nb_enzymes3+1;i++)
    cum_sum_nb_children3[i]=cum_sum_nb_children3[i-1]+nb_children3[i-1];

  /*initialisation of father index*/
  for (int i=0;i<nb_enzymes2;i++)
    {
      for (int j=cum_sum_nb_children2[i];j<cum_sum_nb_children2[i+1];j++)
	father_index3[j]=i;
    }
  //print_tab(father_index3,nb_enzymes3);

  /*creation of a graph identical to the previous one but with labels adjusted to a lower threshold*/
  Graph *graph1=new Graph(graph);
  graph1->reset_colors(myData->correspondance_colours,1);
  Graph *graph2=new Graph(graph);
  graph2->reset_colors(myData->correspondance_colours,2);
  Graph *graph3=new Graph(graph);
  graph3->reset_colors(myData->correspondance_colours,3);
  Graph *graph4=new Graph(graph);
  graph4->reset_colors(myData->correspondance_colours,4);

  //exit(0);
  char *enzymes=(char*)malloc((MAX)*sizeof(char));
  int nbsol=0;


  /*If the threshold is 1, then we simply search for all possible motifs formed with EC numbers cut at threshold 1*/
  if (threshold==1)
    {
      while(!last_multichoice1)
	{
	  enzymes[0]='\0';  //reinitialize enzymes
	  for (int i=0;i<motif_size;i++)
	    {
	      enzymes = strcat(enzymes, tab1[color_choice1[i]]);
	      enzymes = strcat(enzymes, " ");
	    }

	  //nbsol=motif_search(graph,color_choice1,motif_size);
	  nbsol=motif_search(myData,graph1,enzymes);
	  fprint_nb_solutions(enzymes,nbsol,output);
	  last_multichoice1 = !next_multichoice(color_choice1,motif_size,nb_enzymes1);
	}
    }


  /*If the threshold is 2, 3 or 4, then we use a trick based on the EC
    numbers to speed up the process, namely, if 1.1 2.3 4.2 has no
    occurrence, then there is no need to search for 1.1.1 2.3.3 4.2.1
    or any of its brothers (block)*/
  /*start with s=2, then if it gives solution, traverse the
    corresponding block of s=3, do the same from s=3 to s=4*/
  else
    {
      while(!last_multichoice2)
	{
	  enzymes[0]='\0';  //reinitialize enzymes
	  for (int i=0;i<motif_size;i++)
	    {
	      enzymes = strcat(enzymes, tab2[color_choice2[i]]);
	      enzymes = strcat(enzymes, " ");
	    }
	  //nbsol=motif_search(graph2,color_choice2,motif_size);
	  nbsol=motif_search(myData,graph2,enzymes);
	  	  
	  if (threshold==2)
	    {
	      if (nbsol>0)
		fprint_nb_solutions(enzymes, nbsol,output);
	    }
	  
	  else  //we go on for threshold 3
	    {
	      if (nbsol!=0)  //the nb of solutions is ok for threshold 2, let's try threshold 3
		{
		  
		  last_multichoice3=0;
		  startstop3=0;
		  for (int i=0;i<motif_size;i++)
		    {
		      start3[i]=cum_sum_nb_children2[color_choice2[i]];
		      stop3[i]=cum_sum_nb_children2[color_choice2[i]+1];
		      startstop3=(startstop3 || (start3[i]==stop3[i])); //in case there are no EC numbers with 4 numbers in the class considered
		      color_choice3[i]=start3[i];
		    }
		  // cout<<"level 3 start :";
		  // 	  print_tab(start3,motif_size);
		  // 	  cout<<"level 3 stop :";
		  // 	  print_tab(stop3,motif_size);
		  while(!last_multichoice3 && !startstop3)  
		    {
		      enzymes[0]='\0';  //reinitialize enzymes
		      for (int i=0;i<motif_size;i++)
			{
			  enzymes = strcat(enzymes, tab3[color_choice3[i]]);
			  enzymes = strcat(enzymes, " ");
			}
		      //Solution_Group *all_solutions3 = new Solution_Group();
		      //cout<<enzymes<<endl;
		      //motif_search(myData,enzymes,all_solutions3);
		      //cout<<enzymes<<endl;
		      //nbsol=motif_search(graph,color_choice3,motif_size);
		      nbsol=motif_search(myData,graph3,enzymes);

		      //all_solutions3->delete_solutions();
		      //delete(all_solutions3);
		      
		      if (threshold==3)
			{
			  if(nbsol>0)
			    fprint_nb_solutions(enzymes,nbsol,output);
			}
		      else
			{
			  
			  if (nbsol!=0)  //the nb of solutions is ok for threshold 3, let's try threshold 4
			    {
			      last_multichoice4=0;
			      startstop=0;
			      for (int i=0;i<motif_size;i++)
				{
				  start[i]=cum_sum_nb_children3[color_choice3[i]];
				  stop[i]=cum_sum_nb_children3[color_choice3[i]+1];
				  startstop=(startstop || (start[i]==stop[i])); //in case there are no EC numbers with 4 numbers in the class considered
				  color_choice4[i]=start[i];
				}
			      // cout<<"start :";
			      // 		  print_tab(start,motif_size);
			      // 		  cout<<"stop :";
			      // 		  print_tab(stop,motif_size);
			      while(!last_multichoice4 && !startstop)  
				{
				  enzymes[0]='\0';  //reinitialize enzymes
				  for (int i=0;i<motif_size;i++)
				    {
				      enzymes = strcat(enzymes, tab4[color_choice4[i]]);
				      enzymes = strcat(enzymes, " ");
				    }
				  last_multichoice4 = !next_multichoice_in_block(color_choice4,motif_size,start,stop);
				  
				  //nbsol=motif_search(graph,color_choice4,motif_size);
				  nbsol=motif_search(myData,graph4,enzymes);

				  //nbsol=all_solutions4->size();
				  //cout<<"nbsol="<<nbsol<<endl;
				  if (nbsol>0)
				    fprint_nb_solutions(enzymes, nbsol,output);
				  
				  //all_solutions4->delete_solutions();
				  //delete(all_solutions4);
				}
			    }
			  else //jump to next class of EC numbers , level 3
			    {
			    }
			}
		      
		      //whatever if threshold is 3 or 4, we move to the next choice at level 3
		      last_multichoice3=!next_multichoice_in_block(color_choice3,motif_size,start3,stop3);
		      
		    }
		}
	      
	      else //jump level 2
		{
		}
	    }
	  last_multichoice2 = !next_multichoice(color_choice2,motif_size,nb_enzymes2);
	}
    }

  output.close();

  /*free memory*/
  delete graph1;
  delete graph2;
  delete graph3;
  delete graph4;
  free(color_choice1);
  free(color_choice2);
  free(color_choice3);
  free(color_choice4);
  free(enzymes);
  for (int i=0; i<MAX_NB_EC; i++)
    {
      free(tab1[i]);
      free(tab2[i]);
      free(tab3[i]);
      free(tab4[i]);
    }
  free(tab1);
  free(tab2);
  free(tab3);
  free(tab4);
  free(nb_children2);
  free(nb_children3);
  free(cum_sum_nb_children2);
  free(cum_sum_nb_children3);
  free(start);
  free(stop);
  free(start3);
  free(stop3);
  delete father_index3;
}
