/* Occurrence.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Occurrence.h"
#include <iostream>
#include <fstream>
#include <cstdlib>


Occurrence::Occurrence(int *tab, int size)
{
  nodes=new int[size];
  this->size=size;
  for (int i=0;i<size;i++)
    nodes[i]=tab[i];
  adj_matrix=NULL;
}

/*Creates an occurrence, giving the correspondance between the original nodes and the filtered nodes in the graph*/
Occurrence::Occurrence(int *tab, int size, int *parent_nodes)
{
  // cerr<<"before creating occurrence, sol is"<<endl;
//   print_tab(tab,size);
  nodes=new int[size];
  this->size=size;
  for (int i=0;i<size;i++)
    //nodes[i]=tab[parent_nodes[i]];
    nodes[i]=parent_nodes[tab[i]];
  adj_matrix=NULL;

  // cerr<<"after creating occurrence, sol is"<<endl;
//   print_tab(nodes,size);

}


Occurrence::~Occurrence()
{
  delete[] nodes;
  if (adj_matrix!=NULL)
    for (int i=0;i<size;i++)
      delete[] adj_matrix[i];
  delete[] adj_matrix;
}

/**
 * Get the first hex number characterising the canonical labelling of
 * the induced graph. Useful for solving efficiently graph
 * isomorphism problems.
 *
 * @return topo1 first hex number characterising the canonical labelling of
 * the induced graph.
 *
 */
long int Occurrence::get_topo1()
{
  return topo1;
}


/**
 * Get the second hex number characterising the canonical labelling of
 * the induced graph. Useful for solving efficiently graph
 * isomorphism problems.
 *
 * @return topo2 second hex number characterising the canonical labelling of
 * the induced graph.
 *
 */
long int Occurrence::get_topo2()
{
  return topo2;
}


/**
 * Recovers the topology of the occurrence in the original graph. Note that node indices have to be coherent between the occurrence and the graph
 *
 * @param whole_graph graph corresponding to the whole network
 *
 */
void Occurrence::set_adj_matrix(Graph * whole_graph)
{
  /*if not allocated, then allocate*/
  if (adj_matrix==NULL)
    {
      adj_matrix = new bool *[size];
      for (int i=0;i<size;i++)
	adj_matrix[i] = new bool[size];
    }

  for (int i=0;i<size;i++)
    for (int j=0;j<size;j++)
      adj_matrix[i][j]=whole_graph->get_adjacency_matrix()[nodes[i]][nodes[j]];


  /*fill reaction_colors*/
  colors = new list<int> *[size];
  for (int j=0;j<size;j++)
    {
      colors[j]=new list<int>;
      copy(whole_graph->get_colors()[nodes[j]]->begin(),whole_graph->get_colors()[nodes[j]]->end(),back_inserter(*colors[j]));
    }


 //  cerr<<"before leaving function"<<endl;

//   print_adj_matrix();
}


/**
 * Print the graph to file in dreadnaut format. The first nodes are the compounds, they are the source nodes for each edge. Each time, we want to change the source node we print ";". Finally, we print a "." when the job is finished.
 *
 */
void Occurrence::print_dreadnaut(ostream &output)
{
  /*print graph*/
  output<<"n="<<size<<" g"<<endl;
  for (int i=0;i<size;i++)
    {
      for (int j=0;j<size;j++)
	{
	  if (adj_matrix[i][j])
	    output<<j<<" ";
	}
      /*end of the graph*/
      if (i==size-1)
	output<<"."<<endl;
      /*end of the line*/
      else
	output<<";"<<endl;
    }

  /*print permutation - i.e. vertex colours - for the moment only compounds and reactions are separated*/
  //output<<"f=[0:"<<nb_compounds-1<<"]"<<endl;

}

/**
 * Print the graph to file in dreadnaut format. The first nodes are the compounds, they are the source nodes for each edge. Each time, we want to change the source node we print ";". Finally, we print a "." when the job is finished.
 *
 */
void Occurrence::print_dreadnaut_ordered(ostream &output)
{
  /*print graph*/
  output<<"n="<<size<<" g"<<endl;
  for (int i=0;i<size;i++)
    {
      for (int j=0;j<size;j++)
	{
	  if (adj_matrix[i][j])
	    output<<j<<" ";
	}
      /*end of the graph*/
      if (i==size-1)
	output<<"."<<endl;
      /*end of the line*/
      else
	output<<";"<<endl;
    }
  
  
  /*print permutation - i.e. vertex colours - for the moment only compounds and reactions are separated*/
  /*You can use the abbreviation \x:y" for the range x; x+1; : : : ; y. Any vertices not
    named are put in a cell of their own at the end.*/
  /*Example: If n = 10, then f=[3:7 | 0,2]" establishes the partition
    [3, 4, 5, 6, 7 | 0, 2 | 1, 8, 9].*/
  /*note that the f=[3:7 | 0,2] and explicitely setting f=[3, 4, 5, 6, 7 | 0, 2 | 1, 8, 9] will not generate the same codes. This problem happened only once and I could not find it again when trying (not so hard).*/
  
  /*In order to tell dreadnaut what the colour groups are, we need to
    build a reverse structure telling us for each colour, which nodes
    have it*/
  /*For this purpose, we first compute the number of unique colors,
    which we store in a tab: unique_colors*/
  /*Second, we fill the tab of lists: colors_reactions which can be
    used directly to tell dreadnaut the color_classes*/


  /*get number of distinct colours*/
  list<int> reactions_colors; //one reaction has only one colour

  /*convert reaction colors into a list, keeping only the first color
    of each node*/
  for (int i=0;i<size;i++)
    reactions_colors.push_back(colors[i]->front());

  //print_list(reactions_colors.begin(),reactions_colors.end());

  reactions_colors.sort();
  reactions_colors.unique();
  int nb_colors=reactions_colors.size();

  /*convert list to tab*/
  int *unique_colors = new int[nb_colors];
  int i=0;
  for (list<int>::iterator it=reactions_colors.begin(); it!= reactions_colors.end(); it++)
    {
      unique_colors[i]=(*it);
      i++;
    }


  // cout<<"le tableau des couleurs est "<<endl;
//   print_tab(unique_colors,nb_colors);

  list<int> *colors_reactions= new list<int>[nb_colors];
  for (int i=0;i<nb_colors;i++)
    {
      /*get nodes with color unique_colors[i]*/
      for (int j=0;j<size;j++)
	if (colors[j]->front()==unique_colors[i])
	  colors_reactions[i].push_back(j);
    }


  // cout<<" la fin, "<<endl;
//   for (int i=0;i<nb_colors;i++)
//     print_list(colors_reactions[i].begin(), colors_reactions[i].end());



  /*print colors for dreadnaut*/
  //output<<"f=[0:"<<nb_compounds-1<<"|";
  output<<"f=[";

  for (int i=0;i<nb_colors;i++)
    {
      /*print all nodes sharing the same color*/
      for (list<int>::iterator it=colors_reactions[i].begin(); it!=colors_reactions[i].end(); it++)
	{
	  output<<*it;
	  it++;
	  if (it!=colors_reactions[i].end())
	    output<<",";
	  it--;
	}
      if (i==nb_colors-1)
	output<<"]"<<endl;
      else
	output<<"|";
    }


  delete[] unique_colors;
  delete[] colors_reactions;
  //exit(0);

}





/**
 * Computes the canonical labelling of the bipartite graph. This
 * operation is done by nauty (No AUTomorphism, Yes ?). The result is
 * stored as two hexadecimal numbers. In case the bipartite graph has
 * not been extracted, this operation is not possible.
 *
 */
void Occurrence::compute_canonical_labelling(bool order)
{
  /*In case the graph has not been extracted, this operation is not possible.*/
  // if (graph == NULL)
  if (adj_matrix == NULL)
    {
      cerr<<"you are trying to compute the canonical labelling of a solution that has no corresponding graph"<<endl;
      exit(0);
    }

  /*write dreadnaut query file*/
  std::ofstream output;
  // char *file_name=new char[MAX];
//   // file_name[0]='\0';
//   strcpy(file_name,write_path);
//   file_name=strcat(file_name,"/hop");
  output.open("query.dre");
  
  if (!order)
    print_dreadnaut(output);
  
  else
    print_dreadnaut_ordered(output);

  /*tell nauty to compute a canonical labelling*/
  output<<"c"<<endl;
  /*do not print automorphisms and markers*/
  output<<"-a -m";
  /*execute nauty (compute automorphisms and canonical labelling)*/
  output<<" x";
  /*print two digit hex numbers that depend only on the graph with canonical labelling*/
  output<<" z"<<endl;
  output.close();


  /*execute dreadnaut*/
  int error = system("../nauty24b7/dreadnaut < query.dre > dreadnaut.res");
  if (error == -1)
    {
      cerr<<"error while executing dreadnaut - most probably dreadnaut is not installed in your computer - you can get it from http://cs.anu.edu.au/~bdm/nauty/ install it on your computer and then copy the exec file 'dreadnaut' in the Motus bin directory."<<endl;
      exit(0);
    }

  /*parse the output of dreadnaut and store it in long ints*/
  char str[200];
  char word[8];
  std::ifstream file_op("dreadnaut.res");
  /*ignore the first 3 lines and consider only the 4th*/
  for (int i=0;i<4;i++)
    file_op.getline(str,200);

  //cerr<<"str "<<str<<endl;

  /*parsing the string containing the 2 hexadecimal numbers*/
  //  cout<<"the string i am about to parse"<<str<<endl;
  
  /*skip the first [ and the blanks*/
  int i=1; 
  while (str[i]==' ')
    i++;

  /*first word to extract: useless prefix ?*/
  extract_word(str, word, ' ',&i);
  
  //cerr<<"word1 "<<word<<endl;

  /*skip occasional blanks (when the first hex number contains only 6 digits for example*/
  while (str[i]==' ')
    i++;

  /*second word to extract: first hex number characterising the
    topology of the graph*/
  extract_word(str,word, ' ',&i);
  //cerr<<"word2 "<<word<<endl;
  topo1=strtol(word,(char**)NULL,16) ;

  /*skip occasional blanks (when the second hex number contains only 6 digits for example*/
  while (str[i]==' ')
    i++;

  /*third word to extract: second hex number characterising the
  topology of the graph*/
  extract_word(str,word, ']',&i);
  //cerr<<"word3 "<<word<<endl;
  topo2=strtol(word,(char**)NULL,16) ;

  //cout<<"topo1 :"<<topo1<<endl;
  //cout<<"topo2 :"<<topo2<<endl;
 
  file_op.close();
  //exit(0);
  system("rm query.dre dreadnaut.res");

}
/**
 * Compares the topologies of two solutions. If the codes
 * characterising their canonical labellings are both identical (it
 * means that the graphs are isomorphic), then the method returns
 * true.
 *
 * @param sol_to_compare Solution to compare to this solution
 * @return boolean set to true if solutions have the same topology
 *
 */
bool Occurrence::same_topology(Occurrence *occ_to_compare)
{
  return (occ_to_compare->get_topo1() == topo1 && occ_to_compare->get_topo2() == topo2);
}



/*print the solution in a file*/
void Occurrence::print(std::ostream& output)
{
  for (int i=0;i<size;i++)
    output<<nodes[i]<<" ";
  // output<<endl;
}

/*print the solution*/
void Occurrence::print()
{
  for (int i=0;i<size;i++)
    cout<<nodes[i]<<" ";
 //  cout<<endl;
}

/*print the matrix in a file*/
void Occurrence::print_adj_matrix(std::ostream& output)
{
  for (int i=0;i<size;i++)
    {
      for (int j=0;j<size;j++)
	output<<adj_matrix[i][j];
      output<<endl;
    }
  // output<<endl;
}

/*print the matrix*/
void Occurrence::print_adj_matrix()
{
  for (int i=0;i<size;i++)
    {
      for (int j=0;j<size;j++)
	cout<<adj_matrix[i][j];
      cout<<endl;
    }
  //cout<<endl;
}
