/* Solution.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Solution.h"
#include "../include/Pathway.h"
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <algorithm>

/*constructor used with the old version of enumeration when only linear occurrences were enumerated*/
Solution::Solution(list<Reaction *> *rea_list)
{
  reactions=new list<Reaction *>;
  min_pathways=new list<Pathway *>;
  max_pathways=new list<Pathway *>;
  inner_compounds=new list<list<Compound *> *>;
  compound_matrix=NULL;
  /*copy the list of reactions*/   
  list<Reaction *>::iterator rea_iter;
  rea_iter =  rea_list->begin();
  while (rea_iter != rea_list->end())
    {
      reactions->push_back(*rea_iter);
      rea_iter++;
    }

  
  calculate_max_pathways();
  calculate_min_pathways();
  fill_inner_compounds();   
}


/**
 *Creates a Solution from a list of integers, the corresponding list
 *of reactions (or the full list) and a correspondance array between
 *the reduced and the full list of reactions.
 *
 * @param rea_tab0 indices in the of reactions forming the occurrence
 * @param size0 motif size (number of reactions in the occurrence
 * @param tab_reactions array of reactions in which indices of rea_tab0 are taken. It can either be the full list of reactions or a partial list (in the case of a filtered graph)
 * @param parent_nodes correspondance array of indices of reactions in the filtered graph and in the full list of reactions. In the case where tab_reactions represents the full list of reactions, this array is the identity permutation.
 */
Solution::Solution(int *rea_tab0, int size0, Reaction **tab_reactions, int *parent_nodes)
{
  /*Copy the array of integers*/
  // cerr<<"avant reat_ab, size="<<size0<<endl;
//   print_tab(parent_nodes, 10);
  this->size=size0;
  this->rea_tab=new int[size];
  for (int i=0;i<size;i++)
    this->rea_tab[i]=parent_nodes[rea_tab0[i]];
  

//   print_tab(rea_tab0, size0);

//   print_tab(this->rea_tab, this->size);

  /*Allocate memory*/
  reactions=new list<Reaction *>;
  min_pathways=new list<Pathway *>;
  //cerr<<"constructeur Solution "<<min_pathways->size()<<endl;
  max_pathways=new list<Pathway *>;
  //inner_compounds=new list<list<Compound *> *>;
  inner_compounds=NULL;
  /*allocation of compound matrix*/
  compound_matrix=new list<Compound *> **[size];
  for (int i=0;i<size;i++)
    compound_matrix[i]=new list<Compound *> *[size];
  /*initialization of compound matrix*/
  for (int i=0;i<size;i++)
    for (int j=0;j<size;j++)
      compound_matrix[i][j]=NULL;

  /*copy the list of reactions*/ 
  for (int i=0;i<size;i++)
    reactions->push_back(tab_reactions[rea_tab0[i]]);
  
 //  for (int i=0;i<size;i++)
//     reactions->push_back(tab_reactions[i]);

  /*calculate stuff*/  
  /*this is useful when you want to print several levels of detail
    like EC numbers and compounds*/
  calculate_max_pathways();
  calculate_min_pathways();
  first_reaction=NULL;
  last_reaction=NULL;
  bipartite_graph=NULL;
  fill_compound_matrix();
  //cerr<<"fin craction solutions"<<endl;

  /*initialising the topological characterizers of the graph*/
  topo1=-1;
  topo2=-1;
}



Solution::~Solution()
{
  //cerr<<"clearing one solution"<<endl;
  // reactions->clear();
//   max_pathways->clear();
//   min_pathways->clear();
  //int size=reactions->size();
  
  delete(reactions);
  delete[] rea_tab;
  delete(max_pathways);
  delete(min_pathways);
  if(inner_compounds!=NULL)
    {
      for (list<list<Compound *> *>::iterator it=inner_compounds->begin();it!=inner_compounds->end();it++)
	delete(*it);
      delete(inner_compounds);
    }
  if (compound_matrix!=NULL)
    {
      for (int i=0;i<size;i++)
	{
	  for (int j=0;j<size;j++)
	    if(compound_matrix[i][j]!=NULL)
	      delete(compound_matrix[i][j]);
	  delete[](compound_matrix[i]);
	}
      delete[](compound_matrix);
    }

  if (bipartite_graph!=NULL)
    delete bipartite_graph;

}

/**
 * Get the number of reactions in the occurrence.
 *
 * @return number of reactions in the occurrence.
 *
 */
int Solution::get_size()
{
  return reactions->size();
}

/**
 * Get the list of reactions of the occurrence
 *
 * @return list of reactions of the occurrence
 *
 */
list<Reaction *> *Solution::get_reactions()
{
  return reactions;
}

/**
 * Get the first hex number characterising the canonical labelling of
 * the induced bipartite graph. Useful for solving efficiently graph
 * isomorphism problems.
 *
 * @return topo1 first hex number characterising the canonical labelling of
 * the induced bipartite graph.
 *
 */
long int Solution::get_topo1()
{
  return topo1;
}


/**
 * Get the second hex number characterising the canonical labelling of
 * the induced bipartite graph. Useful for solving efficiently graph
 * isomorphism problems.
 *
 * @return topo2 second hex number characterising the canonical labelling of
 * the induced bipartite graph.
 *
 */
long int Solution::get_topo2()
{
  return topo2;
}


long int Solution::get_topo(bool order)
{
  /*We have to give the topology without order*/
  compute_canonical_labelling(order);
  long int topo=topo1+topo2;
  return topo;

  // bipartite_graph->get_topo();
}



/*this is for filtering solutions by input_compound and output_compound*/
bool Solution::contains_similar_compounds(Compound *cpd1, Compound *cpd2, int threshold)
{
  /*declaration + initialization*/
  list<Reaction *>::iterator rea_iter;
  bool *matched_compounds = (bool *)malloc(2*sizeof(bool));
  for (int i=0;i<2;i++)
    matched_compounds[i]=0;

  /*processing*/
  rea_iter =  reactions->begin();
  while (rea_iter != reactions->end())
    {
      (*rea_iter)->contains_similar_compounds(cpd1, cpd2, matched_compounds, threshold);
      rea_iter++;
    }

  /*return + free*/
  bool out = (matched_compounds[0] && matched_compounds[1]);
  free(matched_compounds);
  return out;
}

/*this function is identical to the one in Reaction_Graph.cpp*/
/*intersection of two lists*/
/*this operation would be useless if the edges of the graph were labelled with the compounds shared by the reactions*/
void intersect2(list<Compound *> *l1, list<Compound *> *l2, list<Compound *> *l_inter)  
{
  l_inter->clear();
  //  cerr<<"les tailles des listes sont :"<<l1->size()<<l2->size()<<l_inter->size()<<endl;
  for (list<Compound *>::iterator it1 = l1->begin(); it1!=l1->end(); it1++)
    {
      for (list<Compound *>::iterator it2 = l2->begin(); it2!=l2->end(); it2++)
	{
	  if (*it1 == *it2)
	    {
	      //(*it1)->print();
	      //Compound *c = *it1;
	      //l_inter->push_back(c);
	      l_inter->push_back(*it1);
	    }
	}
    }
  //cout<<"l_inter_size ="<<l_inter->size()<<endl;
}

/*this function won't be here when we change the way of coding graphs (i.e. adding info on the edges)*/
/*it returns the (first) common compound of two reactions*/
/*!!!!! should use the list_intersect method that is in the Reaction_Graph.cpp file !!!!!!*/
Compound * common_compound(Reaction *rea1, Reaction *rea2)
{
  list<Compound *>::iterator it1=rea1->get_compounds()->begin();
  list<Compound *>::iterator it2=rea2->get_compounds()->begin();
  while (it1!=rea1->get_compounds()->end())
    {
      it2=rea2->get_compounds()->begin();
      while (it2!=rea2->get_compounds()->end())
	{
	  if ((*it1)==(*it2))
		     return *it1;
	  it2++;
	}
      it1++;
    }
  return NULL;
}

/*small function called by is_linear() to decide if a candidate reaction is valid for extension*/
/*2 criteria are needed : this reaction has not been visited before and none of the compounds have been visited already*/
bool extend(Reaction *rea1,Reaction *rea2,list<Compound *> *used_compounds,list<Reaction *> *used_reactions,list<Compound *> *l_inter,bool left)
{
  bool extend_ok=0;
  // cout<<"extend :considering reactions :";
//   rea1->print();cout<<" and ";rea2->print();cout<<endl;
  if(find(used_reactions->begin(),used_reactions->end(),rea2) == used_reactions->end())  //if the reaction has not been visited already
    {
      intersect2(rea1->get_compounds(), rea2->get_compounds(),l_inter);
      if (l_inter->size() != 0)
	{
	  list<Compound *> *l_inter2 = new list<Compound *>;
	  intersect2(l_inter,used_compounds,l_inter2);
	  extend_ok = (l_inter2->size() == 0);
	  // cout<<"intersect : les composs ne sont pas visits ? "<<extend_ok<<endl;
	  delete(l_inter2);
	}
      if (extend_ok)//if none of the common compounds have been visited already
	{
	  //mark common_compounds as used compounds
	  for (list<Compound *>::iterator it=l_inter->begin();it!=l_inter->end();it++) 
	    used_compounds->push_back(*it);
	  //mark new reaction as used reaction
	  if (left)
	    used_reactions->push_front(rea2);  
	  else
	    used_reactions->push_back(rea2);  
	  //return true
	  return 1;
	}
    }
  // cout<<"don't extend, try another reaction"<<endl;
  return 0; //in any other case don't extend with this reaction...try another one
}


/*THERE IS A BUG HERE, THIS FUNCTION USED TO WORK WHEN THE SOLUTIONS WERE ORDERED***/
/*to decide if a solution is linear, take a reaction extend it to the right using a different compound at each extension, once you can't extend anymore, go back to the first reaction and extend to the left
in the end, if you have not covered all reactions then the solution is not linear
Additional verification : you can verify that there's an unused compound at each end (those will be the input and the output) in case you have a circular solution ?*/
/*note that this function gives a sorted version of the list of reactions if the solution is linear*/
bool Solution::is_linear()
{
 //  this->print();
//   cout<<"  :";
  bool extension_ok=1;
  list<Compound *> *used_compounds=new list<Compound *>;
  list<Reaction *> *used_reactions=new list<Reaction *>;  //note that this list of reactions is not only useful to know which reactions have already been visited but also to maintain a sorted version of the solution
  list<Compound *> *l_inter = new list<Compound *>;
  list<Reaction *>::iterator it1=reactions->begin();
  list<Reaction *>::iterator it2;

  //list<Reaction *> *sorted_reactions=new list<Reaction *>; //new version of the list of reactions, sorted when the solution is linear

  used_reactions->push_back(*it1);
  
  /*extension to the right (1)*/
  while (extension_ok)
    {
      for (it2=reactions->begin();it2!=reactions->end();it2++)  //try to extend
	{
	  if (it1!=it2)
	    {
	      extension_ok=extend(*it1,*it2,used_compounds,used_reactions,l_inter,0);
	      if (extension_ok)
		break;
	    }
	}

      if (extension_ok)
	it1=it2; //move to next reaction
    }
  
  last_reaction=*it2;
  /*test end*/
  if (used_reactions->size()==reactions->size())
    {
      first_reaction=*reactions->begin();  //should also give info on outer compounds
      delete(used_compounds);
      delete(l_inter);
      delete(reactions);
      reactions=used_reactions;
      // cout<<"linear du premier coup"<<endl;
      return 1;
    }

  /*extension to the left (0)*/
  // cout<<"extension to the left"<<endl;
  extension_ok=1;
  it1=reactions->begin();
  while(extension_ok)
    {
      for (it2=reactions->begin();it2!=reactions->end();it2++)  //try to extend
	{
	  if (it1!=it2)
	    {
	      extension_ok=extend(*it1,*it2,used_compounds,used_reactions,l_inter,1);
	      if (extension_ok)
		break;
	    }
	}
      if (extension_ok)
	it1=it2; //move to next reaction
    }

/*test end*/
  if (used_reactions->size()==reactions->size())
    {
      first_reaction=*it2;
      delete(used_compounds);
      delete(l_inter);
      delete(reactions);
      reactions=used_reactions;
      //fill_inner_compounds(); //this line is temporary, in the end inner_compounds should be put in another structure, not specific to linear occurrences : the compound matrix
      // cout<<"linear ok"<<endl;
      return 1;
    }

  else
    {
      delete(used_compounds);
      delete(l_inter);
      delete(used_reactions);
      // cout<<"non linear"<<endl;
      return 0;
    }
}

/*returns true if one color is shared by the two reactions*/
bool common_color(Reaction *rea1,Reaction *rea2)
{
  for (list<int>::iterator it1=rea1->get_colours()->begin();it1!=rea1->get_colours()->end();it1++)
    {
      for (list<int>::iterator it2=rea2->get_colours()->begin();it2!=rea2->get_colours()->end();it2++)
	{
	  if (*it1==*it2) //if we find a common colour, then return true
	    return 1;
	}
    }
  
  return 0;  //if no common colour was found return false
}

/*tells you if the solution tested has its colors in the same order as this one*/
bool Solution::same_order(Solution *solution)
{
  list<Reaction *>::iterator it1=reactions->begin();
  list<Reaction *>::iterator it2=solution->get_reactions()->begin();
  bool order_ok=1;
  /*test direct order*/
  for (int i=0;i<(int)reactions->size();i++)
    {
      if (!common_color(*it1,*it2))
	{
	  order_ok=0;
	  break;
	}
      it1++;
      it2++;
    }

  /*if the order is ok , then return true, else, test the reverse order*/
  if (order_ok)
    return 1;
  
  it1=reactions->end() ,it2=solution->get_reactions()->begin();
  it1--;
  order_ok=1;
  /*test reverse order*/
  for (int i=0;i<(int)reactions->size();i++)
    {
      if (!common_color(*it1,*it2))
	{
	  order_ok=0;
	  break;
	}
      it1--;
      it2++;
    }
  return order_ok;
}

/*auxiliary function used in calculate_max_pathways*/
bool in_list(Pathway *pat, list<Pathway *> *pat_list)
{
  list<Pathway *>::iterator pat_iter = pat_list->begin();
  while (pat_iter != pat_list->end())
    {
      if (*pat_iter == pat)
	return true;
      pat_iter++;
    }
  return false;
}

void Solution::calculate_max_pathways()
{
  list<Reaction *>::iterator rea_iter;
  list<Pathway *>::iterator pat_iter;
  /*calculation of the maximum set of pathways*/
  rea_iter = reactions->begin();
  while (rea_iter != reactions->end())
    {
      pat_iter = (*rea_iter)->get_pathways()->begin();
      while (pat_iter != (*rea_iter)->get_pathways()->end())
	{
	  if (! in_list(*pat_iter, max_pathways))  //this is rather slow but very understandable
	    max_pathways->push_back(*pat_iter);
	  pat_iter++;
	}
      rea_iter++;
    }
}

/*to each reaction reaction corresponds a set of pathways, the problem here is to find the minimum set of pathways representing all reactions*/
/*this is quite difficult to do exhaustively*/
/*for the moment, the function calculates the intersection of all the sets of pathways*/
/*it is done in a way that is easily expandable to finding the minimum set of pathways representing all reactions*/
void Solution::calculate_min_pathways()
{
  list<Reaction *>::iterator rea_iter;
  list<Pathway *>::iterator pat_iter;
  list<Pathway *>::iterator pat_iter2;
  bool out=true;
  /*calculation of the minimum set of pathways*/
  pat_iter = max_pathways->begin();
  while (pat_iter != max_pathways->end())
    {
      out=true;
      rea_iter = reactions->begin();
      while (rea_iter != reactions->end())
	{
	  out = out & in_list(*pat_iter, (*rea_iter)->get_pathways());
	  rea_iter++;
	}
      if (out)
	{
	  //cout<<"found a common pathway"<<endl;
	  min_pathways->push_back(*pat_iter);
	}
      pat_iter++;
    }
}

bool Solution::is_intra_pathway()
{
  //cerr<<"is_intra_pat"<<min_pathways->size()<<endl;
  return (min_pathways->size()!=0);
}

int Solution::nb_max_pathways()
{
  return ((int)max_pathways->size());
}

void Solution::print_more()
{
  /*print reactions*/
  list<Reaction *>::iterator rea_iter = reactions->begin();
  while (rea_iter != reactions->end())
    {
      (*rea_iter)->print();
      cout <<" ";
      rea_iter++;
    }
  // cout<<endl<<" ";
    
  /*print pathways*/
  list<Pathway *>::iterator pat_iter;

  // pat_iter = max_pathways.begin();
//   cout<<"      maximum set of pathways :  ";
//   while (pat_iter != max_pathways.end())
//     {
//       (*pat_iter)->print();
//       cout <<"-";
//       pat_iter++;
//     }
//   cout<<endl<<" ";

  pat_iter = min_pathways->begin();
  // cout<<"      minimum set of pathways :  ";
  cout <<"\t";
  while (pat_iter != min_pathways->end())
    {
      (*pat_iter)->print();
      cout <<"\t";
      pat_iter++;
    }
}

void Solution::print_enzymes_pathways()
{
  /*print reactions*/
  list<Enzyme *>::iterator enz_iter;
  list<Reaction *>::iterator rea_iter = reactions->begin();
  while (rea_iter != reactions->end())
    {
      enz_iter=(*rea_iter)->get_enzymes()->begin();
      while (enz_iter!=(*rea_iter)->get_enzymes()->end())
	{
	  (*enz_iter)->print();
	  enz_iter++;
	  if (((*rea_iter)->get_enzymes()->size())>1 && enz_iter != (*rea_iter)->get_enzymes()->end())
	    cout<<"|";
	}
      // (*rea_iter)->print();
      cout <<" ";
      rea_iter++;
    }
  list<Pathway *>::iterator pat_iter;
  // pat_iter = max_pathways.begin();
//   cout<<"      maximum set of pathways :  ";
//   while (pat_iter != max_pathways.end())
//     {
//       (*pat_iter)->print();
//       cout <<"-";
//       pat_iter++;
//     }
//   //cout<<endl<<" ";

  pat_iter = min_pathways->begin();
  // cout<<"      minimum set of pathways :  ";
  cout <<"\t";
  while (pat_iter != min_pathways->end())
    {
      (*pat_iter)->print();
      cout <<"\t";
      pat_iter++;
    }
  cout <<"\t";
  pat_iter = max_pathways->begin();
  //cout<<"      maximum set of pathways :  ";
  while (pat_iter != max_pathways->end())
    {
      (*pat_iter)->print();
      cout <<"\t";
      pat_iter++;
    }
  //cout<<endl<<" ";
}


/*this is useful for stats, for motifs of size bigger than 2, it returns the mean arity*/

float Solution::cpd_arity()
{
  list<Reaction *>::iterator it1=reactions->begin();
  list<Reaction *>::iterator it2=it1++;
  
  int mean_arity=0;
  Compound *cpd=NULL;
  if (reactions->size()>1)
    {
      for (int i=0;i<(int)reactions->size()-1;i++)
	{
	  /*find the common compound (the compound that is on the edge of the reaction_graph*/
	  cpd = common_compound(*it2, *it1);
	  if (cpd!=NULL)
	    {
	      mean_arity+=cpd->arity();
	    }
	  else
	    {
	      cerr<<"while calculating mean compound arity"<<endl;
	      cerr<<"there is no common compound between those reactions, sorry"<<endl;
	      exit(0);
	    }
	  it1++;
	  it2++;
	}
    }
  else //for single reactions
    {
      cerr<<"this operation only works for motifs of size bigger than 2"<<endl;
      return 0.0;
      //exit(0);
    }
  return ((float)mean_arity/(float)(reactions->size()-1));
}


/*test if the compound is already in the compound_list*/
/*note that this function is the same as in Reaction.cpp*/
bool in_list4(Compound *compound_to_add, list<Compound *> *comp_list)
{
  list<Compound *>::iterator it = comp_list->begin();
  while (it != comp_list->end())
    {
      if (compound_to_add == *it)
	return true;
      it++;
    }
  return false;
}

/*gives the number of inner compounds in a solution*/
/*note that it calculates the list of inner compounds too, this could be used as an output*/
int Solution::nb_inner_compounds()
{
  list<Reaction *>::iterator it1=reactions->begin();
  list<Reaction *>::iterator it2=it1++;
  list<Compound *> *cpd_list = new list<Compound *>;
  Compound *cpd=NULL;
  if (reactions->size()>1)
    {
      for (int i=0;i<(int)reactions->size()-1;i++)
	{
	  /*find the common compound (the compound that is on the edge of the reaction_graph*/
	  cpd = common_compound(*it2, *it1);
	  if (cpd!=NULL)
	    {
	      if (!in_list4(cpd, cpd_list))
	     //  if (!in_list(cpd, &cpd_list))   //use the function that is in Reaction.cpp
		cpd_list->push_back(cpd);
	    }
	  else
	    {
	      cerr <<"while looking for linear solutions"<<endl;
	      cerr<<"there is no common compound between those reactions, sorry"<<endl;
	      exit(0);
	    }
	  it1++;
	  it2++;
	}
    }
  else //for single reactions
    {
      cerr<<"this operation only works for motifs of size bigger than 2"<<endl;
      delete(cpd_list);
      return 0;
      //exit(0);
    }
  int size=cpd_list->size();
  delete(cpd_list);
  return size;
}

// /*new version using the compound_matrix*/
// int Solution::nb_inner_compounds()
// {
//   int size=reactions->size();
//   int nb_multiple_compounds=0;
//   list<Compound *> *cpd_list = new list<Compound *>;
//   for (int i=0;i<size;i++)
//     {
//       for (int j=0;j<size;j++)
// 	{
// 	  if (compound_matrix[i][j]!=NULL)
// 	    copy(compound_matrix[i][j]->begin(),compound_matrix[i][j]->end(),back_inserter(*cpd_list))
// 	}
//     }
// }


/*weird stuff*/
void Solution::fill_inner_compounds()
{
  cout<<"fill_inner_compounds"<<endl;
  list<Reaction *>::iterator it1=reactions->begin();
  list<Reaction *>::iterator it2=it1++;
  list<Compound *> *l_inter;
  if (reactions->size()>1)
    {
      for (int i=0;i<(int)reactions->size()-1;i++)
	{
	  cout<<"boucle for"<<endl;
	  l_inter=new list<Compound *>;
	  intersect2((*it2)->get_compounds(), (*it1)->get_compounds(), l_inter);  //fills l_inter with the common compounds of the two considered reactions
	  cout<<"after intersect entre ";
	  (*it1)->print(); (*it2)->print();
	  cout<<endl;
	  cerr<<"l_inter size ="<<l_inter->size();
	  inner_compounds->push_back(l_inter);
	  cerr<<"aprs push_back()"<<endl;
	  it1++;
	  it2++;
	}
    }
   
  else //for single reactions
    {
      cerr<<"this operation only works for motifs of size bigger than 2"<<endl;
      cerr <<"while calculating inner_compounds"<<endl;
    }
}

/*new version to take into account non linear solutions (i.e. linear in the bipartite graph)*/
/*we should use a half matrix*/
void Solution::fill_compound_matrix()
{
  list<Compound *> *l_inter=new list<Compound *>;
  int i=0;
  int j=0;
  if (reactions->size()>1)
    {
      for (list<Reaction *>::iterator it1=reactions->begin();it1!=reactions->end();it1++)
	{
	  j=0;
	  for (list<Reaction *>::iterator it2=reactions->begin();it2!=reactions->end();it2++)
	    {
	      if (it1!=it2) //doesn't calculate the common compounds intra reaction (useless)
		{
		  intersect2((*it2)->get_compounds(), (*it1)->get_compounds(), l_inter);  //fills l_inter with the common compounds of the two considered reactions
		  if (l_inter->size()!=0) //when the intersection is not null, then allocate memory and fill compound_matrix
		    {
		      compound_matrix[i][j]=new list<Compound *>;
		      copy(l_inter->begin(),l_inter->end(),back_inserter(*compound_matrix[i][j]));
		    }
		}
	      j++;
	    }
	  i++;
	}      
    }
   
  else //for single reactions
    {
      // cerr<<"this operation only works for motifs of size bigger than 2"<<endl;
//       cerr <<"while calculating inner_compounds"<<endl;
    }
  delete(l_inter);
}


void Solution::print()
{
  /*print reactions*/
  list<Reaction *>::iterator iter = reactions->begin();
  while (iter != reactions->end())
    {
      (*iter)->print();
      cout <<" ";
      iter++;
    }
}




/**
 * Recovers the topology of the occurrence in the bipartite_graph.
 *
 * @param whole_graph bipartite graph corresponding to the whole network
 *
 */
void Solution::set_bipartite_graph(Bipartite_Graph * whole_graph, int *colors_to_filter)
{
  bipartite_graph = new Bipartite_Graph(whole_graph, rea_tab, size, colors_to_filter);
}


// void Solution::set_colour_order()
// {

// }

// /**
//  * Computes the canonical labelling of the bipartite graph. This
//  * operation is done by nauty (No AUTomorphism, Yes ?). The result is
//  * stored as two hexadecimal numbers. In case the bipartite graph has
//  * not been extracted, this operation is not possible.
//  *
//  */
// void Solution::compute_canonical_labelling()
// {
//   /*In case the bipartite graph has not been extracted, this operation is not possible.*/
//   if (bipartite_graph == NULL)
//     {
//       cerr<<"you are trying to compute the canonical labelling of a solution that has no corresponding bipartite graph"<<endl;
//       exit(0);
//     }


//   bipartite_graph->set_canonical_labelling();

// }



/**
 * Computes the canonical labelling of the bipartite graph. This
 * operation is done by nauty (No AUTomorphism, Yes ?). The result is
 * stored as two hexadecimal numbers. In case the bipartite graph has
 * not been extracted, this operation is not possible.
 *
 */
void Solution::compute_canonical_labelling(bool order)
{

  /*In case the bipartite graph has not been extracted, this operation is not possible.*/
  if (bipartite_graph == NULL)
    {
      cerr<<"you are trying to compute the canonical labelling of a solution that has no corresponding bipartite graph"<<endl;
      exit(0);
    }


  /*write dreadnaut query file*/
  std::ofstream output;
  // char *file_name=new char[MAX];
//   // file_name[0]='\0';
//   strcpy(file_name,write_path);
//   file_name=strcat(file_name,"/hop");

  output.open("query.dre");
  

  if (!order)
    bipartite_graph->print_dreadnaut(output);
  
  else
    bipartite_graph->print_dreadnaut_ordered(output);

  /*tell nauty to compute a canonical labelling*/
  output<<"c"<<endl;
  /*do not print automorphisms and markers*/
  output<<"-a -m";
  /*execute nauty (compute automorphisms and canonical labelling)*/
  output<<" x";
  /*print two digit hex numbers that depend only on the graph with canonical labelling*/
  output<<" z"<<endl;
  output.close();


  /*execute dreadnaut*/
  int error = system("../nauty24b7/dreadnaut < query.dre > dreadnaut.res");
  if (error == -1)
    {
      cerr<<"error while executing dreadnaut - most probably dreadnaut is not installed in your computer - you can get it from http://cs.anu.edu.au/~bdm/nauty/ install it on your computer and then copy the exec file 'dreadnaut' in the Motus bin directory."<<endl;
      exit(0);
    }

  /*parse the output of dreadnaut and store it in long ints*/
  char str[200];
  char word[8];
  std::ifstream file_op("dreadnaut.res");
  /*ignore the first 3 lines and consider only the 4th*/
  for (int i=0;i<4;i++)
    file_op.getline(str,200);

  // cerr<<"str "<<str<<endl;

  /*parsing the string containing the 2 hexadecimal numbers*/
  //  cout<<"the string i am about to parse"<<str<<endl;
  
  /*skip the first [ and the blanks*/
  int i=1; 
  while (str[i]==' ')
    i++;

  /*first word to extract: useless prefix ?*/
  extract_word(str, word, ' ',&i);
  
  //cerr<<"word1 "<<word<<endl;

  /*skip occasional blanks (when the first hex number contains only 6 digits for example*/
  while (str[i]==' ')
    i++;

  /*second word to extract: first hex number characterising the
    topology of the graph*/
  extract_word(str,word, ' ',&i);
  //cerr<<"word2 "<<word<<endl;
  topo1=strtol(word,(char**)NULL,16) ;

  /*skip occasional blanks (when the second hex number contains only 6 digits for example*/
  while (str[i]==' ')
    i++;

  /*third word to extract: second hex number characterising the
  topology of the graph*/
  extract_word(str,word, ']',&i);
  //cerr<<"word3 "<<word<<endl;
  topo2=strtol(word,(char**)NULL,16) ;

  //cout<<"topo1 :"<<topo1<<endl;
  //cout<<"topo2 :"<<topo2<<endl;
 
  file_op.close();
  //exit(0);

  error = system("rm query.dre dreadnaut.res");
  if (error!=0)
    {
      cerr <<"Error while deleting files query.dre and dreadnaut.res"<<endl;
      exit(0);
    }
}


/**
 * Compares the topologies of two solutions. If the codes
 * characterising their canonical labellings are both identical (it
 * means that the graphs are isomorphic), then the method returns
 * true.
 *
 * @param sol_to_compare Solution to compare to this solution
 * @return boolean set to true if solutions have the same topology
 *
 */
bool Solution::same_topology(Solution *sol_to_compare)
{
  return (sol_to_compare->get_topo1() == topo1 && sol_to_compare->get_topo2() == topo2);
}


/*print the solution in a file*/
void Solution::print(std::ostream& output)
{
  list<Reaction *>::iterator iter = reactions->begin();
  while (iter != reactions->end())
    {
      (*iter)->print(output);
      output <<"\t";
      iter++;
    }
}

/**
 *Print the solution to a file. Depending on the verbose parameter, several levels of detail can be printed.
 *
 * @param output
 * @param verbose : level of detail for each reaction (0: only reaction IDs 1: reaction IDs + EC numbers + pathways when the solution is totally included in one of them)
 * 
 */
void Solution::print(std::ostream& output, int verbose)
{
  list<Reaction *>::iterator iter = reactions->begin();
  while (iter != reactions->end())
    {
      (*iter)->print(output,verbose);
      output <<"\t";
      iter++;
    }

  if (verbose>0)
    {
      /*print pathways*/
      list<Pathway *>::iterator pat_iter;
      pat_iter = min_pathways->begin();
      while (pat_iter != min_pathways->end())
	{
	  output <<"\t";
	  (*pat_iter)->print(output);
	  pat_iter++;
	}

    }

}

/*depending on the verbose parameter, we print the Reaction ids, and the EC numbers and the inner compounds*/
void Solution::print(int verbose)
{
  list<Reaction *>::iterator rea_iter = reactions->begin();

  /*we don't print the compounds*/
  if (verbose<2)
    {
      while (rea_iter != reactions->end())
	{
	  (*rea_iter)->print(verbose);
	  cout <<" ";
	  rea_iter++;
	}
    }

  /*we print the reactions and then the compound matrix*/
  if (verbose==2)
    {
      /*print reactions*/
      while (rea_iter != reactions->end())
	{
	  (*rea_iter)->print(verbose);
	  cout <<" ";
	  rea_iter++;
	}
      cout<<endl;


      /*print compound matrix*/
      for (int i=0;i<(int)reactions->size();i++)
	{
	for (int j=0;j<(int)reactions->size();j++)
	  {
	    if (compound_matrix[i][j]!=NULL)
	      for (list<Compound *>::iterator it=compound_matrix[i][j]->begin(); it != compound_matrix[i][j]->end(); it++)
		{
		  (*it)->print();
		  cout<<" ";
		}
	    else
	      cout<<"0 ";
	  }
	cout<<endl;
	}
    }


  /*print pathways*/
  list<Pathway *>::iterator pat_iter;
  pat_iter = min_pathways->begin();
  //  cout <<"\t";
  while (pat_iter != min_pathways->end())
    {
      cout <<"\t";
      (*pat_iter)->print();
      pat_iter++;
    }
}


/**
 *Print the bipartite graph to standard output. The format used is a simple matrix of 1's and 2's.
 *
 */
void Solution::print_bipartite_graph()
{
  if (bipartite_graph!=NULL)
    bipartite_graph->print();

  else 
    cerr<<"cannot print the bipartite graph for this solution, it has not been calculated !"<<endl;
}

/**
 *Print the order of colours to standard output. 
 *
 */
void Solution::print_colour_order()
{
  if (bipartite_graph!=NULL)
    bipartite_graph->print_colour_order();

  else 
    cerr<<"cannot print the bipartite graph for this solution, it has not been calculated !"<<endl;
}
