/* Solution_Group.cpp
 *
 * Copyright (C) 2007 Laboratoire de Biomtrie et Biologie volutive
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../include/Solution_Group.h"
#include <iostream>


Solution_Group::Solution_Group()
{
  solutions = new list<Solution *>;
}


Solution_Group::Solution_Group(Solution *solution)
{
  solutions = new list<Solution *>;
  solutions->push_back(solution);
}

/**
 *Destructor of Solution_Group : Note that when you delete the group,
 *you don't delete the solutions that belong to this group. This is
 *useful when you deal with groups of solutions that contain solutions
 *shared among several groups.
 */
Solution_Group::~Solution_Group()
{
  // for (list<Solution *>::iterator it=solutions->begin();it!=solutions->end();it++)
//     delete(*it);
  delete(solutions);
}

/**
 *Delete the solutions contained in the group. This method is not
 *included in the destructor or Solution_Group since in some cases, we
 *are dealing with solutions that are shared among several groups
 *without being copied.
 */
void Solution_Group::delete_solutions()
{
  for (list<Solution *>::iterator it=solutions->begin();it!=solutions->end();it++)
    delete(*it);
  solutions->erase(solutions->begin(), solutions->end());
}


void Solution_Group::clear_solutions()
{
  solutions->clear();
}

list<Solution *> * Solution_Group::get_solutions()
{
  return solutions;
}

void Solution_Group::add(Solution *solution)
{
  solutions->push_back(solution);
}


int Solution_Group::size()
{
  int out = solutions->size();
  return out;
  // return (int)solutions->size();
}

long int Solution_Group::get_topo(int order)
{
 //  long int topo1=solutions->front()->get_topo1();
//   long int topo2=solutions->front()->get_topo2();

//   return topo1+topo2;
  // return (int)solutions->size();
  return solutions->front()->get_topo(order);

}


/**
 * Recovers the topology of the occurrences in the bipartite_graph.
 *
 * @param whole_graph bipartite graph corresponding to the whole network
 *
 */
void Solution_Group::set_bipartite_graph(Bipartite_Graph *whole_graph, int *colors_to_filter)
{
  for (list<Solution *>::iterator it=solutions->begin(); it!=solutions->end(); it++)
    (*it)->set_bipartite_graph(whole_graph,colors_to_filter);
}


/**
 *  Computes the canonical labelling of the bipartite graph of the
 * solutions. This operation is done by nauty (No AUTomorphism, Yes
 * ?). The result is stored as two hexadecimal numbers.
 *
 */
void Solution_Group::compute_canonical_labelling(bool order)
{
  for (list<Solution *>::iterator it=solutions->begin(); it!=solutions->end(); it++)
    (*it)->compute_canonical_labelling(order);
}


/*useful for stats*/
int Solution_Group::nb_intra_pathway()
{
  int out=0;
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      //cerr<<"nb_intra_pat ::xolutions_group"<<endl;
      out+=(*sol_iter)->is_intra_pathway();
      //cerr<<"nb_intra_pat ::xolutions_group, aprs"<<endl;
      sol_iter++;
    }
  return out;
}


/**
 *Calculate the mean number of pathways per solution. The solution is rounded with two figures after the coma.
 *
 * @return mean mean number of pathways per solution
  */
float Solution_Group::mean_nb_pathway_per_occurrence()
{
  int out=0; //sum of number of pathways
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      out+=(*sol_iter)->nb_max_pathways();
      sol_iter++;
    }
  float result = (float)out/(float)solutions->size();
  float approx_result = (float)((int)(100*result))/(float)100;
  //return (float)out/(float)solutions->size();
  return approx_result;
}



/**
 *Calculate the variance of the number of pathways per solution. The solution is rounded with two figures after the coma.
 *
 * @return variance variance of the number of pathways per solution
  */
float Solution_Group::var_nb_pathway_per_occurrence()
{
  int out=0; //sum of squares
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      out+=(*sol_iter)->nb_max_pathways() * (*sol_iter)->nb_max_pathways();
      sol_iter++;
    }
  // float variance=0.0;
//   variance = (float)out/(float)solutions->size() - this->mean_nb_pathway_per_occurrence() * this->mean_nb_pathway_per_occurrence();
//   return variance;

  float result = (float)out/(float)solutions->size() - this->mean_nb_pathway_per_occurrence() * this->mean_nb_pathway_per_occurrence();
  float approx_result = (float)((int)(100*result))/(float)100;
  //return (float)out/(float)solutions->size();
  return approx_result;
}


/*useful for stats*/
float Solution_Group::cpd_arity()
{
  float out=0.0;
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      //cout<<"solution_group cpd arity :"<<out<<endl;
      out+=(*sol_iter)->cpd_arity();
      // cerr<<"out = "<<out<<endl;
      sol_iter++;
    }
  // cerr<<"next group"<<endl;
  return out;
}

/*useful for stats*/
/*useful to calculate the mean arity*/
int Solution_Group::nb_max_sol(Metabolic_Motif *motif)
{
  int out=0;
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      //cout<<"solution_group cpd arity :"<<out<<endl;
      //out+=((*sol_iter)->size() == motif->size());
      out+=((*sol_iter)->get_size() > 1);
      sol_iter++;
    }
  return out;
}

//returns true if the solutions concern the same set of reactions (i.e. permutation of each other)
bool is_permutation(Solution *sol1, Solution *sol2)
{
  /*they have to have the same number of reactions in order to have the same set*/
  //if (iter1->size() != iter2->size())
  if (sol1->get_size() != sol2->get_size())
    return 0;

  bool out=0;
  list<Reaction *>::iterator rea_iter1 = sol1->get_reactions()->begin();
  list<Reaction *>::iterator rea_iter2 = sol2->get_reactions()->begin();
  while (rea_iter1 != sol1->get_reactions()->end())
    {
      out=0;
      rea_iter2 = sol2->get_reactions()->begin();
      while (rea_iter2 != sol2->get_reactions()->end() && !out)
	{
	  if ((*rea_iter2) == (*rea_iter1))
	  out=1;
	  rea_iter2++;
	}
      if (!out)
	{return 0;}
      rea_iter1++;
    }
  return 1;
}

bool common_reaction(Solution *sol1, Solution *sol2)
{
  list<Reaction *>::iterator rea_iter1 = sol1->get_reactions()->begin();
  list<Reaction *>::iterator rea_iter2 = sol2->get_reactions()->begin();
  while (rea_iter1 != sol1->get_reactions()->end())
    {
      rea_iter2 = sol2->get_reactions()->begin();
      while (rea_iter2 != sol2->get_reactions()->end())
	{
	  /*as soon as we found a common reaction, we return true*/
	  if ((*rea_iter2) == (*rea_iter1))
	    return true;
	  rea_iter2++;
	}
      rea_iter1++;
    }
  return false;
}

/*withdraws the solutions that are permutations of each other, this could be done while enumerating solutions*/
// void Solution_Group::withdraw_redundance()
// {
//   list< Solution *> *unique_solutions = new list<Solution *>;
//   bool ok=0;
//   list< Solution *> ::iterator iter1 = solutions->begin();
//   list< Solution *> ::iterator iter2 = unique_solutions->begin();
//   while (iter1 != solutions->end())
//     {
//       //cout <<"je suis en dbut de while1"<<endl;
//       ok=0;
//       iter2 = unique_solutions->begin();
//       while (iter2 != unique_solutions->end() && !ok)
// 	{
// 	  //cout <<"je suis en dbut de while2"<<endl;
// 	  if (is_permutation(*iter1,*iter2))
// 	    {
// 	      ok=1;
// 	      //cout<<"je sius dans le cas d'une permutation"<<endl;
// 	    }
// 	  //cout<<"je vais itrer iter2"<<endl;
// 	  iter2++;
// 	}
//       if (!ok)
// 	{
// 	  //cout <<"je rajoute une nouvelle solution dans unique"<<endl;
// 	  unique_solutions->push_back(*iter1);
// 	}
//       else 
// 	{
// 	  //this solution should be deleted
// 	  delete(*iter1);  //seems to work
// 	}
//       //cout <<"je vais itrer iter1"<<endl;
//       iter1++;
//       //cout<<"j'ai itr iter1"<<endl;
//       }

//     /*transfer of unique_solutions to solutions : this could be simplified easily*/
//     solutions->clear();   //hope no one wants to keep that, it would be nice to erase the solutions that are counted several times
//     iter2 = unique_solutions->begin();
//     while (iter2 != unique_solutions->end())
//       {      
// 	solutions->push_back(*iter2);
// 	iter2++;
//       }
//     delete(unique_solutions);
// }

/*performs the fusion of two groups by adding one to the other*/
void Solution_Group::add_group(Solution_Group *group_to_add)
{
  list<Solution *>::iterator sol_iter = group_to_add->get_solutions()->begin();
  while (sol_iter != group_to_add->get_solutions()->end())
    {
      solutions->push_back(*sol_iter);
      sol_iter++;
    }
  group_to_add->clear_solutions();  //note that we don't delete the solutions
  // cout<<"avant de deleter mon group_to _add"<<endl;
  //delete(group_to_add);
//   cout <<"aprs de deleter mon groupe to add"<<endl;
}

/*performs the fusion of two groups by adding one to the other, the main difference with the previous method is that we do not lose the previous group*/
void Solution_Group::copy_group(Solution_Group *group_to_copy)
{
  list<Solution *>::iterator sol_iter = group_to_copy->get_solutions()->begin();
  while (sol_iter != group_to_copy->get_solutions()->end())
    {
      // Solution *new_sol=*sol_iter;
      // solutions->push_back(new_sol);
      solutions->push_back(*sol_iter);
      sol_iter++;
    }
  // group_to_add->clear_solutions();  //note that we don't delete the solutions
}



bool Solution_Group::contains_similar_solution(Solution *solution)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      if (common_reaction(solution, *sol_iter))
	return true;
      sol_iter++;
    }
  return false;
}

/*creates a partition of the solutions, every two solutions that share a reaction are placed in the same group*/
void Solution_Group::group_by_reaction(list<Solution_Group *> *groups)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  list<Solution_Group *>::iterator group_iter;
  bool first_match=0;
  Solution_Group *first_matched_group=NULL;
  //for all solutions
  while (sol_iter != solutions->end())
    {
      first_match=0;
      //for all groups
      group_iter = groups->begin();
      while (group_iter != groups->end())
	{
	  //1er match, on stocke et les suivants on fusionne
	  //no match on cre un groupe
	  if ((*group_iter)->contains_similar_solution(*sol_iter))
		{
		  //cout<<"cas ou on a trouv une solution similaire"<<endl;
		  if (!first_match)
		    {
		      first_match=true;
		      (*group_iter)->add(*sol_iter);
		      first_matched_group = *group_iter;
		    }
		  else
		    {
		      first_matched_group->add_group(*group_iter);
		      //the erase method doesn't delete the object !
		      //we could add this line : 
		      //group_iter->delete_solutions();
		      //(*group_iter)->clear_solutions();
		      //cout<<"je delete un groupe"<<endl;
		      delete(*group_iter);  //we don't delete the solutions inside the group, we just delete the group
		      group_iter=groups->erase(group_iter);
		      //cout<<"peut-tre cela va-t-il foirer !"<<endl;
		      group_iter--;
		    }
		}
	  group_iter++;
	}
      //the solution shares no reaction with any other
      if (!first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  Solution_Group *new_group = new Solution_Group(*sol_iter);
	  groups->push_back(new_group);
	}
      sol_iter++;
    }
}

/*creates a partition of the solutions, every two solutions that share the same order are placed in the same group*/
/*note that in some particular cases, a solution can be placed in several groups : Ex : 0/1 2 3 0/1 has the same order as 0 2 3 1 and 1 2 3 0*/
void Solution_Group::group_by_order(list<Solution_Group *> *groups)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  list<Solution_Group *>::iterator group_iter;
  bool first_match=1;
  //for all solutions
  while (sol_iter != solutions->end())
    {
      //for all groups
      first_match=1;
      group_iter = groups->begin();
      while (group_iter != groups->end())
	{
	  //1er match, on stocke 
	  //no match on cre un groupe
	  if ((*group_iter)->get_solutions()->front()->same_order(*sol_iter))
		{
		  first_match=0;
		  (*group_iter)->add(*sol_iter);
		  break;
		}
	  group_iter++;
	}
      //the order has not been found before
      if (first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  Solution_Group *new_group = new Solution_Group(*sol_iter);
	  groups->push_back(new_group);
	}
      sol_iter++;
    }
}


/**
 *Creates a partition of solutions according to their topology in the
 *bipartite graph. Two solutions sharing the same topology are put in
 *the same group.
 *
 * @param groups resulting partition
 */
void Solution_Group::group_by_topology(list<Solution_Group *> *groups)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  list<Solution_Group *>::iterator group_iter;
  bool first_match=1;
  //for all solutions
  while (sol_iter != solutions->end())
    {
      //for all groups
      first_match=1;
      group_iter = groups->begin();
      while (group_iter != groups->end())
	{
	  //1er match, on stocke 
	  //no match on cre un groupe
	  if ((*group_iter)->get_solutions()->front()->same_topology(*sol_iter))
		{
		  first_match=0;
		  (*group_iter)->add(*sol_iter);
		  break;
		}
	  group_iter++;
	}
      //the order has not been found before
      if (first_match)
	{
	  //cout<<"je cre un groupe"<<endl;
	  Solution_Group *new_group = new Solution_Group(*sol_iter);
	  groups->push_back(new_group);
	}
      sol_iter++;
    }
}


/**
 * Counts the number of times a topology occurs in the list of
 * solutions. The list of topologies and their identifiers is given as
 * input to the function. Note that if, in this set of solutions, a
 * new topology is identified, it will not be reported. Only the
 * topologies that have been observed in the real graph are considered
 * (this means that p-values are not calculated for motifs that are
 * not observed).
 *
 * @param observed_topologies1 first hex number characterising each of these topologies
 * @param observed_topologies2 second hex number characterising each of these topologies
 * @param nb_observed_topologies number of distinct topologies observed among the occurrences of the motif in the real graph
 * @param result counts of the number of same topologies in this set of solutions.
 */
void Solution_Group::count_observed_topologies(long int *observed_topologies1, long int *observed_topologies2, int nb_observed_topologies, int *result)
{
  for (list<Solution *>::iterator iter=solutions->begin(); iter!=solutions->end(); iter++)
    {
      int j=0;
      bool topology_found=0;
      /*one can arrive at the end of the loop both in the case where
	the topology has been found or not found*/
      while (j<nb_observed_topologies && !topology_found)
	{
	     //for (int j=0;j<nb_observed_topologies;j++)
	  if (observed_topologies1[j] == (*iter)->get_topo1() && observed_topologies2[j] == (*iter)->get_topo2())
	    {
	      result[j]++;
	      topology_found=true;
	    }    
	  j++;
	}
    }
}


void Solution_Group::filter_by_compounds(Compound *input, Compound *output, int threshold)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      if (!(*sol_iter)->contains_similar_compounds(input, output, threshold))
	{
	  //cout<<"j'erase une solution"<<endl;
	  delete(*sol_iter);
	  sol_iter=solutions->erase(sol_iter);
	  sol_iter--;
	}
      else
	{
	  //cout<<"celle-l, je la garde"<<endl;
	}
      sol_iter++;
    }
}

/*withdraws (but not delete) the non_linear_solutions*/
void Solution_Group::withdraw_non_linear_solutions()
{
  // cout<<"je suis dans withdrawnonlinearsolutions"<<endl;
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      if (!(*sol_iter)->is_linear())
	{
	  //cout<<"j'erase une solution"<<endl;
	  //delete(*sol_iter);
	  sol_iter=solutions->erase(sol_iter);
	  sol_iter--;
	}
      else
	{
	  //cout<<"celle-l, je la garde"<<endl;
	}
      sol_iter++;
    }
}

/*old version : doesn't work properly*/
/*withdraws the non_linear_solutions*/
void Solution_Group::filter_by_nb_compounds(int motif_size)
{
  list<Solution *>::iterator sol_iter = solutions->begin();
  while (sol_iter != solutions->end())
    {
      if ((*sol_iter)->nb_inner_compounds() < (motif_size -1) )
	{
	  //cout<<"j'erase une solution"<<endl;
	  delete(*sol_iter);
	  sol_iter=solutions->erase(sol_iter);
	  sol_iter--;
	}
      else
	{
	  //cout<<"celle-l, je la garde"<<endl;
	}
      sol_iter++;
    }
}


void Solution_Group::print(int verbose)
{
  list<Solution *>::iterator iter=solutions->begin();
  while (iter!=solutions->end())
    {
      (*iter)->print(verbose);
      iter++;
      cout<<endl;    
    }
  cout <<"this group contains "<<solutions->size()<<" solutions"<<endl;
}


/*print the solution_group in a file*/
void Solution_Group::print(std::ostream& output)
{
  for (list<Solution *>::iterator iter=solutions->begin();iter!=solutions->end();iter++)
    {
      (*iter)->print(output);
      output<<endl;
    }
}

/*print the solution_group in a file keeping only the solutions that are intra-pathway*/
void Solution_Group::print_intra_pathway(std::ostream& output)
{
  for (list<Solution *>::iterator iter=solutions->begin();iter!=solutions->end();iter++)
    {
      if ((*iter)->is_intra_pathway())
	{
	  (*iter)->print(output);
	  output<<endl;
	}
    }
}



/**
 *Print the solution_group to a file. Depending on the verbose parameter, several levels of detail can be printed.
 *
 * @param output : any output
 * @param verbose : level of detail for each reaction (0: only reaction IDs 1: reaction IDs + EC numbers)
 * 
 */
void Solution_Group::print(std::ostream& output, int verbose)
{
  for (list<Solution *>::iterator iter=solutions->begin();iter!=solutions->end();iter++)
    {
      (*iter)->print(output,verbose);
      output<<endl;
    }
}


void Solution_Group::print_bipartite_graphs()
{
  for (list<Solution *>::iterator iter=solutions->begin();iter!=solutions->end();iter++)
    {
      (*iter)->print_bipartite_graph();
      cout<<endl;
    }
}

void Solution_Group::print_bipartite_graph()
{
  list<Solution *>::iterator iter=solutions->begin();
  (*iter)->print_bipartite_graph();
 //  cout<<endl;
}

void Solution_Group::print_colour_order()
{
  list<Solution *>::iterator iter=solutions->begin();
  (*iter)->print_colour_order();
 //  cout<<endl;
}


void Solution_Group::print()
{
  list<Solution *>::iterator iter=solutions->begin();
  while (iter!=solutions->end())
    {
      //(*iter)->print_enzymes_pathways();
      (*iter)->print();
      iter++;
      cout<<endl;    
    }
  cout <<"this group contains "<<solutions->size()<<" solutions"<<endl;
}
